package com.monica.interfaces;

import com.monica.interfaces.*;

/** General purpose cipher algorithm. Provides functionality for encryption/decryption.
    <b>Using the same instance of classes implementing this interface for both encyption and
    decyption may not be supported.</b>
*/

public interface CipherAlgorithm extends Configureable {

  /** Returns the internal name of the of the CipherAlgorithm.
      @return String: the internal name of the CipherAlgorithm
  */
  public String getCipherAlgorithmName();
  
  /** Returns the internal version of the CipherAlgorithm.
      @return String: the internal version of the CipherAlgorithm
  */
  public String getCipherAlgorithmVersion();
  
  /** Method for setting the cryptographic key and the initialization vector.
      @param String key: the key for the cryptography
      @param string IV: the initialization vector
  */
  public void setKeys(String Key, String IV); 
  
  /** Returns the state of the cipher mechanism. This state can be restored with the setState
      method.
      @return Object: the state of the cipher mechanism
  */  
  public Object getState();
  
  /** Sets the state of the cipher mechanism.
      @param Object o: the to be set state of the cipher mechanism 
  */
  public void setState(Object state);

  /** Method for encryption of a String. The to be encrypted String has to be of a length
      that is a multipy of getCipherBlockSize(). Encryption is done with the key set with
      setKeys(String, String).
      @param String s: the String to be encrypted
      @return String: the encrypted String
  */
  public String encode(String data);
  
  /** Method for decryption of a String. The to be decrypted String has to be of a length
      that is a multiply of getCipherBlockSize(). Decryption is done with the key set with
      setKeys(String, String).
      @param String s: the String to be decrypted
      @return String: the decrypted String
  */
  public String decode(String coded_data);

  /** Returns the length of the minimal encryption length. Strings for encryption/decryption
      have to be of a length that is a multiply of this.
      @return int: the minimal encryption length
  */      
  public int getCipherBlockSize();
}

