package com.monica.interfaces;

import com.monica.interfaces.*;

/** General purpose compression algorithm. Provides functionality for compression/decompression. 
    <b>Using the same instance of a class implementing this interface for both compression
    and decompression may not be supported.</b>
*/    
public interface CompressionAlgorithm extends Configureable {

  /** Returns the internal name of the CompressionAlgorithm.
      @return String: the internal name of the CompressionAlgorithm
  */    
  public String getCompressionAlgorithmName();
  
  /** Returns the internal version of the CompressionAlgorithm.
      @return String: the internal version of the CompressionAlgorithm.
  */      
  public String getCompressionAlgorithmVersion();

  /** Compresses the given String.
      @param String s: the to be compressed String
      @return String: the compressed String
  */      
  public String compress(String data);
  
  /** Decompresses the given String.
      @param String s: the to be decompressed String
      @return String: the decompressed String
  */      
  public String decompress(String compressed_data);
}