package com.monica.interfaces;

import java.math.*;

/** General purpose hash algorithm. Provides functionality for hashing Strings.
*/

public interface HashAlgorithm extends Configureable {

  /** Returns the internal name of the HashAlgorithm.
      @return String: the internal name of the HashAlgorithm
  */      
  public String getHashAlgorithmName();
  
  /** Returns the internal version of the HashAlgorithm.
      @return String: the internal version of the HashAlgorithm
  */      
  public String getHashAlgorithmVersion();

  /** Returns the hash value of the given String.
      @param String s: the String to be hashed
      @return String: the hash value of the given String
  */      
  public String hash(String s);
  
  /** Returns the hash value of the given String as BigInteger. Converting 
      the hash value from String to BigInteger is done as follows: the String
      hash value is interpreted as a nonnegative integer number with s.charAt(0)
      as MSB and s.charAt(s.length()-1) as LSB.
      @param String s: the String to be hashed
      @return BigInteger: the hash value of the given String
  */      
  public BigInteger hashAsBigInteger(String s);
  
  /** Returns the length of the hash value (as String) in number of bytes.
      @return int: the length of the hash value
  */      
  public int getHashLength();
}
