package com.monica.interfaces;

import com.monica.interfaces.*;

/** Provides general purpose Message Authentication functionality.
*/
public interface MACAlgorithm extends Configureable {

  /** Returns the internal name of the MACAlgorithm.
      @return String: the internal name of the MACAlgorithm
  */      
  public String getMACAlgorithmName();
  
  /** Returns the internal version of the MACAlgorithm.
      @return String: the internal version of the MACAlgorithm
  */      
  public String getMACAlgorithmVersion();

  /** Creates the Message Authentication Code for the provided String (data) 
      according to the provided shared secret String (secret).
      @param String data: the data for which the MAC has to be created
      @param String secret: the shared secret for the MAC creation
      @return String: the resulting MAC
  */            
  public String createMAC(String data, String secret);
  
  /** Authenticates a message String (data) with a specified MAC String (MAC) accoring
      to the specified shared secret String (secret). Returns true if Message
      Authentication was successful.
      @param String data: the message body
      @param String MAC: the Message Authentication Code for the message body
      @param String secret: the shared secret for authenticating
      @return boolean: wether the authentication was successful or not
  */      
  public boolean authenticate(String data, String MAC, String secret);

  /** Return the length of the MAC in bytes.
      @return int: the length of the MAC in bytes
  */      
  public int getMACLength();
}