package com.monica.interfaces;

import com.monica.interfaces.*;

/** Implements the source part of the read functionality in the
    TM layer hierarchy. Readable is always the source of data for
    a Reader. Reading in the TM layer hierarchy is asynchronous.
    If there is data available from the Readable, the readEvent of
    the Reader is called. Which then can read through the getInput
    method of the Readable. Data during the time between the readEvent 
    and getInput will be buffered.
*/    
public interface Readable {

  /** Returns the internal name of the Readable.
      @return String: the internal name of the Readable
  */      
  public String getReadableName();
  
  /** Returns the internal version of the Readable.
      @return String: the internal version of the Readable
  */      
  public String getReadableVersion();
  
  /** Returns the number of bytes that are available for read out from
      this Readable.
      @return int: the number of bytes available for read out
  */      
  public int getAvailable();
  
  /** Returns the contents of the read buffer from this Readable.
      @return String: the contents of the read buffer
  */      
  public String getInput();
  
  /** Returns wether the Readable is still connected to its source.
      @return boolean: wether the Readable is still connected to its 
        source
  */	
  public boolean isReadableConnected();
  
  /** Sets the Reader to which the readEvents have to be sent.
      @param Reader reader: the Reader to which the readEvents
        have to be sent
  */	
  public void setReader(Reader reader);
  
  /** Returns the Reader to which readEvents will be sent.
      @return Reader: the Reader to which readEvents will be sent
  */      
  public Reader getReader();
  
}    