package com.monica.interfaces;

import com.monica.interfaces.*;

/** Implements the destination part of the read functionality in the
    TM layer hierarchy. Reader is always the destination of data from
    a Readable. Reading in the TM layer hierarchy is asynchronous.
    If there is data available from the Readable, the readEvent of
    the Reader is called. Which then can read through the getInput
    method of the Readable. Data during the time between the readEvent 
    and getInput will be buffered in the Readable.
*/    
public interface Reader {

  /** Returns the internal name of the Reader.
      @return String: the internal name of the Reader
  */      
  public String getReaderName();
  
  /** Returns the internal version of the Reader.
      @return String: the internal version of the Reader
  */      
  public String getReaderVersion();
  
  /** Indicates that there is data available from the specified  
      Readable.
      @param Readable readable: there is data available from the
        readable
  */	
  public void readEvent(Readable readable);
  
  /** Indicates that the source connection of the specified
      Readable has been closed.
      @param Readable readable: the Readable whose source connection
        has been closed
  */	
  public void connectionClosedEvent(Readable readable);
  
  /** Indicates that there was an error in the source connection
      of the specified Readable.
      @param String errorDescription: the error message
      @param Readable readable: the Readable where the error
        occured
  */	
  public void errorEvent(String errorDescription, Readable readable);  
  
  /** Sets the Readable from which data has to be read.
      @param Readable readable: the Readable from which data has
        to be read
  */	
  public void setReadable(Readable readable);
  
  /** Returns the Readable from which data will be read.
      @return Readable: the Readable from which data will be read
  */      
  public Readable getReadable();
  
}    