package com.monica.interfaces;

import java.awt.*;
import java.io.*;

/** Provides functionality for drawing fixed space text on the graphical display.
    This interface is also referenced as "<b>text renderer</b>" for the JavaCom VirtualDisplay.
*/
public interface Text {

  /** Returns the internal name of the Text.
      @return String: the internal name of the Text
  */      
  public String getTextName();
  
  /** Returns the internal version of the Text.
      @return String: the internal version of the Text
  */      
  public String getTextVersion();

  /** Initializes the Text by specifying charWith, charHeight and an InputStream
      for configuration processing.
      @param int sX: the charWith in pixels
      @param int sY: the charheight in pixels
      @param InputStream is: the configuration InputStream
  */      
  public void init(int sX, int sY, InputStream is);
  
  /** Returns the charWidth of a single character.
      @return int: the charWidth of a single characted
  */      
  public int getWidth();
  
  /** Returns the charHeight of a single character.
      @return int: the charHeight of a single character
  */      
  public int getHeight();
  
  /** Draws the specified character (c) at the specified position (x,y: lower left corner)
      with the specified foregroung (f) and background (b) colors on the specified Graphics (g).
      @param char c: the char to be drawn
      @param int x: the left X coordinate of the char
      @param int y: the lower Y coordinate of the char
      @param Color f: the foreground color of the char
      @param Color b: the background color of the char
      @param Graphics g: the Graphics on that the char is drawn
  */
  public void drawChar(char c, int x, int y, Color f, Color b, Graphics g);
}