package com.monica.interfaces;

/** Provides functionality for implementing timers. Started timers will 
    supply event notifications in regular intervals until they are killed.
*/    
public interface Timer {

  /** Returns the internal name of the Timer.
      @return String: the internal name of the Timer
  */      
  public String getTimerName();
  
  /** Returns the internal version of the Timer.
      @return String: the internal version of the Timer
  */      
  public String getTimerVersion();

  /** Starts the timer. The timer will call the callback method each passed interval 
      milliseconds supplying the provided timerID as well until the timer is 
      explicitly killed with the killTimer method.
      @param int timerID: the Id that will be supplied to the callback method
      @param int interval: interval in milliseconds between callbacks
      @return boolean: wether the timer could be started or not
  */
  public boolean setTimer(int timerID, int interval);
  
  /** Kills the timer with the specified timerID.
      @param int timerID: the timerID of the to be killed timer
      @return boolean: wether the timer could be killed or not
  */      
  public boolean killTimer(int timerID);

}