package com.monica.interfaces;

import com.monica.interfaces.*;

/** Implements the source part of the write functionality in the
    TM layer hierarchy. Writer is always the source of data
    fom the Writeable. Writing in the TM layer hierarchy is synchronous.
    If the Writer is willing to write, it calls one of the write
    methods of the Writeable. The write will return after the data is 
    submitted to bottom layers. Data can be buffered by the Writeable.
*/    
public interface Writer {

  /** Returns the internal name of the Writer.
      @return String: the internal name of the Writer
  */      
  public String getWriterName();
  
  /** Returns the internal version of the Writer.
      @return String: the internal version of the Writer
  */      
  public String getWriterVersion();
  
  /** Sets the Writeable to which this Writer will write.
      @param Writeable writeable: the Writeable to which this
        Writer will write
  */	
  public void setWriteable(Writeable writeable);
  
  /** Returns the Writeable to which this Writer will write.
      @return Writeable: the Writeable to which this Writer
        will write
  */	
  public Writeable getWriteable();
  
}    