package com.monica.javacom.application;

import com.monica.interfaces.*;

import com.monica.util.debug.*;
import com.monica.util.configuration.*;

import com.monica.javacom.ssh2.*;
import com.monica.javacom.ssh2.scp2.*;
import com.monica.javacom.ssh2.interfaces.*;

import java.util.*;

public class SCP2 implements SSH2ChannelManager, LayerBottomView, Runnable {
    
    // ---------------------------------------------------------------
    // Constants and variables
    // ---------------------------------------------------------------
    
    private static final String DP="SCP2";
    private static final String NAME="SCP2";
    private static final String VERSION="0.1";
    
    private static final String CONF_NAME="JavaCom.ssh2client";
    
    private SSH2Channel ch;
    
    private Configuration topLevel;
    
    private SSH2IO io;
    
    private String inBufS;
    private Vector inBufP;
    
    private int id;
    
    private Object waitHandleLock;
    private String lastHandle;
    
    private Object waitNameLock;
    private SCP2Name[] lastNames;
    
    private Thread myThread;
    
    // ---------------------------------------------------------------
    // Entry point
    // ---------------------------------------------------------------
    
    public static void main(String args[]) {
        
        try {
            
            if (args.length!=1) {
                System.out.println("Usage: java SCP2 loaderFile");
                System.exit(-1);
            }
            
            Configuration topLevel=ConfigurationLoader.load(args[0]);
            DebugManager.startUp(topLevel);
            
            SCP2 s=new SCP2(topLevel);
            s.start();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    // ---------------------------------------------------------------
    // SCP2
    // ---------------------------------------------------------------
    
    private SCP2(Configuration tL) {
        topLevel=tL;
        io=new SSH2IO();
        inBufS=new String();
        inBufP=new Vector();
        id=0;
        lastHandle=null;
    }
    
    private void start() throws Exception {
            
        int port=topLevel.getConfiguration(CONF_NAME).getInt("destPort");        
        String host=topLevel.getConfiguration(CONF_NAME).getString("destHost");        
        boolean performServerCheck=topLevel.getConfiguration(CONF_NAME).getBoolean("performServerCheck", false);        
            
        SSH2Facade facade=new SSH2Facade();
        facade.setConfiguration(topLevel);
        
        SSH2ConfigurationManager ssh2configM=new SSH2ConfigurationManager(topLevel);
        ssh2configM.registerClasses(facade);
            
        SSH2ClientSide clientSide=facade.createClient(this);           
        ssh2configM.enableClasses(clientSide);
            
        clientSide.startTransportLayer(host,port,performServerCheck);
            
        DebugManager.logOut(DP, "Transport layer established.");
        
        if (clientSide.connected()) {
            String uamethod=topLevel.getConfiguration(CONF_NAME).getString("userauthentication");          
            DebugManager.logOut(DP, uamethod);
            clientSide.startUserAuthenticationLayer(uamethod, this);
    
            DebugManager.logOut(DP, "User authentication layer established.");
          
            if (clientSide.connected()) {
	    
                DebugManager.logOut(DP, "Clied Side connected");            
                
                clientSide.startConnectionLayer(this, this);
                    
                SSH2ConnectionLayer cL=clientSide.getConnectionLayer();
                
                ch=cL.getNewChannel();
                cL.startChannel(ch);
            }
        }
    }
    
    // ---------------------------------------------------------------
    // scp2 functionality
    // ---------------------------------------------------------------
    
    private void _ready() throws Exception {        
        String s=io.wUint32(SCP2Constants.PROTO_VERSION);        
        _sendPacket(SCP2Constants.SSH_FXP_INIT, s);                
    }
    
    private synchronized void _sendPacket(int type, String s) throws Exception {
        DebugManager.logOut(DP, "Packet sent: "+type);
        String toSend=io.wUint32(s.length()+1);
        toSend+=io.wByte(type);
        toSend+=s;
        ch.write(toSend);
        ch.flush();        
    }
    
    private synchronized void _processStream() throws Exception {
        String read=ch.getInput();
        
        inBufS+=read;
        
        int len=io.rUint32Int(inBufS);
        if (inBufS.length()>=(4+len)) { // there is a whole scp packet
            SCP2Packet pack=new SCP2Packet();
            pack.length=len;
            pack.type=io.rByteInt(inBufS.substring(4, 5));
            pack.payload=inBufS.substring(5, 4+len);
            if (inBufS.length()>(4+len)) {
                inBufS=inBufS.substring(5+len);
            }
            else {
                inBufS=new String();
            }
            inBufP.addElement(pack);
        }
    }
    
    private synchronized void _processData() throws Exception {
        int len=inBufP.size();
        SCP2Packet p;
        for (int i=0; i<len; i++) {
            p=(SCP2Packet)(inBufP.elementAt(0));
            inBufP.removeElementAt(0);
            _processPacket(p);
        }
    }
    
    private void _processPacket(SCP2Packet p) throws Exception {
    
        switch (p.type) {
            
            // SSH_FXP_INIT
            
            case SCP2Constants.SSH_FXP_INIT :
                DebugManager.logOut(DP, "SSH_FXP_INIT");
                break;
                
            // SSH_FXP_VERSION
                
            case SCP2Constants.SSH_FXP_VERSION :
                DebugManager.logOut(DP, "SSH_FXP_VERSION");
                int remoteProtoVersion=io.rUint32Int(p.payload);
                DebugManager.logOut(DP, "rpv="+remoteProtoVersion);
                if (remoteProtoVersion>=SCP2Constants.PROTO_VERSION) {                    
                    _protocolReady();
                }
                else {
                    throw new Exception("Incompatible remote protocol version.");
                }
                break;
                
            // SSH_FXP_OPEN
                
            case SCP2Constants.SSH_FXP_OPEN :
                DebugManager.logOut(DP, "SSH_FXP_OPEN");
                break;
                
            // SSH_FXP_CLOSE    
                
            case SCP2Constants.SSH_FXP_CLOSE :
                DebugManager.logOut(DP, "SSH_FXP_CLOSE");
                break;
                
            // SSH_FXP_READ    
                
            case SCP2Constants.SSH_FXP_READ :
                DebugManager.logOut(DP, "SSH_FXP_READ");
                break;
              
            // SSH_FXP_WRITE    
                
            case SCP2Constants.SSH_FXP_WRITE :
                DebugManager.logOut(DP, "SSH_FXP_WRITE");
                break;
                
            // SSH_FXP_LSTAT    
                
            case SCP2Constants.SSH_FXP_LSTAT :
                DebugManager.logOut(DP, "SSH_FXP_LSTAT");
                break;
                
            // SSH_FXP_FSTAT    
                
            case SCP2Constants.SSH_FXP_FSTAT :
                DebugManager.logOut(DP, "SSH_FXP_FSTAT");
                break;
                
            // SSH_FXP_SETSTAT    
                
            case SCP2Constants.SSH_FXP_SETSTAT :
                DebugManager.logOut(DP, "SSH_FXP_SETSTAT");
                break;
                
            // SSH_FXP_FSETSTAT    
                
            case SCP2Constants.SSH_FXP_FSETSTAT :
                DebugManager.logOut(DP, "SSH_FXP_FSETSTAT");
                break;
                
            // SSH_FXP_OPENDIR    
                
            case SCP2Constants.SSH_FXP_OPENDIR :
                DebugManager.logOut(DP, "SSH_FXP_OPENDIR");
                break;
                
           // SSH_FXP_READDIR     
                
            case SCP2Constants.SSH_FXP_READDIR :
                DebugManager.logOut(DP, "SSH_FXP_READDIR");
                break;
                
            // SSH_FXP_REMOVE    
                
            case SCP2Constants.SSH_FXP_REMOVE :
                DebugManager.logOut(DP, "SSH_FXP_REMOVE");
                break;
                
            // SSH_FXP_MKDIR    
                
            case SCP2Constants.SSH_FXP_MKDIR :
                DebugManager.logOut(DP, "SSH_FXP_MKDIR");
                break;
                
            // SSH_FXP_RMDIR    
                
            case SCP2Constants.SSH_FXP_RMDIR :
                DebugManager.logOut(DP, "SSH_FXP_RMDIR");
                break;
                
            // SSH_FXP_REALPATH    
                
            case SCP2Constants.SSH_FXP_REALPATH :
                DebugManager.logOut(DP, "SSH_FXP_REALPATH");
                break;
                
            // SSH_FXP_STAT    
                
            case SCP2Constants.SSH_FXP_STAT :
                DebugManager.logOut(DP, "SSH_FXP_STAT");
                break;
                
            // SSH_FXP_RENAME    
                
            case SCP2Constants.SSH_FXP_RENAME :
                DebugManager.logOut(DP, "SSH_FXP_RENAME");
                break;
                
            // SSH_FXP_STATUS    
                
            case SCP2Constants.SSH_FXP_STATUS :
                DebugManager.logOut(DP, "SSH_FXP_STATUS");
                break;
                
            // SSH_FXP_HANDLE    
                
            case SCP2Constants.SSH_FXP_HANDLE :
                DebugManager.logOut(DP, "SSH_FXP_HANDLE");
                
                if (lastHandle!=null) {
                    throw new Exception("Last handle not handled.");
                }
                lastHandle=io.rString(p.payload.substring(4));
                if (waitHandleLock!=null) {
                    synchronized (waitHandleLock) {
                        waitHandleLock.notify();
                    }
                }
                
                break;
                
            // SSH_FXP_DATA    
                
            case SCP2Constants.SSH_FXP_DATA :
                DebugManager.logOut(DP, "SSH_FXP_DATA");
                break;
                
            // SSH_FXP_NAME    
                
            case SCP2Constants.SSH_FXP_NAME :
                DebugManager.logOut(DP, "SSH_FXP_NAME");
                
                if (lastNames!=null) {
                    throw new Exception("Last names not handled.");
                }
                
                int count=io.rUint32Int(p.payload.substring(4));
                lastNames=new SCP2Name[count];
                String d=p.payload.substring(8);
                
                DebugManager.logOut(DP, "count="+count);
                
                for (int i=0; i<count; i++) {
                    lastNames[i]=new SCP2Name();
                    
                    lastNames[i].filename=io.rString(d);
                    d=d.substring(4+lastNames[i].filename.length());
                    
                    lastNames[i].longname=io.rString(d);
                    d=d.substring(4+lastNames[i].longname.length());
                    
                    lastNames[i].attrs=_readATTRS(d);
                    d=d.substring(lastNames[i].attrs._bLen);
                }
                
                if (waitNameLock!=null) {
                    synchronized (waitNameLock) {
                        waitNameLock.notify();
                    }
                }
                
                break;
                
            // SSH_FXP_ATTRS    
                
            case SCP2Constants.SSH_FXP_ATTRS :
                DebugManager.logOut(DP, "SSH_FXP_ATTRS");
                break;
                
            // SSH_FXP_EXTENDED    
                
            case SCP2Constants.SSH_FXP_EXTENDED :
                DebugManager.logOut(DP, "SSH_FXP_EXTENDED");
                break;
                
            // SSH_FXP_EXTENDED_REPLY    
                
            case SCP2Constants.SSH_FXP_EXTENDED_REPLY :
                DebugManager.logOut(DP, "SSH_FXP_EXTENDED_REPLY");
                break;
        }
    }
    
    private void _finish() {
        DebugManager.logOut(DP, "Finishing.");
        ch.flush();
        ch.closeConnections();
        System.exit(0);
    }
    
    private SCP2ATTRS _readATTRS(String s) {
        SCP2ATTRS attrs=new SCP2ATTRS();
                        
        attrs.flags=io.rUint32Int(s);
        s=s.substring(4);
        attrs._bLen=4;
        
        
        if ((attrs.flags&SCP2Constants.SSH_FILEXFER_ATTR_SIZE)>0) {        
            attrs.size=io.rUint64Long(s);
            s=s.substring(8);
            attrs._bLen+=8;
        }
        
        if ((attrs.flags&SCP2Constants.SSH_FILEXFER_ATTR_UIDGID)>0) {        
            attrs.uid=io.rUint32Int(s);
            attrs.gid=io.rUint32Int(s.substring(4,8));
            s=s.substring(8);
            attrs._bLen+=8;
        }

        if ((attrs.flags&SCP2Constants.SSH_FILEXFER_ATTR_PERMISSIONS)>0) {        
            attrs.permissions=io.rUint32Int(s);
            s=s.substring(4);
            attrs._bLen+=4;
        }
        
        if ((attrs.flags&SCP2Constants.SSH_FILEXFER_ATTR_ACMODTIME)>0) {        
            attrs.atime=io.rUint32Int(s);
            attrs.mtime=io.rUint32Int(s.substring(4,8));        
            s=s.substring(8);
            attrs._bLen+=8;
        }
        
        if ((attrs.flags&SCP2Constants.SSH_FILEXFER_ATTR_EXTENDED)>0) {        
            int extended_count=io.rUint32Int(s);
            s=s.substring(4);
            attrs._bLen+=4;
        
            DebugManager.logOut(DP, "ec="+extended_count);
        
            if (extended_count>0) {
                attrs.extended=new SCP2Extended[extended_count];
                for (int i=0; i<extended_count; i++) {
                    attrs.extended[i]=_processExtended(s);
                    s=s.substring(attrs.extended[i]._bLen);
                    attrs._bLen+=attrs.extended[i]._bLen;
                }
            }
        }
        
        return attrs;
    }
    
    private SCP2Extended _processExtended(String s) {
        SCP2Extended ext=new SCP2Extended();
        
        ext.extended_type=io.rString(s);
        s=s.substring(4+ext.extended_type.length());
        
        ext.extended_data=io.rString(s);
        s=s.substring(4+ext.extended_data.length());
        
        ext._bLen=(8+ext.extended_data.length()+ext.extended_type.length());
        
        return ext;
    }
    
    private void _protocolReady() {
        myThread=new Thread(this);
        myThread.start();
    }
    
    private synchronized int _getNewID() {
        return id++;
    }
    
    private String _waitHandle(int type, String toSend) {
        waitHandleLock=new Object();
        try {
            synchronized (waitHandleLock) {
                _sendPacket(type, toSend);
                waitHandleLock.wait();
                String t=new String(lastHandle);
                lastHandle=null;
                return t;
            }
        }
        catch (Exception e) {
            DebugManager.logException(DP, e);
            return null;
        }
    }
    
    private SCP2Name[] _waitName(int type, String toSend) {
        waitNameLock=new Object();
        try {
            synchronized (waitNameLock) {
                _sendPacket(type, toSend);
                waitNameLock.wait();
                SCP2Name[] n=lastNames;
                lastNames=null;
                return n;
            }
        }
        catch (Exception e) {
            DebugManager.logException(DP, e);
            return null;
        }
    }
    
    // ---------------------------------------------------------------
    // Runnable
    // ---------------------------------------------------------------
    
    public void run() {
        DebugManager.logOut(DP, "Protocol initialization ready.");
        
        // opening dir
        
        String toSend=io.wUint32(_getNewID());
        toSend+=io.wString(".");
        String handle=_waitHandle(SCP2Constants.SSH_FXP_OPENDIR, toSend);
        DebugManager.logOut(DP, "Handle="+handle);
        
        // reading dir
        
        toSend=io.wUint32(_getNewID());
        toSend+=io.wString(handle);
        
        SCP2Name[] names;
        while (true) {
            names=_waitName(SCP2Constants.SSH_FXP_READDIR, toSend);
            if (names==null) {
                break;
            }
            else {
                for (int i=0; i<names.length; i++) {
                    DebugManager.logOut(DP, names[i].longname);
                }
            }
        }
    }
                    
    // ---------------------------------------------------------------
    // SSH2ChannelManager
    // ---------------------------------------------------------------
                    
    public void channelArrived(SSH2Channel channel) {
    }
    
    public void channelClosed(SSH2Channel channel) {
        System.exit(0);
    }
    
    public boolean channelRequest(SSH2Channel channel, SSH2Packet pack) {
        DebugManager.logOut(DP, "Requested: "+io.rString(pack.payload.substring(1)));
        return false;
    }
    
    public void channelReady(SSH2Channel channel) {
        try {
            channel.setReader(this);
            _ready();
        }
        catch (Exception e) {
            DebugManager.logException(DP, e);
        }
    }
    
    // ---------------------------------------------------------------
    // readable
    // ---------------------------------------------------------------
    
    public String getReaderName() {
        return NAME;
    }
    
    public String getReaderVersion() {
        return VERSION;
    }
    
    public void readEvent(Readable readable) {
        try {
            _processStream();
            _processData();
        }
        catch (Exception e) {
            DebugManager.logException(DP, e);
        }
    }
    
    public void connectionClosedEvent(Readable readable) {
        _finish();
    }
    
    public void errorEvent(String errorDescription, Readable readable) {
        _finish();
    }
    
    public void setReadable(Readable r) {
    }
    
    public Readable getReadable() {
        return ch;
    }
                                    
    // ---------------------------------------------------------------
    // Writer
    // ---------------------------------------------------------------
    
    public String getWriterName() {
        return NAME;
    }
    
    public String getWriterVersion() {
        return VERSION;
    }
    
    public void setWriteable(Writeable writeable) {
    }
    
    public Writeable getWriteable() {
        return ch;
    }
    
    // ---------------------------------------------------------------
    // Configureable
    // ---------------------------------------------------------------
    
    public void setConfiguration(Configuration conf) {
    }
    
    public Configuration getConfiguration() {
        return topLevel;
    }
    
    // ---------------------------------------------------------------
    // LayerBottomView
    // ---------------------------------------------------------------
    
    public String getLayerBottomViewName() {
        return NAME;
    }
    
    public String getLayerBottomViewVersion() {
        return VERSION;
    }
}
