package com.monica.javacom.application;

import com.monica.javacom.interfaces.*;
import com.monica.javacom.transport.*;
import com.monica.javacom.terminal.*;
import com.monica.javacom.virtualinput.*;
import com.monica.javacom.virtualdisplay.*;
import com.monica.javacom.window.*;
import com.monica.javacom.application.*;
import com.monica.util.*;
import com.monica.javacom.ssh2.*;
import com.monica.interfaces.*;
import com.monica.util.configuration.*;
import com.monica.javacom.ssh2.interfaces.*;
import com.monica.javacom.ssh2.userauthentication.*;
import com.monica.javacom.ssh2.connection.*;

import java.io.*;
import java.awt.*;
import java.awt.event.*;

/** Class for running the client side functionality of the SSH2 protocol. This class has to
    be run fromt he command line.
    <b>Usage: java com.monica.javacom.application.SSHClient destHost destPort</b><br><br>
    The configuration data will be read from ./.conf and the softfont configuration 
    from ./fonts.conf.
*/
public class SSH2ClientApp {
    
  /** Entry point for the JAVA VM.
      @param String[] args: the command line parameters
  */
  public static void main(String[] args) {
      
    VirtualDisplay display=null;

    try {
        
      if (args.length>=1) {                            
        String loaderFile=args[0];
        
        Configuration loadedConfig=ConfigurationLoader.load(loaderFile);                  
                
        display=new AWTDisplay(loadedConfig);
        VirtualInput input=new KeyHandler();
        Terminal terminal=new VT220Terminal();      

        terminal.setConfiguration(loadedConfig);
        terminal.setVirtualDisplay(display);

        input.setConfiguration(loadedConfig);
        input.setWriteable(terminal);
                    
        com.monica.javacom.interfaces.Window window=new AWTWindow(display, loadedConfig, (KeyListener)input);      
        window.setConfiguration(loadedConfig);
        ((Frame)window).setVisible(true);
        
        SSH2ClientConnectionParam cParam=new SSH2ClientConnectionParam();
        cParam.display=display;
        cParam.terminal=terminal;
                
        SSH2ClientSide clientSide=SSH2ClientImpl.startUp(loadedConfig, terminal, cParam);
        
        Object o=new Object();
        synchronized (o) {
          o.wait();
        }
        
      }
      else {
        System.out.println("Usage: ");
        System.out.println("java SSHClient loaderFile");
        System.exit(-1);                    
      }
    }
    catch (Exception e) {
      if (display!=null) {  
        display.clear();
        display.setCPos(0,0);
        display.addS("Connection to the server could not be established. Reason: "+e.toString());
        display.refresh();
        e.printStackTrace();
      }
      else {
        System.err.println("Connection to the server could not be established. Reason: "+e.toString());
        e.printStackTrace();
      }
    }
  }
}