package com.monica.javacom.interfaces;

import com.monica.javacom.interfaces.*;
import com.monica.interfaces.*;
import java.io.*;

/** Bottom level layer in the TM layer hierarchy responsible for
    the basic connection between the client and server side. This
    layer has to ensure the transfer of the streams between the 
    two sides.
*/    

public interface Transport extends LayerTopView, Closeable, Configureable, Runnable {

  /** Returns the internal name of the Transport.
      @return String: the internal name of the Transport
  */      
  public String getTransportName();
  
  /** Returns the internal version of the Transport.
      @return String: the internal version of the Transport
  */      
  public String getTransportVersion();

  /** Initializes the Transport as the server side. Clients
      can connect to the specified port.
      @param int port: the port where the clients can connect
      @exception IOException: if the server could not be started
  */  
  public void initAsServer(int port) throws IOException;
  
  /** Initializes the Transport as the client side. This client
      will try to connect to the specified host and port.
      @param String host: the destination host
      @param int port: the destination port
      @exception IOException: if the client cannot connect to
        the specified host and port
  */
  public void initAsClient(String host, int port) throws IOException;
  
  /** Closes all connections of the Transport.
  */  
  public void closeConnections();
  
  /** Generates a checksum for a String. 
      @param String: the String for which a checksum is needed
      @return String: the checksum
  */  
  public String checkSum(String s);
    
  /** Sets the ServerManager which will be notified if a client
      connects.
      @param ServerManager serverManager: the ServerManager
        that has to be notified if a client connects
  */	  
  public void setServerManager(ServerManager serverManager);
  
  /** Returns the ServerManager which will be notified if a client
      connects.
      @return ServerManager: the ServerManager which will
        be notified if a client connects
  */	
  public ServerManager getServerManager();
  
}
