package com.monica.javacom.ssh2;

import com.monica.javacom.ssh2.*;
import com.monica.interfaces.*;
import com.monica.javacom.interfaces.*;
import com.monica.javacom.ssh2.interfaces.*; 

import com.monica.util.debug.*;

import java.util.*;

public abstract class SSH2BothSideFeatures implements Closeable {

  // public return codes
  public static final int REG_NO_OPT=0;
  public static final int REG_ALREADY_ENABLED=1;
  public static final int REG_OK=2;
  public static final int REG_NOT_REGISTERED=3;

  public static final int RET_OK=0;
  public static final int RET_ERROR=1;

  public static final int WAIT_NO=0;
  public static final int WAIT_TR=1;
  public static final int WAIT_UA=2;
  public static final int WAIT_CO=3;
  
  public static final String ERR_NOT_ENABLED = "Feature not enabled.";
  
  private static final String DP="SSH2BothSideFeatures";

  // error messages
  private static final String ERR_NOT_INITIALIZED="Owner SSH2Facade not set.";
  private static final String ERR_NO_OPT="Such a list is not maintained.";
  protected static final String ERR_FAILED="Already failed.";
  
  // closeable
  
  private static final String NAME="SSH2BothSideFeatures";
  private static final String VERSION="0.31";

  // protected variables
  protected SSH2Facade owner;
  protected SSH2IO io;
  protected Transport transport;
  protected SSH2Tokenizer tokenizer;
  protected Configuration configuration;
  
  protected int waiting;
  protected boolean failed;
  
  protected SSH2ConnectionLayer connLayer;
  
  private Object waiter; 

  // private variables
  private Vector features[];

  protected void bsfInit() {
    features=new Vector[SSH2Facade.OPT_COUNT];
    for (int i=0; i<SSH2Facade.OPT_COUNT; i++) {
      features[i]=new Vector();
    }
    io=new SSH2IO();
    waiter=new Object();
    waiting=WAIT_NO;
  }

  public synchronized int enableFeature(int opt, String name) throws Exception {
    if (owner==null) {
      throw new Exception(ERR_NOT_INITIALIZED);
    }
    if ((opt<0) || (opt>=SSH2Facade.OPT_COUNT)) {
      return REG_NO_OPT;
    }
    if (features[opt].indexOf(name)!=-1) {
      return REG_ALREADY_ENABLED;
    }
    String registered=owner.getFeatureList(opt);    
    if (io.listContains(registered,name,SSH2Constants.SEP_COMMA_NA)) {
      features[opt].addElement(name);
      return REG_OK;
    }
    else {
      return REG_NOT_REGISTERED;
    }
  }

  public synchronized String getFeatureList(int opt) throws Exception {
    if ((opt<0) || (opt>=SSH2Facade.OPT_COUNT)) {
      throw new Exception(ERR_NO_OPT);
    }
    return io.createList(features[opt].elements(), SSH2Constants.SEP_COMMA_NA);
  }

  public synchronized Object getFeature(int opt, String name) throws Exception {
    if ((opt<0) || (opt>=SSH2Facade.OPT_COUNT)) {
      throw new Exception(ERR_NO_OPT);
    }
    if (features[opt].indexOf(name)!=-1) {
      return owner.getFeature(opt, name);
    }
    return ERR_NOT_ENABLED;
  }
  
  public synchronized SSH2UserAuthenticationLayer getUserAuthenticationLayer(String name, Object param) throws Exception {
    if (features[SSH2Facade.OPT_USERAUTHENTICATION_LAYER].indexOf(name)!=-1) {
      return owner.getUserAuthenticationLayer(name, param);
    }
    throw new Exception(ERR_NOT_ENABLED);
  }

  public synchronized SSH2ConnectionLayer getConnectionLayer(Object param, SSH2ChannelManager chManager) throws Exception {
    return owner.getConnectionLayer(param, chManager);
  }
    
  public synchronized Object getFeatureParam(int opt, String name) throws Exception {
    return owner.getFeatureParam(opt, name);
  }

  public void waitEvent(int event) {
    synchronized (waiter) {
      waiting=event;
      while (waiting==event) {
        try {
          waiter.wait();
	}
	catch (Exception e) {
	}
      }
    }
  }      

  public void event(int status) {
    if (status==waiting) {
      synchronized (waiter) {
        try {
	  waiting=WAIT_NO;
	  waiter.notify();
	}
	catch (Exception e) {
	}
      }
    }
  }
  
  public String getCloseableVersion() {
      return VERSION;
  }
  
  public String getCloseableName() {
      return NAME;
  }

  public void closeConnections() {
    failed=true;
    try {
      transport.closeConnections();
    }
    catch (Exception e) {
    }
    closeConnectionsImpl();
  }
  
  protected abstract void closeConnectionsImpl();
  
  public SSH2Facade getFacade() {
    return owner;
  }
  
  public boolean connected() {
    if (tokenizer!=null) {  
      return tokenizer.connected();
    }
    return false;
  }
  
  public SSH2Tokenizer getTokenizer() {
      return tokenizer;
  }


}
