package com.monica.javacom.ssh2;

import com.monica.javacom.interfaces.*;
import com.monica.javacom.transport.*;
import com.monica.util.*;
import com.monica.javacom.ssh2.*;
import com.monica.interfaces.*;
import com.monica.util.configuration.*;
import com.monica.javacom.ssh2.interfaces.*;
import com.monica.javacom.ssh2.userauthentication.*;
import com.monica.javacom.ssh2.connection.*;
import com.monica.util.debug.*;

public class SSH2ClientImpl {
    
    private static final String DP="SSH2ClientImpl";
    
    private static final String CONF_NAME="JavaCom.ssh2client";
    
    public static SSH2ClientSide startUp(Configuration loadedConfig, LayerBottomView owner, Object cParam, SSH2ChannelManager chManager) throws Exception {

        int port=loadedConfig.getConfiguration(CONF_NAME).getInt("destPort");        
        String host=loadedConfig.getConfiguration(CONF_NAME).getString("destHost");        
        boolean performServerCheck=loadedConfig.getConfiguration(CONF_NAME).getBoolean("performServerCheck", false);        
        
        DebugManager.logOut(DP, " ********** ");
        DebugManager.logOut(DP, " Connecting to "+host+":"+port);
        DebugManager.logOut(DP, " ********** ");
        
        SSH2Facade facade=new SSH2Facade();
	facade.setConfiguration(loadedConfig);
        
	SSH2ConfigurationManager ssh2configM=new SSH2ConfigurationManager(loadedConfig);
	ssh2configM.registerClasses(facade);
        
        SSH2ClientSide clientSide=facade.createClient(owner);           
	ssh2configM.enableClasses(clientSide);
        
        clientSide.startTransportLayer(host,port,performServerCheck);
	
	DebugManager.logOut(DP, "Transport layer established.");
        
        if (clientSide.connected()) {
            String uamethod=loadedConfig.getConfiguration(CONF_NAME).getString("userauthentication");          
	    clientSide.startUserAuthenticationLayer(uamethod, owner);
	    
	    DebugManager.logOut(DP, "User authentication layer established.");
          
            if (clientSide.connected()) {
	    
//		DebugManager.logOut(DP, "Clied Side connected");
	    
                if (chManager==null) {
		
//		    DebugManager.logOut(DP, "chManager == null");
		
                    clientSide.startConnectionLayer(cParam);
                }
                else {
		
//		    DebugManager.logOut(DP, "chManager != null");
		
                    clientSide.startConnectionLayer(cParam, chManager);
                }
		
                DebugManager.logOut(DP, "returning client side");
		
                return clientSide;
            }
        }        
	
	DebugManager.logOut(DP, "client side could not be created. returning null");
	
        return null;               
    }
    
    public static SSH2ClientSide startUp(Configuration loadedConfig, LayerBottomView owner, Object cParam) throws Exception {
        return startUp(loadedConfig, owner, cParam, null);
    }
    
}