package com.monica.javacom.ssh2;

import com.monica.javacom.ssh2.*;
import com.monica.javacom.interfaces.*;
import com.monica.interfaces.*;
import com.monica.javacom.transport.*;
import com.monica.javacom.ssh2.interfaces.*;
import com.monica.util.debug.*;

public class SSH2ClientSide extends SSH2BothSideFeatures implements SSH2ChannelManager {
    
  private static final String DP="SSH2ClientSide";  

  private static final String NAME="SSH2ClientSide";
  private static final String VERSION="0.31";

  private LayerBottomView topLevel;

  SSH2ClientSide(SSH2Facade owner, LayerBottomView topLevel) {
    this.owner=owner;
    this.topLevel=topLevel;
    failed=false;
    bsfInit();
  }

  public int startTransportLayer(Transport transport, boolean performServerCheck) throws Exception {      
    sTL_1();
    if (performServerCheck) {
        tokenizer.performServerCheck();
    }
    this.transport=transport;
    sTL_2();
    return waitForTokenizer(WAIT_TR);
  }

  public int startTransportLayer(String destHost, int destPort, boolean performServerCheck) throws Exception {
    sTL_1();
    if (performServerCheck) {
        tokenizer.performServerCheck();
    }    
    transport=new TCPTransport();
    sTL_2();
    transport.initAsClient(destHost, destPort);
    return waitForTokenizer(WAIT_TR);
  }
  
  private void sTL_1() throws Exception {
    if (failed) {
      throw new Exception(ERR_FAILED);
    }
    tokenizer=new SSH2Tokenizer(this, SSH2Constants.SSH_SIDE_CLIENT);
  }
  
  private void sTL_2() throws Exception {
    if (configuration!=null) {
      transport.setConfiguration(configuration);
      tokenizer.setConfiguration(configuration);
    }

    tokenizer.setReadable(transport);
    tokenizer.setWriteable(transport);

    transport.setReader(tokenizer);
  }    

  private int waitForTokenizer(int status) throws Exception {    
    waitEvent(status);
    if (tokenizer.getLastError().equals("")) {
      return RET_OK;
    }
    else {
      closeConnections();
      return RET_ERROR;
    }
  }

  public int startUserAuthenticationLayer(String method, Object param) throws Exception {
    SSH2UserAuthenticationLayer layer=getUserAuthenticationLayer(method,param);
    layer.setTokenizer(tokenizer);
    layer.setSide(SSH2Constants.SSH_SIDE_CLIENT);
    tokenizer.startUserAuthentication(layer);
    return waitForTokenizer(WAIT_UA);
  }
  
  public int startConnectionLayer(Object param, SSH2ChannelManager chManager) throws Exception {
    connLayer=getConnectionLayer(param, chManager);
    connLayer.setTokenizer(tokenizer);
    connLayer.setSide(SSH2Constants.SSH_SIDE_CLIENT);
    tokenizer.startConnection(connLayer);
    if (tokenizer.isConnectionLayerReady()) {
        return RET_OK;
    }
    else {
        return waitForTokenizer(WAIT_CO);      
    }
  }
  
  public SSH2ConnectionLayer getConnectionLayer() {
      return connLayer;
  }
  
  public SSH2Channel startNewChannel() {
      return connLayer.startNewChannel();
  }
  
  public int startConnectionLayer(Object param) throws Exception {
      return startConnectionLayer(param, this);
  }
  
  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
  }

  public Configuration getConfiguration() {
    return configuration;
  }

  public void closeConnectionsImpl() {
    owner.clientDone(this);
  }
  
  public boolean channelRequest(SSH2Channel channel, SSH2Packet pack) {
    return false;  
  }
  
  public void channelArrived(SSH2Channel channel) {
    topLevel.setReadable(channel);
    topLevel.setWriteable(channel);
    channel.setReader(topLevel);
  }
  
  public void channelClosed(SSH2Channel channel) {
  }  
  
  public void channelReady(SSH2Channel channel) {
  }
  
}

