package com.monica.javacom.ssh2;

import com.monica.interfaces.*;
import com.monica.util.configuration.*;
import com.monica.util.debug.*;

import java.util.*;

public class SSH2ConfigurationManager {
    
  private static final String DP="SSH2ConfigurationManager";  
  
  private static final String CONF_NAME="JavaCom.ssh2";

  private Configuration configuration;  
  private Configuration ssh2Configuration;
  
  private static final String CONF_KA="KEX";
  private static final String CONF_SHKA="HOST_KEY";
  private static final String CONF_EA_CTS="ENCRYPTION_CTS";
  private static final String CONF_EA_STC="ENCRYPTION_STC";
  private static final String CONF_MA_CTS="MAC_CTS";
  private static final String CONF_MA_STC="MAC_STC";
  private static final String CONF_CO_CTS="COMPRESSION_CTS";
  private static final String CONF_CO_STC="COMPRESSION_STC";
  private static final String CONF_LA_CTS="LANGUAGES_CTS";
  private static final String CONF_LA_STC="LANGUAGE_STC";
  
  private static final String CONF_UA="USERAUTHENTICATION";
  private static final String CONF_C="CONNECTION";
  
  private static final String CONF_IV_CTS_PREFIX="ENCRYPTION_CTS_IV_";
  private static final String CONF_IV_STC_PREFIX="ENCRYPTION_STC_IV_";
  private static final String CONF_CI_CTS_PREFIX="ENCRYPTION_CTS_CI_";
  private static final String CONF_CI_STC_PREFIX="ENCRYPTION_STC_CI_";
  private static final String CONF_MAC_CTS_PREFIX="MAC_CTS_LEN_";
  private static final String CONF_MAC_STC_PREFIX="MAC_STC_LEN_";
  
  private static final String CONF_UA_PREFIX="USERAUTHENTICATION";
  private static final String CONF_C_PREFIX="CONNECTION";
  
  private static final String CONF_SEP="_";
  
  private static final String[] CONF_CI_CTS_ARR={CONF_CI_CTS_PREFIX, CONF_IV_CTS_PREFIX};
  private static final String[] CONF_CI_STC_ARR={CONF_CI_STC_PREFIX, CONF_IV_STC_PREFIX};
  private static final String[] CONF_MA_CTS_ARR={CONF_MAC_CTS_PREFIX};
  private static final String[] CONF_MA_STC_ARR={CONF_MAC_STC_PREFIX};
  private static final String[] CONF_CO_CTS_ARR={};
  private static final String[] CONF_CO_STC_ARR={};    
  
  private static final String[] CONF_UA_ARR={};
  private static final String[] CONF_C_ARR={};
  
  private static final int REG=0;
  private static final int ENA=1;
  
  private SSH2IO io;
  
  public SSH2ConfigurationManager(Configuration configuration) throws Exception {
    this.configuration=configuration;
    ssh2Configuration=configuration.getConfiguration(CONF_NAME);
    io=new SSH2IO();
  }
  
  public void registerClasses(SSH2Facade facade) {    
  
    // cipher
    process(facade, null, CONF_EA_CTS, CONF_CI_CTS_ARR, REG, SSH2Facade.OPT_CIPHER_ALGORITHM_CTS);
    process(facade, null, CONF_EA_STC, CONF_CI_STC_ARR, REG, SSH2Facade.OPT_CIPHER_ALGORITHM_STC);
    
    // mac
    process(facade, null, CONF_MA_CTS, CONF_MA_CTS_ARR, REG, SSH2Facade.OPT_MAC_ALGORITHM_CTS);
    process(facade, null, CONF_MA_STC, CONF_MA_STC_ARR, REG, SSH2Facade.OPT_MAC_ALGORITHM_STC);
    
    // compression
    process(facade, null, CONF_CO_CTS, CONF_CO_CTS_ARR, REG, SSH2Facade.OPT_COMPRESSION_ALGORITHM_CTS);
    process(facade, null, CONF_CO_STC, CONF_CO_STC_ARR, REG, SSH2Facade.OPT_COMPRESSION_ALGORITHM_STC);    
    
    // userauthentication
    process(facade, null, CONF_UA, CONF_UA_ARR, REG, SSH2Facade.OPT_USERAUTHENTICATION_LAYER);
    
    // connection
    process(facade, null, CONF_C, CONF_C_ARR, REG, SSH2Facade.OPT_CONNECTION_LAYER);
  }
  
  public void enableClasses(SSH2BothSideFeatures side) {
    
    // cipher
    process(null, side, CONF_EA_CTS, null, ENA, SSH2Facade.OPT_CIPHER_ALGORITHM_CTS);
    process(null, side, CONF_EA_STC, null, ENA, SSH2Facade.OPT_CIPHER_ALGORITHM_STC);
    
    // mac
    process(null, side, CONF_MA_CTS, null, ENA, SSH2Facade.OPT_MAC_ALGORITHM_CTS);
    process(null, side, CONF_MA_STC, null, ENA, SSH2Facade.OPT_MAC_ALGORITHM_STC);
    
    // compression
    process(null, side, CONF_CO_CTS, null, ENA, SSH2Facade.OPT_COMPRESSION_ALGORITHM_CTS);
    process(null, side, CONF_CO_STC, null, ENA, SSH2Facade.OPT_COMPRESSION_ALGORITHM_STC);
    
    // userauthentication
    process(null, side, CONF_UA, null, ENA, SSH2Facade.OPT_USERAUTHENTICATION_LAYER);
    // process(null, side, CONF_C, null, ENA, SSH2Facade.OPT_CONNECTION_LAYER);
  }    
    
  private void process(SSH2Facade facade, SSH2BothSideFeatures side, String prefix, String[] arr, int status, int opt) {
  
    try {
      String list=ssh2Configuration.getString(prefix);
      String cur_item;
      String className;
      int o[];
      int i;
      
      while (list.length()>0) {
      
        cur_item=io.getNextToken(list, SSH2Constants.SEP_COMMA);
	
	switch (status) {
	
	  case ENA :
	    side.enableFeature(opt,cur_item);
	    break;
	    
	  case REG :
	    className=ssh2Configuration.getString(prefix+CONF_SEP+cur_item);
	    o=new int[arr.length];
	    for (i=0; i<arr.length; i++) {
	      o[i]=ssh2Configuration.getInt(arr[i]+cur_item);
	    }
  //          DebugManager.logOut(DP, "Registering: ["+opt+"] "+className);
	    facade.registerClassForFeature(opt, cur_item, className, o);
	    break;
	
	}
	
	list=io.delNextToken(list, SSH2Constants.SEP_COMMA);
      }	
    }
    catch (Exception e) {
      DebugManager.logException(DP, e);
    }
  }
}          
