package com.monica.javacom.ssh2;

/** Class for providing public constants that are defined by the SSH2 standard. */
public class SSH2Constants {

  // *******************************************************************************************************
  // Transport layer constants
  // *******************************************************************************************************

  // Compression

  public static final String SSH_TRANSPORT_COMPRESSION_NONE="none";
  public static final String SSH_TRANSPORT_COMPRESSION_ZLIB="zlib";

  // Encryption

  public static final String SSH_TRANSPORT_ENCRYPTION_3DES_CBC="3des-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_BLOWFISH_CBC="blowfish-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_TWOFISH256_CBC="twofish256-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_TWOFISH_CBC="twofish-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_TWOFISH192_CBC="twofish192-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_TWOFISH128_CBC="twofish128-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_AES256_CBC="aes256-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_AES192_CBC="aes192-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_AES128_CBC="aes128-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_SERPENT256_CBC="serpent256-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_SERPENT192_CBC="serpent192-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_SERPENT128_CBC="serpent128-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_ARCFOUR="arcfour";
  public static final String SSH_TRANSPORT_ENCRYPTION_IDEA_CBC="idea-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_CAST128_CBC="cast128-cbc";
  public static final String SSH_TRANSPORT_ENCRYPTION_NONE="none";

  // MAC

  public static final String SSH_TRANSPORT_MAC_HMAC_SHA1="hmac-sha1";
  public static final String SSH_TRANSPORT_MAC_HMAC_SHA1_96="hmac-sha1-96";
  public static final String SSH_TRANSPORT_MAC_HMAC_MD5="hmac-md5";
  public static final String SSH_TRANSPORT_MAC_HMAC_MD5_96="hmac-md5-96";
  public static final String SSH_TRANSPORT_MAC_NONE="none";

  // Key Exchange

  public static final String SSH_TRANSPORT_KEX_DIFFIE_HELLMAN_GROUP1_SHA1="diffie-hellman-group1-sha1";

  // Public Key Algorithms

  public static final String SSH_TRANSPORT_PKA_SSH_DSS="ssh-dss";
  public static final String SSH_TRANSPORT_PKA_SSH_RSA="ssh-rsa";
  public static final String SSH_TRANSPORT_PKA_X509V3_SIGN_RSA="x509v3-sign-rsa";
  public static final String SSH_TRANSPORT_PKA_X509V3_SIGN_DSS="x509v3-sign-dss";
  public static final String SSH_TRANSPORT_PKA_SPKI_SIGN_RSA="spki-sign-rsa";
  public static final String SSH_TRANSPORT_PKA_SPKI_SIGN_DSS="spki-sign-dss";
  public static final String SSH_TRANSPORT_PKA_PGP_SIGN_RSA="pgp-sign-rsa";
  public static final String SSH_TRANSPORT_PKA_PGP_SIGN_DSS="pgp-sign-dss";

  // Disconnect reasons

  public static final int SSH_TRANSPORT_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT=1;
  public static final int SSH_TRANSPORT_DISCONNECT_PROTOCOL_ERROR=2;
  public static final int SSH_TRANSPORT_DISCONNECT_KEY_EXCHANGE_FAILED=3;
  public static final int SSH_TRANSPORT_DISCONNECT_RESERVED=4;
  public static final int SSH_TRANSPORT_DISCONNECT_MAC_ERROR=5;
  public static final int SSH_TRANSPORT_DISCONNECT_COMPRESSION_ERROR=6;
  public static final int SSH_TRANSPORT_DISCONNECT_SERVICE_NOT_AVAILABLE=7;
  public static final int SSH_TRANSPORT_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED=8;
  public static final int SSH_TRANSPORT_DISCONNECT_HOST_KEY_NOT_VERIFIABLE=9;
  public static final int SSH_TRANSPORT_DISCONNECT_CONNECTION_LOST=10;
  public static final int SSH_TRANSPORT_DISCONNECT_BY_APPLICATION=11;
  public static final int SSH_TRANSPORT_DISCONNECT_TOO_MANY_CONNECTIONS=12;
  public static final int SSH_TRANSPORT_DISCONNECT_AUTH_CANCELLED_BY_USER=13;
  public static final int SSH_TRANSPORT_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE=14;
  public static final int SSH_TRANSPORT_DISCONNECT_ILLEGAL_USER_NAME=15;

  // MSG numbers

  public static final int SSH_TRANSPORT_MSG_DISCONNECT=1;
  public static final int SSH_TRANSPORT_MSG_IGNORE=2;
  public static final int SSH_TRANSPORT_MSG_UNIMPLEMENTED=3;
  public static final int SSH_TRANSPORT_MSG_DEBUG=4;
  public static final int SSH_TRANSPORT_MSG_SERVICE_REQUEST=5;
  public static final int SSH_TRANSPORT_MSG_SERVICE_ACCEPT=6;
  public static final int SSH_TRANSPORT_MSG_KEXINIT=20;
  public static final int SSH_TRANSPORT_MSG_NEWKEYS=21;
  public static final int SSH_TRANSPORT_MSG_KEXDH_INIT=30;
  public static final int SSH_TRANSPORT_MSG_KEXDH_REPLY=31;

  // *******************************************************************************************************
  // Transport layer MSG descriptions
  // *******************************************************************************************************

  // KEXINIT

  public static final String[] KEX_INIT_MSG_DATA_DESC={
    "B","U16","S","S","S","S","S","S","S","S","S","S","O","3"};

  public static final int DI_KEXINIT_TYPE=0;
  public static final int DI_KEXINIT_COOKIE=1;
  public static final int DI_KEXINIT_KEX_ALGORITHMS=2;
  public static final int DI_KEXINIT_SERVER_HOST_KEY_ALGORITHMS=3;
  public static final int DI_KEXINIT_ENCRYPTION_ALGORITHMS_CLIENT_TO_SERVER=4;
  public static final int DI_KEXINIT_ENCRYPTION_ALGORITHMS_SERVER_TO_CLIENT=5;
  public static final int DI_KEXINIT_MAC_ALGORITHMS_CLIENT_TO_SERVER=6;
  public static final int DI_KEXINIT_MAC_ALGORITHMS_SERVER_TO_CLIENT=7;
  public static final int DI_KEXINIT_COMPRESSION_ALGORITHMS_CLIENT_TO_SERVER=8;
  public static final int DI_KEXINIT_COMPRESSION_ALGORITHMS_SERVER_TO_CLIENT=9;
  public static final int DI_KEXINIT_LANGUAGES_CLIENT_TO_SERVER=10;
  public static final int DI_KEXINIT_LANGUAGES_SERVER_TO_CLIENT=11;
  public static final int DI_KEXINIT_FIRST_KEY_PACKET_FOLLOWS=12;
  public static final int DI_KEXINIT_RESERVED=13;

  // KEXDH_REPLY

  public static final String[] KEXDH_REPLY_MSG_DATA_DESC={
    "B","S","M","S"};

  public static final int DI_KEXDH_REPLY_TYPE=0;
  public static final int DI_KEXDH_REPLY_K_S=1;
  public static final int DI_KEXDH_REPLY_F=2;
  public static final int DI_KEXDH_REPLY_SIGNATURE=3;

  // KEXDH_REPLY_SSH_DSS

  public static final String[] KEXDH_REPLY_SSH_DSS_MSG_DATA_DESC={
    "S","M","M","M","M"};

  public static final int DI_KEXDH_REPLY_SSH_DSS_TYPE=0;
  public static final int DI_KEXDH_REPLY_SSH_DSS_P=1;
  public static final int DI_KEXDH_REPLY_SSH_DSS_Q=2;
  public static final int DI_KEXDH_REPLY_SSH_DSS_G=3;
  public static final int DI_KEXDH_REPLY_SSH_DSS_Y=4;

  // KEXDH_REPLY_DSS_BLOB

  public static final String[] KEXDH_REPLY_DSS_BLOB_MSG_DATA_DESC={
    "S","S"};

  public static final int DI_KEXDH_REPLY_DSS_BLOB_TYPE=0;
  public static final int DI_KEXDH_REPLY_DSS_BLOB_DATA=1;

  // *******************************************************************************************************
  // Connection Protocol Constants
  // *******************************************************************************************************

  // Channel open failure

  public static final int SSH_CONNECTION_OPEN_ADMINISTRATIVELY_PROHIBITED=1;
  public static final int SSH_CONNECTION_OPEN_CONNECT_FAILED=2;
  public static final int SSH_CONNECTION_OPEN_UNKNOWN_CHANNEL_TYPE=3;
  public static final int SSH_CONNECTION_OPEN_RESOURCE_SHORTAGE=4;

  // MSG numbers

  public static final int SSH_CONNECTION_MSG_GLOBAL_REQUEST=80;
  public static final int SSH_CONNECTION_MSG_REQUEST_SUCCESS=81;
  public static final int SSH_CONNECTION_MSG_REQUEST_FAILURE=82;
  public static final int SSH_CONNECTION_MSG_CHANNEL_OPEN=90;
  public static final int SSH_CONNECTION_MSG_CHANNEL_OPEN_CONFIRMATION=91;
  public static final int SSH_CONNECTION_MSG_CHANNEL_OPEN_FAILURE=92;
  public static final int SSH_CONNECTION_MSG_CHANNEL_WINDOW_ADJUST=93;
  public static final int SSH_CONNECTION_MSG_CHANNEL_DATA=94;
  public static final int SSH_CONNECTION_MSG_CHANNEL_EXTENDED_DATA=95;
  public static final int SSH_CONNECTION_MSG_CHANNEL_EOF=96;
  public static final int SSH_CONNECTION_MSG_CHANNEL_CLOSE=97;
  public static final int SSH_CONNECTION_MSG_CHANNEL_REQUEST=98;
  public static final int SSH_CONNECTION_MSG_CHANNEL_SUCCESS=99;
  public static final int SSH_CONNECTION_MSG_CHANNEL_FAILURE=100;
  
  // *******************************************************************************************************
  // Connection layer MSG descriptions
  // *******************************************************************************************************

  // WINDOW ADJUST
  
  public static final String[] CONN_CHANNEL_WINDOW_ADJUST_MSG_DATA_DESC={
    "B", "3", "3"};
    
  public static final int DI_CONN_CHANNEL_WINDOW_ADJUST_TYPE=0;  
  public static final int DI_CONN_CHANNEL_WINDOW_ADJUST_CHANNELNO=1;
  public static final int DI_CONN_CHANNEL_WINDOW_ADJUST_BYTES=2;
  
  // DATA
  
  public static final String[] CONN_CHANNEL_DATA_MSG_DATA_DESC={
    "B", "3", "S"};
    
  public static final int DI_CONN_CHANNEL_DATA_TYPE=0;
  public static final int DI_CONN_CHANNEL_DATA_CHANNELNO=1;
  public static final int DI_CONN_CHANNEL_DATA_DATA=2;

  // ENDTENDED DATA
  
  public static final String[] CONN_CHANNEL_EXTENDED_DATA_MSG_DATA_DESC={
    "B", "3", "3", "S"};
    
  public static final int DI_CONN_CHANNEL_EXTENDED_DATA_TYPE=0;
  public static final int DI_CONN_CHANNEL_EXTENDED_DATA_CHANNELNO=1;
  public static final int DI_CONN_CHANNEL_EXTENDED_DATA_CODE=2;
  public static final int DI_CONN_CHANNEL_EXTENDED_DATA_DATA=3;  
  
  // CHANNEL_REQUEST
  
  public static final int RET_CHANNEL_REQUEST_UNKNOWN=0;
  public static final int RET_CHANNEL_REQUEST_SUCCESS=1;
  public static final int RET_CHANNEL_REQUEST_FAILURE=2;
  public static final int RET_CHANNEL_REQUEST_ERR=3;    
  
  public static final String[] CONN_CHANNEL_REQUEST_MSG_DATA_DESC={
    "B", "3", "S", "O"};
    
  public static final int DI_CONN_CHANNEL_REQUEST_TYPE=0;
  public static final int DI_CONN_CHANNEL_REQUEST_CHANNELNO=1;
  public static final int DI_CONN_CHANNEL_REQUEST_REQUEST=2;
  public static final int DI_CONN_CHANNEL_REQUEST_WANTREPLY=3;
  
  // CHANNEL OPEN CONFIRMATION
  
  public static final String[] CONN_CHANNEL_OPEN_CONFIRMATION_MSG_DATA_DESC={
    "B", "3", "3", "3", "3"};
    
  public static final int DI_CONN_CHANNEL_OPEN_CONFIRMATION_TYPE=0;
  public static final int DI_CONN_CHANNEL_OPEN_CONFIRMATION_RECIPIENT=1;
  public static final int DI_CONN_CHANNEL_OPEN_CONFIRMATION_SENDER=2;
  public static final int DI_CONN_CHANNEL_OPEN_CONFIRMATION_WINDOW=3;
  public static final int DI_CONN_CHANNEL_OPEN_CONFIRMATION_PACKET=4;
  
  // CHANNEL OPEN
  
  public static final String[] CONN_CHANNEL_OPEN_MSG_DATA_DESC={
    "B", "S", "3", "3", "3"};
    
  public static final int DI_CONN_CHANNEL_OPEN_TYPE=0;
  public static final int DI_CONN_CHANNEL_OPEN_NAME=1;
  public static final int DI_CONN_CHANNEL_OPEN_SENDER=2;
  public static final int DI_CONN_CHANNEL_OPEN_WINDOW=3;
  public static final int DI_CONN_CHANNEL_OPEN_PACKET=4;
  

  // *******************************************************************************************************
  // User Authentiaction Protocol Constants
  // *******************************************************************************************************

  // MSG numbers

  public static final int SSH_USERAUTH_MSG_USERAUTH_REQUEST=50;
  public static final int SSH_USERAUTH_MSG_USERAUTH_FAILURE=51;
  public static final int SSH_USERAUTH_MSG_USERAUTH_SUCCESS=52;
  public static final int SSH_USERAUTH_MSG_USERAUTH_BANNER=53;
  
  // publickey authentication
  
  public static final int SSH_USERAUTH_MSG_USERAUTH_PK_OK=60;
  
  public static final String[] USERAUTH_REQUEST_MSG_DATA_DESC={
    "B", "S", "S", "S"};    
  public static final int USERAUTH_REQUEST_TYPE=0;
  public static final int USERAUTH_REQUEST_USERNAME=1;
  public static final int USERAUTH_REQUEST_SERVICE=2;
  public static final int USERAUTH_REQUEST_METHOD=3;
    
  public static final String[] PASSWORD_USERAUTH_REQUEST_MSG_DATA_DESC={
    "B", "S", "S", "S", "O", "S"};
  public static final int PASSWORD_USERAUTH_REQUEST_TYPE=0;
  public static final int PASSWORD_USERAUTH_REQUEST_USERNAME=1;
  public static final int PASSWORD_USERAUTH_REQUEST_SERVICE=2;
  public static final int PASSWORD_USERAUTH_REQUEST_METHOD=3;
  public static final int PASSWORD_USERAUTH_REQUEST_BOOLEAN=4;
  public static final int PASSWORD_USERAUTH_REQUEST_PASSWORD=5;
  
  public static final String[] PUBKEY_USERAUTH_REQUEST_MSG_DATA_DESC={
    "B", "S", "S", "S", "O", "S", "S"};
  public static final String[] PUBKEY_USERAUTH_REQUEST_MSG_DATA_DESC_2={
    "B", "S", "S", "S", "O", "S", "S", "S"};
  public static final int PUBKEY_USERAUTH_REQUEST_TYPE=0;
  public static final int PUBKEY_USERAUTH_REQUEST_USERNAME=1;
  public static final int PUBKEY_USERAUTH_REQUEST_SERVICE=2;
  public static final int PUBKEY_USERAUTH_REQUEST_METHOD=3;
  public static final int PUBKEY_USERAUTH_REQUEST_BOOLEAN=4;
  public static final int PUBKEY_USERAUTH_REQUEST_ALG_NAME=5;
  public static final int PUBKEY_USERAUTH_REQUEST_ALG_BLOB=6;
  public static final int PUBKEY_USERAUTH_REQUEST_SIGNATURE=7;
  
  public static final String[] NONE_USERAUTH_REQUEST_MSG_DATA_DESC={
    "B", "S", "S", "S"};
  public static final int NONE_USERAUTH_REQUEST_TYPE=0;
  public static final int NONE_USERAUTH_REQUEST_USERNAME=1;
  public static final int NONE_USERAUTH_REQUEST_SERVICE=2;
  public static final int NONE_USERAUTH_REQUEST_METHOD=3;
  
  
  // *******************************************************************************************************
  // Separator Constants
  // *******************************************************************************************************

  public static final String[] SEP_COMMA={","};
  public static final String SEP_COMMA_NA=",";
  public static final String[] SEP_CRLF={""+((char)13)+((char)10), ""+(char)13, ""+(char)10};

  // *******************************************************************************************************
  // Tokenizer Constants
  // *******************************************************************************************************

  public static final boolean SSH_TOKENIZER_CTS=true;
  public static final boolean SSH_TOKENIZER_STC=false;
  
  public static final int SSH_SIDE_NOT_SET=0;
  public static final int SSH_SIDE_CLIENT=1;
  public static final int SSH_SIDE_SERVER=2;
  
  public static final int STAT_INIT=0;
  public static final int STAT_SSH=1;

  public static final String CRLF=""+((char)13)+((char)10);

  public static final String SSH_NAME="TM_SSH2";
  public static final String SSH_VERSION="2.00";
  public static final String SSH_DIST_VERSION="0.31";
  public static final String SSH_VERSION_STRING="SSH-"+SSH_VERSION+"-"+SSH_NAME+"_"+SSH_DIST_VERSION;
  
  public static final int CONF_CI_CI_POS=0;
  public static final int CONF_CI_IV_POS=1;
  public static final int CONF_MA_LEN_POS=0;
  
  // *******************************************************************************************************
  // Information Constants
  // *******************************************************************************************************
  
  public static final int INFO_USERAUTHENTICATION_METHOD=10;  
  public static final int INFO_USERAUTHENTICATION_PARTNER_USERNAME=11;  
  
  public static final int INFO_CONNECTION_NAME=20;
  public static final int INFO_CHANNEL_NAME=21;
  
  public static final int INFO_CIPHER_CTS=30;
  public static final int INFO_CIPHER_STC=31;
  public static final int INFO_MAC_CTS=32;
  public static final int INFO_MAC_STC=33;
  public static final int INFO_COMPRESSION_CTS=34;
  public static final int INFO_COMPRESSION_STC=35;
  
  public static final int INFO_LOCAL_PORT=40;
  public static final int INFO_REMOTE_HOST=41;
  public static final int INFO_REMOTE_PORT=42;     
}