package com.monica.javacom.ssh2;

import com.monica.interfaces.*;
// import com.monica.javacom.ssh2.cipher.dss.*;
import com.monica.util.ccdm.dss.*;
import com.monica.util.hash.*;
import com.monica.util.configuration.*;
import com.monica.util.*;
import com.monica.util.debug.*;

public class SSH2KeyManager {
    
    private static final String DP="SSH2KeyManager";
    
    private static final String CONF_USERS="users.dss";
    private static final String CONF_DSS_PUB="dss.pub";
    private static final String CONF_DSS_PRIV="dss.priv";
    private static final String CONF_DSS_INFO="dss.info";
    private static final String CONF_SERVER_DSS="server.dss.pub";
            
    private Configuration userConf;
    
    private DSS ownDSS;
    private DSS serverDSS;
    private String ownInfo;
    private SHA1 sha1;
    
    public SSH2KeyManager(Configuration conf) {
        
        sha1=new SHA1();
        
        // user conf
        try {
            userConf=conf.getConfiguration(CONF_USERS);
        }
        catch (Exception e) {
            DebugManager.logOut(DP, "No user conf");
        }
                
        // own dss pub
        try {
            Configuration c=conf.getConfiguration(CONF_DSS_PUB);
            ownDSS=new DSS();
            ownDSS.setCommon(c);
            ownDSS.setPublic(c);
        }
        catch (Exception e) {
            DebugManager.logOut(DP, "No own dss pub");
        }
                        
        // own dss priv
        try {
            Configuration c=conf.getConfiguration(CONF_DSS_PRIV);
            if (ownDSS==null) {                
                ownDSS=new DSS();
            }
            ownDSS.setCommon(c);
            ownDSS.setPrivate(c);
        }
        catch (Exception e) {
            DebugManager.logOut(DP, "No own dss priv");
        }
        
        // own dss info
        try {
            Configuration c=conf.getConfiguration(CONF_DSS_INFO);
            ownInfo=c.getString("userName");
        }
        catch (Exception e) {
            DebugManager.logOut(DP, "No own dss info");
        }
        
        // server dss pub
        try {
            Configuration c=conf.getConfiguration(CONF_SERVER_DSS);
            serverDSS=new DSS();
            serverDSS.setCommon(c);
            serverDSS.setPublic(c);
        }
        catch (Exception e) {
            DebugManager.logOut(DP, "No server dss");
        }       
    }
    
    public DSS getUserPublic(String userName) {
        if (userConf!=null) {
            String hash=sha1.hash(userName);            
            DebugManager.logOut(DP, IO.print(hash));            
            String confFile=userConf.getString(IO.print(hash), "");
            DebugManager.logOut(DP, confFile);
            if (confFile.equals("")) {
                return null;
            }
            Configuration dssConf=new FileConfiguration(confFile);
            DSS dss=new DSS();
            dss.setCommon(dssConf);
            dss.setPublic(dssConf);
            return dss;
        }
        return null;                
    }
    
    public DSS getServerPublic() {        
        return serverDSS;
    }
    
    public DSS getOwn() {
        return ownDSS;
    }
        
    public String getOwnInfo() {
        return ownInfo;
    }

}
