package com.monica.javacom.ssh2;

import com.monica.javacom.ssh2.*;
import com.monica.javacom.interfaces.*;
import com.monica.interfaces.*;
import com.monica.javacom.transport.*;
import com.monica.javacom.ssh2.interfaces.*;

import com.monica.util.debug.*;

import java.util.*;

public class SSH2ServerSide implements ServerManager, Configureable, Runnable {
    
  private static final String DP="SSH2ServerSide";  

  private static final String NAME="SSH2ServerSide";
  private static final String VERSION="0.31";    
  
  private Transport transport;
  private SSH2Facade owner;
  private Vector servers;
  private SSH2ConfigurationManager ssh2configM;
  private Configuration configuration;
  private int destPort;
  private SSH2ChannelManager channelManager;
  private Object cParam;

  SSH2ServerSide(SSH2Facade owner, SSH2ChannelManager chManager, SSH2ConfigurationManager ssh2configM, Object cParam) {
    this.owner=owner;
    this.ssh2configM=ssh2configM;
    this.channelManager=chManager;
    this.cParam=cParam;
    servers=new Vector();
  }    

  public void startTransportLayer(int destPort) throws Exception {
    this.destPort=destPort;
    transport=new TCPTransport();
    transport.setServerManager(this);
    sTL_2();
    Thread t=new Thread(this);
    t.start();
  }
  
  public void run() {
      try {
        transport.initAsServer(destPort);      
      }
      catch (Exception e) {
          DebugManager.logException(DP, e);;
      }
  }
      
  private void sTL_2() throws Exception {
    if (configuration!=null) {
      transport.setConfiguration(configuration);
    }
  }    
  
  // --------------------------------------------------------------
  // Configureable
  // --------------------------------------------------------------
  
  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
  }

  public Configuration getConfiguration() {
    return configuration;
  }
    
  public void closeConnections() {
    try {
      int len=servers.size();
      for (int i=0; i<len; i++) {
          ((SSH2Server)servers.elementAt(i)).closeConnections();
      }
      transport.closeConnections();
    }
    catch (Exception e) {
    }
    owner.serverDone(this);
  }
  
  // --------------------------------------------------------------
  // ServerManager
  // --------------------------------------------------------------
    
  public String getServerManagerName() {
      return NAME;
  }
  
  public String getServerManagerVersion() {
      return VERSION;
  }
  
  public void start() {
  }
  
  public synchronized void newClientHasArrivedEvent(Transport server, Transport client) {
      SSH2Server newServer=new SSH2Server(client, owner, this, channelManager, cParam);      
      ssh2configM.enableClasses(newServer);
      newServer.setConfiguration(configuration);
      servers.add(newServer);      
      newServer.start();
  }
  
  public synchronized void unRegisterServer(SSH2Server server) {
      servers.removeElement(server);
  }
    
}

