package com.monica.javacom.ssh2;

import com.monica.javacom.interfaces.*;
import com.monica.util.hash.*;
import com.monica.interfaces.*;
import com.monica.javacom.ssh2.*;
import com.monica.util.*;
import com.monica.javacom.ssh2.cipher.*;
import com.monica.javacom.ssh2.compression.*;
import com.monica.javacom.ssh2.mac.*;
import com.monica.util.configuration.*;
import com.monica.util.hash.*;
import com.monica.javacom.ssh2.interfaces.*;
//import com.monica.javacom.ssh2.cipher.dss.*;
import com.monica.util.ccdm.dss.*;
import com.monica.util.debug.*;

import java.math.*;
import java.util.*;
import java.io.*;

public class SSH2TransportLayer implements Configureable, SSH2Layer {

  // *******************************************************************************************
  // Constants
  // *******************************************************************************************

  private static final String DP="SSH2TransportLayer";
    
  private static final String CONF_KEV_X_KEYBITS="KEV_X_KEYBITS";
  private static final int CONF_DEF_KEV_X_KEYBITS=1024;

  private static final String METHODNAME="ssh-transport";

  private static final String NAME="SSH2TransportLayer";
  private static final String VERSION="0.31";

  // *******************************************************************************************
  // Variables
  // *******************************************************************************************

  private boolean wasInitSSH;
  private int status;

  private SSH2Tokenizer tokenizer;
  private SSH2IO io;
  private Random rnd;
  private Configuration configuration;
  private SSH2BothSideFeatures owner;
  private Configuration ssh2Configuration;
  private int side;
  private boolean performServerCheck;

  // Protocol negotiation

  // possible values
  String PP_KA, PP_SHKA, PP_CI_CTS, PP_CI_STC, PP_IN_CTS, PP_IN_STC, PP_CO_CTS, PP_CO_STC, PP_LA_CTS, PP_LA_STC;

  String PN_KA; // negotiated kex algorithm
  String PN_SHKA; // negotiated server host key algorithm
  String PN_CI_CTS; // negotiated cipher suite client to server
  String PN_CI_STC; // negotiated cipher suite server to client
  String PN_IN_CTS; // negotiated integrity suite client to server
  String PN_IN_STC; // negotiated integrity suite server to client
  String PN_CO_CTS; // negotiated compression suite client to server
  String PN_CO_STC; // negotiated compression suite server to client
  String PN_LA_CTS; // negotiated language suite client to server
  String PN_LA_STC; // negotiated compression suite server to client

  // Key Exchange Variables

  BigInteger KEV_p, KEV_q, KEV_g, KEV_e, KEV_f, KEV_x, KEV_y, KEV_K;
  String KEV_V_C, KEV_V_S, KEV_I_C, KEV_I_S, KEV_K_S, KEV_H;
  
  // DSS
  private DSS my_dss, p_dss;
  private DSS_common my_dss_common, p_dss_common;
  private DSS_public my_dss_public, p_dss_public;
  private DSS_private my_dss_private, p_dss_private;
  private SSHDSS sshdss;

  String OKE_IV_CTS, OKE_IV_STC, OKE_EK_CTS, OKE_EK_STC, OKE_IK_CTS, OKE_IK_STC; // output from keyexchange

  String sessionID;
  
  private boolean wasSTCnewKeys, wasCTSnewKeys; 

  // Algorithms

  private HashAlgorithm hash;
  private SHA1 sha1;

  // *******************************************************************************************
  // Initialization
  // *******************************************************************************************

  // Constructor

  public SSH2TransportLayer() {
    wasInitSSH=false;

    io=new SSH2IO();
    rnd=new Random();

    // init p, g, q for diffie-hellman-group1-sha1

    String temp=new String();
    temp=temp.concat(new String("179769313486231590770839156793787453197860296048756011706444"));
    temp=temp.concat(new String("423684197180216158519368947833795864925541502180565485980503"));
    temp=temp.concat(new String("646440548199239100050792877003355816639229553136239076508735"));
    temp=temp.concat(new String("759914822574862575007425302077447712589550957937778424442426"));
    temp=temp.concat(new String("617334727629299387668709205606050270810842907692932019128194"));
    temp=temp.concat(new String("467627007"));

    KEV_p=new BigInteger(temp);
    KEV_g=new BigInteger("2");
    KEV_q=(KEV_p.subtract(new BigInteger("1"))).divide(KEV_g);
    
    wasSTCnewKeys=false;
    wasCTSnewKeys=false;

    sha1=new SHA1();

    OKE_IV_CTS=new String();
    OKE_IV_STC=new String();
    OKE_EK_CTS=new String();
    OKE_EK_STC=new String();
    OKE_IK_CTS=new String();
    OKE_IK_STC=new String();
    
    performServerCheck=false;
  }

  // *******************************************************************************************
  // Closeable
  // *******************************************************************************************

  public String getCloseableName() {
    return NAME;
  }

  public String getCloseableVersion() {
    return VERSION;
  }

  public void closeConnections() {
    if (tokenizer!=null) {
      tokenizer.closeConnections();
    }
  }

  // *******************************************************************************************
  // SSH2Layer
  // *******************************************************************************************

  public void setTokenizer(SSH2Tokenizer tokenizer) {
    this.tokenizer=tokenizer;
    tokenizer.setStatus(SSH2Constants.STAT_INIT);
  }

  public SSH2Tokenizer getTokenizer() {
    return tokenizer;
  }

  public void setSide(int side) {
    this.side=side;
  }

  public int getSide() {
    return side;
  }

  public String methodName() {
    return METHODNAME;
  }

  public void setParam(Object o) {
  }

  public void setOwner(SSH2BothSideFeatures owner) {
    this.owner=owner;
  }

  public SSH2BothSideFeatures getOwner() {
    return owner;
  }

  // *******************************************************************************************
  // Configureable
  // *******************************************************************************************

  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
    rereadConfig();
  }

  public Configuration getConfiguration() {
    return configuration;
  }

  private void rereadConfig() {
      try {
        ssh2Configuration=new InputStreamConfiguration(configuration.getStream("JavaCom.ssh2"));
      }
      catch (Exception e) {
      }
  }

  // *******************************************************************************************
  // Main Message Processing
  // *******************************************************************************************

  public String processInitMsg(String msg) {

    String cur_msg;

    while (msg.length()>0) {

      cur_msg=io.getNextToken(msg, SSH2Constants.SEP_CRLF);
//      DebugManager.logOut(DP, "processing: "+cur_msg+" side="+side);
      
      switch (side) {
          
        case SSH2Constants.SSH_SIDE_CLIENT :

          if ((cur_msg.length()>2) && (cur_msg.substring(0,3).equals("SSH"))) {
            KEV_V_S=cur_msg;
            wasInitSSH=true;
            sendVersionString();
            msg=io.delNextToken(msg, SSH2Constants.SEP_CRLF);
          }
          else {
            if (wasInitSSH) {
              tokenizer.setStatus(SSH2Constants.STAT_SSH);
              return msg;
            }
            else {
              DebugManager.logOut(DP, cur_msg);
            }
          }
          
          break;
        
        case SSH2Constants.SSH_SIDE_SERVER :  
          if ((cur_msg.length()>2) && (cur_msg.substring(0,3).equals("SSH"))) {
            KEV_V_C=cur_msg;
            wasInitSSH=true;
            sendKexInitMsg();
            msg=io.delNextToken(msg, SSH2Constants.SEP_CRLF);
          }
          else {
            if (wasInitSSH) {
              tokenizer.setStatus(SSH2Constants.STAT_SSH);
              return msg;
            }
            else {
                // TODO
              closeConnections();  
              //DebugManager.logOut(DP, cur_msg);
              //msg=io.delNextToken(msg, SSH2Constants.SEP_CRLF);
            }
          }
          break;
      }    
    }
    return null;
  }

  // *******************************************************************************************
  // Transport protocol messages
  // *******************************************************************************************

  /** Processes the KEX_INIT message received from the SSH2 server.
      @param SSHTransportPacket pack: the KEX_INIT message
  */
  public void processKexInitMsg(SSH2Packet pack) throws Exception {
      
      switch (side) {
          case SSH2Constants.SSH_SIDE_CLIENT :
              _c_processKexInitMsg(pack);
              break;
          case SSH2Constants.SSH_SIDE_SERVER :
              _s_processKexInitMsg(pack);
              break;
      }
  }
              
  private void _s_processKexInitMsg(SSH2Packet pack) throws Exception {      
    _createChosenSuite(pack);
    KEV_I_C=pack.payload;
  }
  
  private void _createChosenSuite(SSH2Packet pack) throws Exception {
    Vector data;
    String kA, sHKA, eACTS, eASTC, mACTS, mASTC, cACTS, cASTC, lCTS, lSTC;
    String s_kA, s_sHKA, s_eACTS, s_eASTC, s_mACTS, s_mASTC, s_cACTS, s_cASTC, s_lCTS, s_lSTC;
    Boolean fKPF;
      
    data=io.parsePayload(pack.payload,SSH2Constants.KEX_INIT_MSG_DATA_DESC);

    kA=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_KEX_ALGORITHMS));
    sHKA=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_SERVER_HOST_KEY_ALGORITHMS));
    eACTS=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_ENCRYPTION_ALGORITHMS_CLIENT_TO_SERVER));
    eASTC=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_ENCRYPTION_ALGORITHMS_SERVER_TO_CLIENT));
    mACTS=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_MAC_ALGORITHMS_CLIENT_TO_SERVER));
    mASTC=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_MAC_ALGORITHMS_SERVER_TO_CLIENT));
    cACTS=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_COMPRESSION_ALGORITHMS_CLIENT_TO_SERVER));
    cASTC=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_COMPRESSION_ALGORITHMS_SERVER_TO_CLIENT));
    lCTS=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_LANGUAGES_CLIENT_TO_SERVER));
    lSTC=(String)(data.elementAt(SSH2Constants.DI_KEXINIT_LANGUAGES_SERVER_TO_CLIENT));
    fKPF=(Boolean)(data.elementAt(SSH2Constants.DI_KEXINIT_FIRST_KEY_PACKET_FOLLOWS));
    
    PP_KA="diffie-hellman-group1-sha1";
    PP_SHKA="ssh-dss";
    PP_CI_CTS=owner.getFeatureList(SSH2Facade.OPT_CIPHER_ALGORITHM_CTS);
    PP_CI_STC=owner.getFeatureList(SSH2Facade.OPT_CIPHER_ALGORITHM_STC);
    PP_IN_CTS=owner.getFeatureList(SSH2Facade.OPT_MAC_ALGORITHM_CTS);
    PP_IN_STC=owner.getFeatureList(SSH2Facade.OPT_MAC_ALGORITHM_STC);
    PP_CO_CTS=owner.getFeatureList(SSH2Facade.OPT_COMPRESSION_ALGORITHM_CTS);
    PP_CO_STC=owner.getFeatureList(SSH2Facade.OPT_COMPRESSION_ALGORITHM_STC);
    PP_LA_CTS="";
    PP_LA_STC="";
    
    DebugManager.logOut(DP, "KA="+PP_KA+"; "+kA);
    DebugManager.logOut(DP, "SHKA="+PP_SHKA+"; "+sHKA);
    DebugManager.logOut(DP, "CI_CTS="+PP_CI_CTS+"; "+eACTS);
    DebugManager.logOut(DP, "CI_STC="+PP_CI_STC+"; "+eASTC);
    DebugManager.logOut(DP, "IN_CTS="+PP_IN_CTS+"; "+mACTS);
    DebugManager.logOut(DP, "IN_STC="+PP_IN_STC+"; "+mASTC);
    DebugManager.logOut(DP, "CO_CTS="+PP_CO_CTS+"; "+cACTS);
    DebugManager.logOut(DP, "CO_STC="+PP_CO_STC+"; "+cASTC);
    DebugManager.logOut(DP, "LA_CTS="+PP_LA_CTS+"; "+lCTS);
    DebugManager.logOut(DP, "LA_STC="+PP_LA_STC+"; "+lSTC);    

/*    s_kA=io.findMatch(kA, PP_KA, 2);
    s_sHKA=io.findMatch(sHKA, PP_SHKA, 2);
    s_eACTS=io.findMatch(eACTS, PP_CI_CTS, 2);
    s_eASTC=io.findMatch(eASTC, PP_CI_STC, 2);
    s_mACTS=io.findMatch(mACTS, PP_IN_CTS, 2);
    s_mASTC=io.findMatch(mASTC, PP_IN_STC, 2);
    s_cACTS=io.findMatch(cACTS, PP_CO_CTS, 2);
    s_cASTC=io.findMatch(cASTC, PP_CO_STC, 2);
    s_lCTS=io.findMatch(lCTS, PP_LA_CTS, 2);
    s_lSTC=io.findMatch(lSTC, PP_LA_STC, 2); */

    s_kA=io.findMatch(kA, PP_KA, 1);
    s_sHKA=io.findMatch(sHKA, PP_SHKA, 1);
    s_eACTS=io.findMatch(eACTS, PP_CI_CTS, 1);
    s_eASTC=io.findMatch(eASTC, PP_CI_STC, 1);
    s_mACTS=io.findMatch(mACTS, PP_IN_CTS, 1);
    s_mASTC=io.findMatch(mASTC, PP_IN_STC, 1);
    s_cACTS=io.findMatch(cACTS, PP_CO_CTS, 1);
    s_cASTC=io.findMatch(cASTC, PP_CO_STC, 1);
    s_lCTS=io.findMatch(lCTS, PP_LA_CTS, 1);
    s_lSTC=io.findMatch(lSTC, PP_LA_STC, 1); 
        
    PN_KA=io.chosenMatch(s_kA);
    PN_SHKA=io.chosenMatch(s_sHKA);
    PN_CI_CTS=io.chosenMatch(s_eACTS);
    PN_CI_STC=io.chosenMatch(s_eASTC);
    PN_CO_CTS=io.chosenMatch(s_cACTS);
    PN_CO_STC=io.chosenMatch(s_cASTC);
    PN_IN_CTS=io.chosenMatch(s_mACTS);
    PN_IN_STC=io.chosenMatch(s_mASTC);
    try {
      PN_LA_CTS=io.chosenMatch(s_lCTS);
      PN_LA_STC=io.chosenMatch(s_lSTC);    
    }
    catch (Exception e) {
      PN_LA_CTS=new String();
      PN_LA_STC=new String();
    }
  }  
  
  private void _c_processKexInitMsg(SSH2Packet pack) {      
    String reply_payload;
    KEV_I_S=pack.payload;
    byte b[]=new byte[16];
    rnd.nextBytes(b);
    String cookie=new String(b);

    try {
      _createChosenSuite(pack);          
    }
    catch (Exception e) {
      DebugManager.logOut(DP, "Could not negotiate protocol algorithms. Exception="+e.toString());
      DebugManager.logException(DP, e);
      closeConnections();
      return;
    }

    reply_payload=new String();
    reply_payload=reply_payload.concat(io.wByte(SSH2Constants.SSH_TRANSPORT_MSG_KEXINIT));    
    reply_payload=reply_payload.concat(cookie);
    reply_payload=reply_payload.concat(io.wString(PN_KA));
    reply_payload=reply_payload.concat(io.wString(PN_SHKA));
    reply_payload=reply_payload.concat(io.wString(PN_CI_CTS));
    reply_payload=reply_payload.concat(io.wString(PN_CI_STC));
    reply_payload=reply_payload.concat(io.wString(PN_IN_CTS));
    reply_payload=reply_payload.concat(io.wString(PN_IN_STC));
    reply_payload=reply_payload.concat(io.wString(PN_CO_CTS));
    reply_payload=reply_payload.concat(io.wString(PN_CO_STC));
    reply_payload=reply_payload.concat(io.wString(PN_LA_CTS));
    reply_payload=reply_payload.concat(io.wString(PN_LA_STC));
    reply_payload=reply_payload.concat(io.wBoolean(true));
    reply_payload=reply_payload.concat(io.wUint32(0));

    KEV_I_C=reply_payload;

    tokenizer.sendP(reply_payload);

    sendKexDHInitMsg();
  }

  /** Client side KEXDH_INIT message creation.
  */
  private void sendKexDHInitMsg() {
  
    if (side!=SSH2Constants.SSH_SIDE_CLIENT) {
      return;
    }

    KEV_x=new BigInteger(ssh2Configuration.getInt(CONF_KEV_X_KEYBITS, CONF_DEF_KEV_X_KEYBITS),rnd);
    KEV_e=KEV_g.modPow(KEV_x, KEV_p);

    String kinit=io.wByte(SSH2Constants.SSH_TRANSPORT_MSG_KEXDH_INIT);
    kinit=kinit.concat(io.wMpint(KEV_e));

    tokenizer.sendP(kinit);
  }
  
  public void processKexDHInitMsg(SSH2Packet pack) throws Exception {
      
      if (side!=SSH2Constants.SSH_SIDE_SERVER) {
        tokenizer.sendUnimplementedMsg(pack);
        return;
      }
      
      KEV_e=io.rMpint(pack.payload.substring(1));
      
      String signature=null;
      
      SSH2KeyManager keyManager=new SSH2KeyManager(configuration);      
      my_dss=keyManager.getOwn();
      
      while (true) {
        KEV_y=new BigInteger(ssh2Configuration.getInt(CONF_KEV_X_KEYBITS, CONF_DEF_KEV_X_KEYBITS),rnd);
        KEV_f=KEV_g.modPow(KEV_y, KEV_p);
        KEV_K=KEV_e.modPow(KEV_y, KEV_p);
                        
        sshdss=new SSHDSS();
        sshdss.setDSS(my_dss);
        KEV_K_S=sshdss.ssh2_pubKey();
            
        String concatenated=new String(io.wString(KEV_V_C));
        concatenated=concatenated.concat(io.wString(KEV_V_S));
        concatenated=concatenated.concat(io.wString(KEV_I_C));
        concatenated=concatenated.concat(io.wString(KEV_I_S));
        concatenated=concatenated.concat(io.wString(KEV_K_S));
        concatenated=concatenated.concat(io.wMpint(KEV_e));
        concatenated=concatenated.concat(io.wMpint(KEV_f));
        concatenated=concatenated.concat(io.wMpint(KEV_K));
      
        KEV_H=sha1.hash(concatenated);
        
        signature=sshdss.ssh2_sign(KEV_H);
        
        if (signature.length()==55) {
//            DebugManager.logOut(DP, "sig="+signature);
            break;
        }      
      }
                        
      String reply=io.wByte(SSH2Constants.SSH_TRANSPORT_MSG_KEXDH_REPLY);
      reply+=io.wString(KEV_K_S);
      reply+=io.wMpint(KEV_f);
      reply+=io.wString(signature);
      
      tokenizer.sendP(reply);
      
      setupSuite();     
      
      if (!wasCTSnewKeys) {
        sendNewKeysMsg();
      }	 
  }

  /** Processes the KEXDH_REPLY message from the SSH2 server.
      @param SSHTransportPacket pack: the KEX_INIT message
  */
  public void processKexDHReplyMsg(SSH2Packet pack) throws Exception {
  
    if (side!=SSH2Constants.SSH_SIDE_CLIENT) {
        tokenizer.sendUnimplementedMsg(pack);
        return;
    }
  
    Vector data;
    String signature;

    // KEXDH_REPLY

    data=io.parsePayload(pack.payload,SSH2Constants.KEXDH_REPLY_MSG_DATA_DESC);

    KEV_K_S=(String)(data.elementAt(SSH2Constants.DI_KEXDH_REPLY_K_S));
//    DebugManager.logOut(DP, KEV_K_S);
    KEV_f=(BigInteger)(data.elementAt(SSH2Constants.DI_KEXDH_REPLY_F));
    signature=(String)(data.elementAt(SSH2Constants.DI_KEXDH_REPLY_SIGNATURE));

    String keyFormat=io.rString(KEV_K_S);

    if (!keyFormat.equals(PN_SHKA)) {
      DebugManager.logOut(DP, "Key format not the same.");
      closeConnections();
      return;
    }

    if (keyFormat.equals("ssh-dss")) {
      processSSHDSSKexDHReply(signature);
    }
    else {
      tokenizer.sendUnimplementedMsg(pack);
      closeConnections();
    }
  }

  /** Processes SSH-DSS key format.
      @param String signature: the signature for the DSS check
  */
  public void processSSHDSSKexDHReply(String signature) throws Exception {
      
      // Initialization
      
      sshdss=new SSHDSS();
      sshdss.ssh2_pubKey(KEV_K_S);
      
      if (performServerCheck) {
          
          SSH2KeyManager keyManager=new SSH2KeyManager(configuration);
          p_dss=keyManager.getServerPublic();
          
          SSHDSS p_sshdss=new SSHDSS();
          p_sshdss.setDSS(p_dss);
          if (!KEV_K_S.equals(p_sshdss.ssh2_pubKey())) {
              DebugManager.logOut(DP, "Server key mismatch.");
              throw new Exception("Server key mismatch");
          }
      }
      
      // Computation

      KEV_K=KEV_f.modPow(KEV_x, KEV_p);

      // DSS Verification

      String concatenated=new String(io.wString(KEV_V_C));
      concatenated=concatenated.concat(io.wString(KEV_V_S));
      concatenated=concatenated.concat(io.wString(KEV_I_C));
      concatenated=concatenated.concat(io.wString(KEV_I_S));
      concatenated=concatenated.concat(io.wString(KEV_K_S));
      concatenated=concatenated.concat(io.wMpint(KEV_e));
      concatenated=concatenated.concat(io.wMpint(KEV_f));
      concatenated=concatenated.concat(io.wMpint(KEV_K));

      KEV_H=sha1.hash(concatenated);
      
      // Verification
      
      if (sshdss.ssh2_verify(KEV_H, signature)) {
          DebugManager.logOut(DP, "Host verified.");
          try {          
              setupSuite();              
              sendNewKeysMsg();
          }
          catch (Exception e) {
              DebugManager.logOut(DP, "Failed at key computation. Exception="+e.toString());
              closeConnections();
          }
      }
      else {
          DebugManager.logOut(DP, "Host verification failed.");
          closeConnections();
      }
  }
  
  private void setupSuite() throws Exception {
      Object ciO_CTS, ciO_STC, maO_CTS, maO_STC;
      String temp=(io.wMpint(KEV_K)).concat(KEV_H);
      
      sessionID=KEV_H;
            
      ciO_CTS=owner.getFeatureParam(SSH2Facade.OPT_CIPHER_ALGORITHM_CTS, PN_CI_CTS);
      ciO_STC=owner.getFeatureParam(SSH2Facade.OPT_CIPHER_ALGORITHM_STC, PN_CI_STC);

      maO_CTS=owner.getFeatureParam(SSH2Facade.OPT_MAC_ALGORITHM_CTS, PN_IN_CTS);
      maO_STC=owner.getFeatureParam(SSH2Facade.OPT_MAC_ALGORITHM_STC, PN_IN_STC);

      int IVCTSLen=((int[])ciO_CTS)[SSH2Constants.CONF_CI_IV_POS];
      int IVSTCLen=((int[])ciO_STC)[SSH2Constants.CONF_CI_IV_POS];
      int CiCTSLen=((int[])ciO_CTS)[SSH2Constants.CONF_CI_CI_POS];
      int CiSTCLen=((int[])ciO_STC)[SSH2Constants.CONF_CI_CI_POS];
      int MACCTSLen=((int[])maO_CTS)[SSH2Constants.CONF_MA_LEN_POS];
      int MACSTCLen=((int[])maO_STC)[SSH2Constants.CONF_MA_LEN_POS];

      OKE_IV_CTS=createKeyData(temp, "A", sessionID, IVCTSLen, sha1);
      OKE_IV_STC=createKeyData(temp, "B", sessionID, IVSTCLen, sha1);
      OKE_EK_CTS=createKeyData(temp, "C", sessionID, CiCTSLen, sha1);
      OKE_EK_STC=createKeyData(temp, "D", sessionID, CiSTCLen, sha1);
      OKE_IK_CTS=createKeyData(temp, "E", sessionID, MACCTSLen, sha1);
      OKE_IK_STC=createKeyData(temp, "F", sessionID, MACSTCLen, sha1);

      DebugManager.logOut(DP, "A="+io.print(OKE_IV_CTS));
      DebugManager.logOut(DP, "B="+io.print(OKE_IV_STC));
      DebugManager.logOut(DP, "C="+io.print(OKE_EK_CTS));
      DebugManager.logOut(DP, "D="+io.print(OKE_EK_STC));
      DebugManager.logOut(DP, "E="+io.print(OKE_IK_CTS));
      DebugManager.logOut(DP, "F="+io.print(OKE_IK_STC));     
  }

  /** Create arbitrary length key data as described in the SSH2 key data creation
      algorithm.
      @param String firstPart: the first, constant part of the iteration
      @param String variant: variant of the first round, e.g. "A"
      @param String session: the session ID
      @param int numBytes: how many bytes should the key data contain
      @param SSHHashAlgorithm: the HASH algorithm to be used for creation of the key data
      @return String: the final key data result
  */
  private String createKeyData(String firstPart, String variant, String session, int numBytes,
                               HashAlgorithm h) {
    String res;
    if (numBytes==0) {
      return new String();
    }
    res=h.hash(firstPart.concat(variant.concat(session)));
    while (res.length()<numBytes) {
      res=res.concat(h.hash(firstPart.concat(res)));
    }
    return res.substring(0,numBytes);
  }

  /** Sends the NEWKEYS message to the server.
  */
  private void sendNewKeysMsg() throws Exception {
  
    DebugManager.logOut(DP, "Sending NEWKEYS Msg.");
    
    String newKeys;  
      
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :  
  
        newKeys=io.wByte(SSH2Constants.SSH_TRANSPORT_MSG_NEWKEYS);

        synchronized (this) {
          tokenizer.sendP(newKeys);       
          wasCTSnewKeys=true;
          try {    
            _activateKeys(SSH2Constants.SSH_TOKENIZER_CTS);    
            DebugManager.logOut(DP, "CTS Keys active");
          }
          catch (Exception e) {
            DebugManager.logOut(DP, "Failed CTS suite setup. Exception="+e.toString());
            closeConnections();
          }
        }
        break;
        
      case SSH2Constants.SSH_SIDE_SERVER :  
  
        newKeys=io.wByte(SSH2Constants.SSH_TRANSPORT_MSG_NEWKEYS);

        synchronized (this) {
          tokenizer.sendP(newKeys);       
          wasSTCnewKeys=true;    
          try {    
            _activateKeys(SSH2Constants.SSH_TOKENIZER_STC);	    
            DebugManager.logOut(DP, "STC Keys active");
	    if ((wasCTSnewKeys) && (wasSTCnewKeys)) {
              tokenizer.transportProtocolReady();
	    }  
          }
          catch (Exception e) {
            DebugManager.logOut(DP, "Failed STC suite setup. Exception="+e.toString());
            closeConnections();
          }
        }
        break;
    }
  }

  /** Processes the NEWKEYS message from the server.
      @param SSHTransportPacket pack: the NEWKEYS message from the server
  */
  public void processNewKeysMsg(SSH2Packet pack) throws Exception {
      
    switch (side) {
        case SSH2Constants.SSH_SIDE_CLIENT :
            _c_processNewKeysMsg(pack);
            break;
        case SSH2Constants.SSH_SIDE_SERVER :
            _s_processNewKeysMsg(pack);
            break;
    }
  }
  
  private void _c_processNewKeysMsg(SSH2Packet pack) {
    try {        
      _activateKeys(SSH2Constants.SSH_TOKENIZER_STC);      
      DebugManager.logOut(DP, "STC Keys active");
      tokenizer.transportProtocolReady();
    }
    catch (Exception e) {
      DebugManager.logOut(DP, "Failed STC suite setup. Exception="+e.toString());
      closeConnections();
    }
  }
  
  private void _s_processNewKeysMsg(SSH2Packet pack) {
    try {
      _activateKeys(SSH2Constants.SSH_TOKENIZER_CTS);      
      wasCTSnewKeys=true;
      DebugManager.logOut(DP, "CTS Keys active");
      if (!wasSTCnewKeys) {
        sendNewKeysMsg();      
      }
      else {
        tokenizer.transportProtocolReady();
      }		
    }
    catch (Exception e) {
      DebugManager.logOut(DP, "Failed CTS suite setup. Exception="+e.toString());
      closeConnections();
    }
  }
  
  private void _activateKeys(boolean direction) throws Exception {
              
      if (direction==SSH2Constants.SSH_TOKENIZER_STC) {  
          
        tokenizer.setKeys(OKE_IV_STC, OKE_EK_STC, OKE_IK_STC, SSH2Constants.SSH_TOKENIZER_STC);  
  
        DebugManager.logOut(DP, "Using "+PN_IN_STC+" as MAC for STC.");
        tokenizer.setMACAlgorithm((MACAlgorithm)(
          owner.getFeature(SSH2Facade.OPT_MAC_ALGORITHM_STC, PN_IN_STC)),
          SSH2Constants.SSH_TOKENIZER_STC);

        DebugManager.logOut(DP, "Using "+PN_CI_STC+" as cipher for STC.");
        tokenizer.setCipherAlgorithm((CipherAlgorithm)(
          owner.getFeature(SSH2Facade.OPT_CIPHER_ALGORITHM_STC, PN_CI_STC)),
          SSH2Constants.SSH_TOKENIZER_STC);

        DebugManager.logOut(DP, "Using "+PN_CO_STC+" as compression for STC.");
        tokenizer.setCompressionAlgorithm((CompressionAlgorithm)(
          owner.getFeature(SSH2Facade.OPT_COMPRESSION_ALGORITHM_STC, PN_CO_STC)),
          SSH2Constants.SSH_TOKENIZER_STC);
      }
      else if (direction==SSH2Constants.SSH_TOKENIZER_CTS) {
        
        tokenizer.setKeys(OKE_IV_CTS, OKE_EK_CTS, OKE_IK_CTS, SSH2Constants.SSH_TOKENIZER_CTS);

        DebugManager.logOut(DP, "Using "+PN_IN_CTS+" as MAC for CTS.");
        tokenizer.setMACAlgorithm((MACAlgorithm)(
          owner.getFeature(SSH2Facade.OPT_MAC_ALGORITHM_CTS, PN_IN_CTS)),
          SSH2Constants.SSH_TOKENIZER_CTS);

        DebugManager.logOut(DP, "Using "+PN_CI_CTS+" as cipher for CTS.");
        tokenizer.setCipherAlgorithm((CipherAlgorithm)(
          owner.getFeature(SSH2Facade.OPT_CIPHER_ALGORITHM_CTS, PN_CI_CTS)),
	        SSH2Constants.SSH_TOKENIZER_CTS);

        DebugManager.logOut(DP, "Using "+PN_CO_CTS+" as compression for CTS.");
        tokenizer.setCompressionAlgorithm((CompressionAlgorithm)(
          owner.getFeature(SSH2Facade.OPT_COMPRESSION_ALGORITHM_CTS, PN_CO_CTS)),
	        SSH2Constants.SSH_TOKENIZER_CTS);
        
      }
  }

  public void processDisconnectMsg(SSH2Packet pack) {
    DebugManager.logOut(DP, "Disconnected. Reason="+io.rString((pack.payload).substring(5))+SSH2Constants.CRLF);
  }

  public void sendVersionString() {
      switch (side) {
          case SSH2Constants.SSH_SIDE_CLIENT :
              KEV_V_C=SSH2Constants.SSH_VERSION_STRING;
              break;
          case SSH2Constants.SSH_SIDE_SERVER :
              KEV_V_S=SSH2Constants.SSH_VERSION_STRING;
              break;
      }
                  
      tokenizer.sendNP(SSH2Constants.SSH_VERSION_STRING+SSH2Constants.CRLF);    
  }
  
  public void sendKexInitMsg() {
      
    try {
      
      String payload=io.wByte(SSH2Constants.SSH_TRANSPORT_MSG_KEXINIT);
      payload+=io.wUint32(rnd.nextInt());
      payload+=io.wUint32(rnd.nextInt());
      payload+=io.wUint32(rnd.nextInt());
      payload+=io.wUint32(rnd.nextInt());
      payload+=io.wString("diffie-hellman-group1-sha1");
      payload+=io.wString("ssh-dss");
      payload+=io.wString(owner.getFeatureList(SSH2Facade.OPT_CIPHER_ALGORITHM_CTS));
      payload+=io.wString(owner.getFeatureList(SSH2Facade.OPT_CIPHER_ALGORITHM_STC));
      payload+=io.wString(owner.getFeatureList(SSH2Facade.OPT_MAC_ALGORITHM_CTS));
      payload+=io.wString(owner.getFeatureList(SSH2Facade.OPT_MAC_ALGORITHM_STC));
      payload+=io.wString(owner.getFeatureList(SSH2Facade.OPT_COMPRESSION_ALGORITHM_CTS));
      payload+=io.wString(owner.getFeatureList(SSH2Facade.OPT_COMPRESSION_ALGORITHM_STC));
      payload+=io.wString("");
      payload+=io.wString("");
      payload+=io.wBoolean(false);
      payload+=io.wUint32(0);
      
      KEV_I_S=payload;

      tokenizer.sendP(payload);
    }
    catch (Exception e) {
      DebugManager.logOut(DP, "Sending possibilities. Exception="+e.toString());
      closeConnections();        
    }

  }
  
  public String getSessionID() {
      return sessionID;
  }
  
  public void performServerCheck() {
      performServerCheck=true;
  }
  
  public String getInformation(int opt) {
      switch (opt) {
          case SSH2Constants.INFO_CIPHER_CTS :
              return PN_CI_CTS;
          case SSH2Constants.INFO_CIPHER_STC :
              return PN_CI_STC;
          case SSH2Constants.INFO_MAC_CTS :
              return PN_IN_CTS;
          case SSH2Constants.INFO_MAC_STC :
              return PN_IN_STC;
          case SSH2Constants.INFO_COMPRESSION_CTS :
              return PN_CO_CTS;
          case SSH2Constants.INFO_COMPRESSION_STC :
              return PN_CO_STC;
          default :
              return null;
      }
  }
}

