package com.monica.javacom.ssh2.cipher;

import com.monica.util.*;
import com.monica.javacom.interfaces.*;
import com.monica.javacom.ssh2.*;
import com.monica.interfaces.*;

/** Class for implementing the ARCFOUR stream cipher as defined in
    draft-kaukonen-cipher-arcfour.
*/
public class SSH2ArcfourCipher implements CipherAlgorithm {

  // ****************************************************************************************
  // Inner Class
  // ****************************************************************************************

  class SSH2ArcfourCipherState {
    private char i_i;
    private char i_j;
    private char i_S[];
    private int i_fi;
    SSH2ArcfourCipherState(char i_ii, char i_jj, char i_SS[]) {
      i_i=i_ii;
      i_j=i_jj;
      i_S=new char[i_SS.length];
      for (i_fi=0; i_fi<i_SS.length; i_fi++) i_S[i_fi]=i_SS[i_fi];
    }
    char getI() {
      return i_i;
    }
    char getJ() {
      return i_j;
    }
    char[] getS() {
      char i_temp[]=new char[i_S.length];
      for (i_fi=0; i_fi<i_S.length; i_fi++) i_temp[i_fi]=i_S[i_fi];
      return i_temp;
    }
  }

  // ****************************************************************************************
  // Constants
  // ****************************************************************************************

  private static final String NAME="SSHArcfourCipher";
  private static final String VERSION="0.2";

  private char[] S;
  private char i;
  private char j;
  private int ii;
  private char temp, K;

  public SSH2ArcfourCipher() {
    S=new char[256];
    for (ii=0; ii<256; ii++) {
      S[ii]=(char)ii;
    }
  }

  public String getCipherAlgorithmName() {
    return NAME;
  }

  public String getCipherAlgorithmVersion() {
    return VERSION;
  }

  public void setKeys(String key, String IV) {
    char S2[]=new char[256];
    for (ii=0; ii<256; ii++) {
      S2[ii]=key.charAt(ii%key.length());
    }
    j=(char)0;
    for (i=0; i<256; i++) {
      j=(char)((j+S[i]+S2[i])%256);
      temp=S[i];
      S[i]=S[j];
      S[j]=temp;
    }
    i=(char)0;
    j=(char)0;
  }

  public void setState(Object o) {
    SSH2ArcfourCipherState state=(SSH2ArcfourCipherState)o;
    i=state.getI();
    j=state.getJ();
    S=state.getS();
  }

  public Object getState() {
    return new SSH2ArcfourCipherState(i,j,S);
  }

  public String encode(String data) {
    int f_len=data.length();
    char result[]=new char[f_len];    
    for (ii=0; ii<f_len; ii++) {
      i=(char)((i+1)%256);
      j=(char)((j+S[i])%256);
      temp=S[i];
      S[i]=S[j];
      S[j]=temp;
      temp=(char)((S[i]+S[j])%256);
      K=S[temp];
      result[ii]=(char)(K^data.charAt(ii));
    }
    return new String(result);
  }

  public String decode(String data) {
    int f_len=data.length();
    char result[]=new char[f_len];    
    for (ii=0; ii<f_len; ii++) {
      i=(char)((i+1)%256);
      j=(char)((j+S[i])%256);
      temp=S[i];
      S[i]=S[j];
      S[j]=temp;
      temp=(char)((S[i]+S[j])%256);
      K=S[temp];
      result[ii]=(char)(K^data.charAt(ii));
    }
    return new String(result);
  }

  public int getCipherBlockSize() {
    return 1;
  }
  
  public void setConfiguration(Configuration configuration) {
  }
  
  public Configuration getConfiguration() {
    return null;
  }    
}


