package com.monica.javacom.ssh2.connection;

import com.monica.javacom.ssh2.interfaces.*;
import com.monica.javacom.ssh2.*;
import com.monica.interfaces.*;
import com.monica.util.*;
import com.monica.util.debug.*;

import java.util.*;

public abstract class SSH2BasicChannel implements SSH2Channel {
    
  public static final int OPT_FLUSHING=2000;
  public static final int OPT_READY=2001;
    
  protected static final String NAME="SSH2BasicChannel";
  protected static final String VERSION="0.31";
  
  private static final String DP_S="SSH2BasicChannel";
  
  private static final int FLUSH_INTERN=1;
  private static final int FLUSH_EXTERN=2;
      
  protected int ownID;
  protected int partnerID;  
  protected Configuration configuration;  
  protected String rBuf;
  protected boolean connected;
  protected int ownWindowSize;
  protected int partnerWindowSize, partnerPacketSize;
  protected Reader reader;
  protected com.monica.util.StringBuffer wBuf;
  protected SSH2ChannelManager channelManager;
  protected SSH2ConnectionLayer connLayer;
  protected SSH2Tokenizer tokenizer;
  protected Object waiter;
  protected SSH2IO io;
  protected int ownMaxPacketSize;
  protected boolean flushing;
  
  private static final int BUF_SIZE=100000;
  private static final int PARTNER_WINDOW_SIZE=10000;
  private static final int PARTNER_MAX_PACKET_SIZE=10000;
  
  private static final String SSH2_CONF="JavaCom.ssh2";
  
  private Thread flushExternThread;
  
//  private Hashtable myInformation;
  
  public void _init(SSH2ConnectionLayer connLayer) {
    rBuf=new String();
    wBuf=new com.monica.util.StringBuffer(BUF_SIZE);
    this.connLayer=connLayer;
    ownID=-1;
    partnerID=-1;
    ownWindowSize=0;
    partnerWindowSize=PARTNER_WINDOW_SIZE;
    partnerPacketSize=PARTNER_MAX_PACKET_SIZE;
    io=new SSH2IO();
    flushing=false;
  }
  
  private void _setConnected() {
    if ((configuration!=null) &&
        (reader!=null) && 
        (tokenizer!=null) &&
        (ownID!=-1) && 
        (partnerID!=-1) &&
        (channelManager!=null)) {
      connected=true;
      return;
    }
    connected=false;
  }
        
  // -------------------------------------------------------------
  // SSH2Channel
  // -------------------------------------------------------------
    
  public void setOwnID(int ID) {
    ownID=ID;
    _setConnected();
  }
  
  public int getOwnID() {
    return ownID;
  }
  
  public void setPartnerID(int ID) {
    partnerID=ID;
    _setConnected();
  }
  
  public int getPartnerID() {
    return partnerID;
  }
     
  public String getInformation(int opt) {
    switch (opt) {
        case SSH2Constants.INFO_CHANNEL_NAME :
            return NAME;
        case SSH2Constants.INFO_CONNECTION_NAME :
            return connLayer.getConnectionLayerName();
        case OPT_FLUSHING :
            if ((flushing) || (wBuf.getBufferLength()>0)) {
                return "";
            }
            else {
                return null;
            }
        case OPT_READY : // TODO HACK
            if (partnerID!=-1) {
                return "";
            }
            else {
                return null;
            }
        default :
            return tokenizer.getInformation(opt);
    }
  }
  
  public void setInformation(int opt, String value) {
  }
  
  public void setTokenizer(SSH2Tokenizer tokenizer) {
    this.tokenizer=tokenizer;
    _setConnected();
  }
  
  public SSH2Tokenizer getTokenizer() {
    return tokenizer;  
  }
    
  public synchronized void modifyWindowSize(int value) {
    ownWindowSize+=value;  
    if ((flushing) || (wBuf.getBufferLength()>0)) {
      flushing=false;
      _flush(FLUSH_INTERN);
    }
  }
  
  public void data(String data) {
    rBuf+=data;
    _processData();
  }
  
  public void extendedData(String data) {
    rBuf+=data;  
    _processData();
  }
  
  private void _processData() {
    if (rBuf.length()>partnerWindowSize) {
      closeConnections();
    }
//    DebugManager.logOut(DP(), "Sending readevent...");
    reader.readEvent(this);
  }
    
  public void setChannelManager(SSH2ChannelManager chManager) {
    channelManager=chManager;  
    channelManager.channelArrived(this);
    _setConnected();    
  }
  
  public SSH2ChannelManager getChannelManager() {
    return channelManager;  
  }
  
  public int getPartnerWindowSize() {
    return partnerWindowSize;
  }
  
  public int getPartnerMaxPacketSize() {
    return partnerPacketSize;
  }
  
  public void setWindowSize(int value) {
    ownWindowSize=value;
  }
  
  public void setMaxPacketSize(int value) {
    ownMaxPacketSize=value;
  }
  
  public abstract void channelRequest(SSH2Packet pack);
  
  public abstract void channelSuccess(SSH2Packet pack);
  
  public abstract void channelFailure(SSH2Packet pack);
  
  public int request(String type, boolean wantReply, String other) {
    String toSend=io.wByte(SSH2Constants.SSH_CONNECTION_MSG_CHANNEL_REQUEST);
    toSend+=io.wUint32(partnerID);
    toSend+=io.wString(type);
    toSend+=io.wBoolean(wantReply);
    if (other!=null) {
      toSend+=other;
    }
    tokenizer.sendP(toSend);
    return SSH2Constants.RET_CHANNEL_REQUEST_UNKNOWN;
  }
  
      
  // -------------------------------------------------------------
  // LayerTopView
  // -------------------------------------------------------------
  
  public String getLayerTopViewName() {
    return NAME;
  }
  
  public String getLayerTopViewVersion() {
    return VERSION;
  }
  
  // -------------------------------------------------------------
  // Closeable
  // -------------------------------------------------------------
  
  public String getCloseableName() {
    return NAME;
  }
  
  public String getCloseableVersion() {
    return VERSION;
  }
  
  public void closeConnections() {
    
    DebugManager.logOut(DP(), "Closing channel.");  
      
    connected=false;
    String toSend=io.wByte(SSH2Constants.SSH_CONNECTION_MSG_CHANNEL_EOF);
    toSend+=io.wUint32(partnerID);
    tokenizer.sendP(toSend);
    toSend=io.wByte(SSH2Constants.SSH_CONNECTION_MSG_CHANNEL_CLOSE);
    toSend+=io.wUint32(partnerID);
    tokenizer.sendP(toSend);    
    
    channelManager.channelClosed(this);
    connLayer.channelClosed(this);        
  }
  
  // -------------------------------------------------------------
  // Readable
  // -------------------------------------------------------------

  public String getReadableName() {
    return NAME;
  }
  
  public String getReadableVersion() {
    return VERSION;
  }
  
  public int getAvailable() {
    return rBuf.length();  
  }
  
  public String getInput() {
    String temp=rBuf;
    rBuf=new String();
//    DebugManager.logOut(DP(), "before add partner window size");
    _addPartnerWindowSize(temp.length());
//    DebugManager.logOut(DP(), "after add partner window size");
    return temp;
  }
  
  protected synchronized void _addPartnerWindowSize(int len) {
//    DebugManager.logOut(DP(), "apws 1");  
    if (connected) {
//      DebugManager.logOut(DP(), "apws 2");    
      String toSend=io.wByte(SSH2Constants.SSH_CONNECTION_MSG_CHANNEL_WINDOW_ADJUST);
      toSend+=io.wUint32(partnerID);
      toSend+=io.wUint32(len);
      tokenizer.sendP(toSend);
//      DebugManager.logOut(DP(), "apws 3");  
      partnerWindowSize+=len;
//      DebugManager.logOut(DP(), "apws 4");  
    }
//    DebugManager.logOut(DP(), "apws 5");  
  }
  
  public boolean isReadableConnected() {
    return connected;  
  }
  
  public void setReader(Reader reader) {
    this.reader=reader;
    _setConnected();
  }
  
  public Reader getReader() {
    return reader;
  }
  
  // -------------------------------------------------------------
  // Writeable
  // -------------------------------------------------------------

  public String getWriteableName() {
    return NAME;
  }
  
  public String getWriteableVersion() {
    return VERSION;
  }
  
  public boolean write(char c) {
    if (connected) {  
      return wBuf.addToBuffer(c);  
    }
    return false;
  }
  
  public boolean write(String s) {
    if (connected) {  
      return wBuf.addToBuffer(s);  
    }
    return false;
  }
  
  public boolean write(char[] c) {
    if (connected) {  
      return wBuf.addToBuffer(c);  
    }
    return false;
  }
  
  public void flush() {
      
    if (!connected) {
      return;
    }
    
    _flush(FLUSH_EXTERN);
  }
  
  private synchronized void _flush(int from) {
    
    String toSend;
    String buf;
    int len;
    while ((ownWindowSize>0) && (wBuf.getBufferLength()>0)) {
      if (ownWindowSize<ownMaxPacketSize) {
        len=ownWindowSize;
      }
      else {
        len=ownMaxPacketSize;
      }            
      buf=wBuf.emptyBuffer(len);
      ownWindowSize-=buf.length();
      toSend=io.wByte(SSH2Constants.SSH_CONNECTION_MSG_CHANNEL_DATA);
      toSend+=io.wUint32(partnerID);
      toSend+=io.wString(buf);
      tokenizer.sendP(toSend);        
    }  
            
/*    if (from==FLUSH_EXTERN) { // extern flush
	if (wBuf.getBufferLength()>0) {
    	    flushExternThread=Thread.currentThread();
    	    try {   
        	synchronized (flushExternThread) {
            	    while (wBuf.getBufferLength()>0) {            
                	flushing=true;
                	DebugManager.logOut(DP(), "wait for flushing: "+wBuf.getBufferLength());
                	flushExternThread.wait();
            	    }
            	    flushExternThread=null;
        	}            
    	    }
    	    catch (Exception e) {
        	DebugManager.logException(DP(), e);
    	    }
	}
    }
    else { // intern flush
        if (wBuf.getBufferLength()==0) {
            if (flushExternThread!=null) {
                synchronized (flushExternThread) {
                    flushExternThread.notify();
                }
            }
        }
    } */
  }
  
  public String getLastWriteableError() {
    return null;  
  }
  
  public boolean isWriteableConnected() {
    return connected;  
  }
    // -------------------------------------------------------------
  // Configureable
  // -------------------------------------------------------------
  
  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
    _rereadConfig();
  }
  
  public Configuration getConfiguration() {
    return configuration;
  }
  
  private void _rereadConfig() {
      
    try {
        Configuration conf=configuration.getConfiguration(SSH2_CONF);
        partnerWindowSize=conf.getInt("SSH2BasicChannel.PARTNER_WINDOW_SIZE", PARTNER_WINDOW_SIZE);        
        partnerPacketSize=conf.getInt("SSH2BasicChannel.PARTNER_PACKET_SIZE", PARTNER_WINDOW_SIZE);
        int bufsize=conf.getInt("SSH2BasicChannel.BUF_SIZE", BUF_SIZE);        
        wBuf=new com.monica.util.StringBuffer(bufsize);
    }
    catch (Exception e) {
        DebugManager.logException(DP_S, e);
    }
                  
    _setConnected();  
    _rereadConfigStub();  
  }
  
  protected void _rereadConfigStub() {
  }
  
  private String DP() {
      return DP_S+"["+Thread.currentThread().hashCode()+"]";
  }
  
}
