package com.monica.javacom.ssh2.interfaces;

import com.monica.javacom.ssh2.interfaces.*;
import com.monica.javacom.ssh2.*;

public interface SSH2ConnectionLayer extends SSH2Layer {

  public String getConnectionLayerName();
  public String getConnectionLayerVersion();
  
  public void setChannelManager(SSH2ChannelManager chManager);
  public SSH2ChannelManager getChannelManager();
  
  public void channelClosed(SSH2Channel channel);
  public SSH2Channel startNewChannel();
  
  public SSH2Channel getNewChannel();
  public void startChannel(SSH2Channel channel);
  
  public void start();
  
  /** Indicates that a GLOBAL_REQUEST message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionGlobalRequest(SSH2Packet pack);
  
  /** Indicates that a REQUEST_SUCCESS message has arrived.
      @param String: the payload of the message
  */  
  public void processConnectionRequestSuccess(SSH2Packet pack);
  
  /** Indicates that a REQUEST_FAILURE message has arrived.
      @param String: the payload of the message
  */  
  public void processConnectionRequestFauilure(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_OPEN message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelOpen(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_OPEN_CONFIRMATION message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelOpenConfirmation(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_OPEN_FAILURE message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelOpenFailure(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_WINDOW_ADJUST message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelWindowAdjust(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_DATA message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelData(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_EXTENDED_DATA message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelExtendedData(SSH2Packet pack);
    
  /** Indicates that a CHANNEL_EOF message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelEOF(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_CLOSE message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelClose(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_REQUEST message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelRequest(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_SUCCESS message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelSuccess(SSH2Packet pack);
  
  /** Indicates that a CHANNEL_FAILURE message has arrived.
      @param String: the payload of the message
  */
  public void processConnectionChannelFailure(SSH2Packet pack);
}

