package com.monica.javacom.ssh2.mac;

import com.monica.javacom.interfaces.*;
import com.monica.javacom.ssh2.*;
import com.monica.util.*;
import com.monica.interfaces.*;
import com.monica.util.debug.*;

import java.util.*;
import java.math.*;
import java.io.*;

/** This class provides HMAC Message Authentication as defined in RFC-2104. */
public class SSH2HMAC implements MACAlgorithm {
    
  private static final String DP="SSH2HMAC";  

  private static final String NAME="SSHHMAC";
  private static final String VERSION="0.2";

  private int B, L;
  private char[] ipad, opad;
  
  private Configuration configuration; 
  private HashAlgorithm hash;
  
  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
  }
  
  public Configuration getConfiguration() {
    return configuration;
  }        

  /** Default constructor. For HMAC to be functional, a hash algorithm
      is required. This has to be provided as parameter.
      @param HashAlgorithm h: the hash algorithm that is required by HMAC
  */
  public SSH2HMAC(HashAlgorithm h) {
    hash=h;
    B=64;
    L=hash.getHashLength();
    ipad=new char[B];
    for (int i=0; i<B; i++) ipad[i]=((char)0x36);
    opad=new char[B];
    for (int i=0; i<B; i++) opad[i]=((char)0x5C);
  }

  public String getMACAlgorithmName() {
    return NAME;
  }

  public String getMACAlgorithmVersion() {
    return VERSION;
  }

  public String createMAC(String data, String K) {
      
    String s1, s2, s3, s4, s5, s6, s7;  
      
    char[] s1ca, s1ca_temp; 
    
    // 0 //

    if (K.length()>B) {
      K=hash.hash(K);
    }

    // 1 //
    s1ca_temp=K.toCharArray();
    s1ca=new char[B];
    System.arraycopy(s1ca_temp, 0, s1ca, 0, s1ca_temp.length);
    
    if (s1ca_temp.length<B) {
      for (int i=s1ca_temp.length; i<B; i++) {
        s1ca[i]=(char)0;
      }
    }
    
    s1=new String(s1ca);

    // 2 //
    s2=XOR(s1,ipad);

    // 3 //
    s3=s2.concat(data);

    // 4 //
    s4=hash.hash(s3);

    // 5 //
    s5=XOR(s1,opad);

    // 6 //
    s6=s5.concat(s4);

    // 7 //
    s7=hash.hash(s6);

    return s7;
  }

  public boolean authenticate(String data, String MAC, String K) {
    String cMAC=createMAC(data, K);

    return cMAC.equals(MAC);
  }

  public int getMACLength() {
    return hash.getHashLength();
  }

  private String XOR(String x1, char[] x2ca) {            
      
    char x1ca[]=x1.toCharArray();  
    char c[]=new char[x1ca.length];
    
    for (int i=0; i<x1ca.length; i++) {
      c[i]=(char)(((int)(x1ca[i]))^((int)(x2ca[i])));
    }
    return new String(c);
  }

}