package com.monica.javacom.ssh2.userauthentication;

import com.monica.interfaces.*;
import com.monica.javacom.interfaces.*;
import com.monica.javacom.ssh2.*;
import com.monica.javacom.ssh2.interfaces.*;
import com.monica.util.*;
import com.monica.util.hash.*;
import com.monica.util.configuration.*;
import com.monica.util.debug.*;

import java.util.*;

public class SSH2NoneUA implements SSH2UserAuthenticationLayer {
    
  private static final String DP="SSH2NoneUA";  

  private static final String NAME="SSH2NoneUA";
  private static final String VERSION="0.31";
  private static final String METHODNAME="none";

  private static final String ERR_NOT_CONNECTED="Not connected.";
  private static final String ERR_NOT_UA_SERVICE_ACCEPT="Not an ssh-userauth service accept msg.";

  private Configuration configuration;
  private SSH2Tokenizer tokenizer;
  private int side;
  private boolean connected;
  private String lastError;
  private SSH2IO io;
  private SSH2BothSideFeatures owner;
  private Object myParam;
  private boolean canContinue;
  private String userName;

  public SSH2NoneUA() {
    lastError=new String();
    connected=false;
    side=SSH2Constants.SSH_SIDE_NOT_SET;
    io=new SSH2IO();
  }

  // **************************************************************************************
  // Closeable
  // **************************************************************************************

  public String getCloseableName() {
    return NAME;
  }

  public String getCloseableVersion() {
    return VERSION;
  }

  public void closeConnections() {
    connected=false;
    tokenizer.closeConnections();
  }

  // **************************************************************************************
  // Configureable
  // **************************************************************************************

  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
    rereadConfig();
  }

  public Configuration getConfiguration() {
    return configuration;
  }

  private void rereadConfig() {
    try {
      setConnected();
    }
    catch (Exception e) {
      DebugManager.logException(DP, e);  
    }        
  }

  // **************************************************************************************
  // SSH2Layer
  // **************************************************************************************

  public void setTokenizer(SSH2Tokenizer tokenizer) {
    this.tokenizer=tokenizer;
    setConnected();
  }

  public SSH2Tokenizer getTokenizer() {
    return tokenizer;
  }

  public void setSide(int side) {
    this.side=side;
    setConnected();
  }

  public int getSide() {
    return side;
  }

  public String methodName() {
    return METHODNAME;
  }

  public void setParam(Object o) throws Exception {
    myParam=o;  
    setConnected();
  }

  public void setOwner(SSH2BothSideFeatures owner) {
    this.owner=owner;
  }

  public SSH2BothSideFeatures getOwner() {
    return owner;
  }

  // **************************************************************************************
  // SSH2UserAuthenticationLayer
  // **************************************************************************************

  public String getUserAuthenticationLayerName() {
    return NAME;
  }

  public String getUserAuthenticationLayerVersion() {
    return VERSION;
  }

  public void start() throws Exception {
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :          
        requestMethod();  
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        break;
    }
  }
  
  private void requestMethod() throws Exception {
    String req=io.wByte(SSH2Constants.SSH_USERAUTH_MSG_USERAUTH_REQUEST);
    req+=io.wString("");
    req+=io.wString("ssh-connection");
    req+=io.wString("none");
    tokenizer.sendP(req);
  }
    
  public void processUserAuthRequestMsg(SSH2Packet pack) throws Exception {
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :
        tokenizer.sendUnimplementedMsg(pack);
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
          
        Vector data;
        
        data=io.parsePayload(pack.payload,SSH2Constants.NONE_USERAUTH_REQUEST_MSG_DATA_DESC);                          
        userName=(String)data.elementAt(SSH2Constants.NONE_USERAUTH_REQUEST_USERNAME);
        
        DebugManager.logOut(DP, "userName="+userName);
                    
        tokenizer.userAuthSuccess();
        tokenizer.userAuthenticationProtocolReady();
        break;
    }
  }
  
  public void processUserAuthPkOkMsg(SSH2Packet pack) throws Exception {
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    tokenizer.sendUnimplementedMsg(pack);
  }

  public void processUserAuthSuccessMsg(SSH2Packet pack) throws Exception {
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :
        tokenizer.userAuthenticationProtocolReady();
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        tokenizer.sendUnimplementedMsg(pack);
        break;
    }
  }

  public void processUserAuthFailureMsg(SSH2Packet pack) throws Exception {
    String possibleContinuation;
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :          
        DebugManager.logOut(DP, "Connection failed.");
        tokenizer.closeConnections();
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        tokenizer.sendUnimplementedMsg(pack);
        break;
    }
  }

  public void processUserAuthBannerMsg(SSH2Packet pack) throws Exception {
    String temp;
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :
        tokenizer.sendUnimplementedMsg(pack);
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        tokenizer.sendUnimplementedMsg(pack);
        break;
    }
  }
  
  private void setConnected() {
    if ((tokenizer!=null) && (side!=SSH2Constants.SSH_SIDE_NOT_SET) &&
        (configuration!=null)) {
        connected=true;
        return;
    }
    connected=false;
  }

  private void throwException(String s) throws Exception {
    lastError=s;
    throw new Exception(s);
  }

  public boolean canContinue() {
      return true;
  }
  
  public String getInformation(int opt) {
      switch (opt) {
          case SSH2Constants.INFO_USERAUTHENTICATION_PARTNER_USERNAME :
              return userName;
          case SSH2Constants.INFO_USERAUTHENTICATION_METHOD :
              return METHODNAME;
          default :
              return null;
      }
  }
  

}