package com.monica.javacom.ssh2.userauthentication;

import com.monica.interfaces.*;
import com.monica.javacom.interfaces.*;
import com.monica.javacom.ssh2.*;
import com.monica.javacom.ssh2.interfaces.*;
import com.monica.util.*;
import com.monica.util.hash.*;

import com.monica.util.debug.*;

import java.util.*;

public class SSH2PasswordUA implements SSH2UserAuthenticationLayer, LayerTopView {
    
  private static final String DP="SSH2PasswordUA";  

  private static final String NAME="SSH2PasswordUA";
  private static final String VERSION="0.31";
  private static final String METHODNAME="password";
  
  private static final String CONF_CLIENT="JavaCom.ssh2client";
  private static final String CONF_PASSWD="JavaCom.password";
  private static final String CONF_SSH2="JavaCom.ssh2";

  private static final String ERR_NOT_CONNECTED="Not connected.";
  private static final String ERR_NOT_UA_SERVICE_ACCEPT="Not an ssh-userauth service accept msg.";

  private static final String UO_USERNAME="Username:";
  private static final String UO_PASSWORD="Password:";
  private static final String UO_INCORRECTLOGIN="Login incorrect."+SSH2Constants.CRLF;
  private static final String UO_CANNOTCONTINUE="Cannot continue login attempt."+SSH2Constants.CRLF;

  private static final int UI_USERNAME_LENGTH=40;
  private static final int UI_PASSWORD_LENGTH=40;

  private String UIS_username, UIS_password;
  private boolean UII_username, UII_password;
  private String presetUserName, presetUserPassword;

  private Configuration configuration;
  private SSH2Tokenizer tokenizer;
  private int side;
  private com.monica.interfaces.Reader reader;
  private com.monica.interfaces.Writer writer;
  private String writeBuf;
  private String readBuf;
  private boolean connected;
  private String lastError;
  private SSH2IO io;
  private SSH2BothSideFeatures owner;
  private Configuration passwords;
  private SHA1 sha1;
  private String username;
  
  private int retries;
  
  private static final String CONF_RETRIES="UA.password.retries";
  private static final int CONF_DEF_RETRIES=3;

  public SSH2PasswordUA() {
    lastError=new String();
    connected=false;
    writeBuf=new String();
    readBuf=new String();
    side=SSH2Constants.SSH_SIDE_NOT_SET;
    UIS_username=new String();
    UIS_password=new String();
    io=new SSH2IO();
    sha1=new SHA1();
  }

  // **************************************************************************************
  // LayerTopView
  // **************************************************************************************

  public String getLayerTopViewName() {
    return NAME;
  }

  public String getLayerTopViewVersion() {
    return VERSION;
  }

  // **************************************************************************************
  // Readable
  // **************************************************************************************

  public String getReadableName() {
    return NAME;
  }

  public String getReadableVersion() {
    return VERSION;
  }

  public int getAvailable() {
    return writeBuf.length();
  }

  public String getInput() {
    String temp=writeBuf;
    writeBuf=new String();
    return temp;
  }

  public boolean isReadableConnected() {
    return connected;
  }

  public void setReader(com.monica.interfaces.Reader reader) {
    this.reader=reader;
  }

  public com.monica.interfaces.Reader getReader() {
    return reader;
  }

  // **************************************************************************************
  // Writeable
  // **************************************************************************************

  public String getWriteableName() {
    return NAME;
  }

  public String getWriteableVersion() {
    return VERSION;
  }

  public boolean write(char c) {
    if (!connected) {
      lastError=ERR_NOT_CONNECTED;
      return false;
    }
    char ca[]=new char[1];
    ca[0]=c;
    addToReadBuf(new String(ca));
    return true;
  }

  public boolean write(char ca[]) {
    if (!connected) {
      lastError=ERR_NOT_CONNECTED;
      return false;
    }
    addToReadBuf(new String(ca));
    return true;
  }

  public boolean write(String s) {
    if (!connected) {
      lastError=ERR_NOT_CONNECTED;
      return false;
    }
    addToReadBuf(s);
    return true;
  }

  public void flush() {
    process(readBuf);
    readBuf=new String();
  }

  public String getLastWriteableError() {
    String temp=lastError;
    lastError=new String();
    return temp;
  }

  public boolean isWriteableConnected() {
    return connected;
  }

  private void addToReadBuf(String s) {
    readBuf=readBuf.concat(s);
  }

  // **************************************************************************************
  // Closeable
  // **************************************************************************************

  public String getCloseableName() {
    return NAME;
  }

  public String getCloseableVersion() {
    return VERSION;
  }

  public void closeConnections() {
    connected=false;
    tokenizer.closeConnections();
  }

  // **************************************************************************************
  // Configureable
  // **************************************************************************************

  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
    _rereadConfig();
  }

  public Configuration getConfiguration() {
    return configuration;
  }

  private void _rereadConfig() {
    try {
      try {  
        passwords=configuration.getConfiguration(CONF_PASSWD);        
      }
      catch (Exception e) {
      }
      
      try { 
        retries=configuration.getConfiguration(CONF_SSH2).getInt(CONF_RETRIES, CONF_DEF_RETRIES);
      }
      catch (Exception e) {
        retries=CONF_DEF_RETRIES;
      }
              
      try {
        Configuration conf=configuration.getConfiguration(CONF_CLIENT);
        presetUserName=conf.getString("userName");
        try {
            presetUserPassword=conf.getString("userPassword");
        }
        catch (Exception e) {
            presetUserPassword="";
        }
      }
      catch (Exception e) {
      }
            
      setConnected();
    }
    catch (Exception e) {
      DebugManager.logException(DP, e);      
    }        
  }

  // **************************************************************************************
  // SSH2Layer
  // **************************************************************************************

  public void setTokenizer(SSH2Tokenizer tokenizer) {
    this.tokenizer=tokenizer;
    setConnected();
  }

  public SSH2Tokenizer getTokenizer() {
    return tokenizer;
  }

  public void setSide(int side) {
    this.side=side;
    setConnected();
  }

  public int getSide() {
    return side;
  }

  public String methodName() {
    return METHODNAME;
  }

  public void setParam(Object o) throws Exception {
    writer=(com.monica.interfaces.Writer)o;
    reader=(com.monica.interfaces.Reader)o;
    reader.setReadable(this);
    writer.setWriteable(this);
    setConnected();
  }

  public void setOwner(SSH2BothSideFeatures owner) {
    this.owner=owner;
  }

  public SSH2BothSideFeatures getOwner() {
    return owner;
  }

  // **************************************************************************************
  // SSH2UserAuthenticationLayer
  // **************************************************************************************

  public String getUserAuthenticationLayerName() {
    return NAME;
  }

  public String getUserAuthenticationLayerVersion() {
    return VERSION;
  }

  public void start() throws Exception {
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :                    
        loginAttempt();  
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        break;
    }
  }

  public void processUserAuthRequestMsg(SSH2Packet pack) throws Exception {
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    Vector data;
    String method, password, hPassword;
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :
        tokenizer.sendUnimplementedMsg(pack);
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        data=io.parsePayload(pack.payload,SSH2Constants.PASSWORD_USERAUTH_REQUEST_MSG_DATA_DESC);  
        username=(String)data.elementAt(SSH2Constants.PASSWORD_USERAUTH_REQUEST_USERNAME);
        password=(String)data.elementAt(SSH2Constants.PASSWORD_USERAUTH_REQUEST_PASSWORD);
        hPassword=IO.print(sha1.hash(password));
        try {
          if (passwords.getString(username).equals(hPassword)) {
            tokenizer.userAuthSuccess();  
            tokenizer.userAuthenticationProtocolReady();
          }
          else {
            retries--;  
            tokenizer.userAuthFailure();
          }
          break;
        }
        catch (Exception e) {
          retries--;
          tokenizer.userAuthFailure();
        }
    }
  }

  public void processUserAuthSuccessMsg(SSH2Packet pack) throws Exception {
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :
        tokenizer.userAuthenticationProtocolReady();
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        tokenizer.sendUnimplementedMsg(pack);
        break;
    }
  }

  public void processUserAuthFailureMsg(SSH2Packet pack) throws Exception {
    String possibleContinuation;
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :
        writeToReader(UO_INCORRECTLOGIN);
        possibleContinuation=io.rString(pack.payload.substring(1));
        if (possibleContinuation.indexOf("password")!=-1) {
          loginAttempt();
        }
        else {
          writeToReader(UO_CANNOTCONTINUE);
          tokenizer.closeConnections();
        }
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        tokenizer.sendUnimplementedMsg(pack);
        break;
    }
  }

  public void processUserAuthBannerMsg(SSH2Packet pack) throws Exception {
    String temp;
    if (!connected) {
      throwException(ERR_NOT_CONNECTED);
    }
    switch (side) {
      case SSH2Constants.SSH_SIDE_CLIENT :
        temp=io.rString(pack.payload.substring(1));
        writeToReader(temp+SSH2Constants.CRLF);
        break;
      case SSH2Constants.SSH_SIDE_SERVER :
        tokenizer.sendUnimplementedMsg(pack);
        break;
    }
  }
  
  public void processUserAuthPkOkMsg(SSH2Packet pack) throws Exception {
      tokenizer.sendUnimplementedMsg(pack);
  }

  private void setConnected() {
    if ((tokenizer!=null) && (side!=SSH2Constants.SSH_SIDE_NOT_SET) &&
        (configuration!=null)) {
            switch (side) {
                case SSH2Constants.SSH_SIDE_CLIENT :
                    if ((reader!=null) && (writer!=null)) {
                        connected=true;
                        return;
                    }
                    break;
                case SSH2Constants.SSH_SIDE_SERVER :
                    connected=true;
                    return;
            }
    }
    connected=false;
  }

  private void throwException(String s) throws Exception {
    lastError=s;
    throw new Exception(s);
  }

  /** Login attempt manager method.
      @param int state: where the login attempt cycle is
  */
  private void loginAttempt() {
    if ((presetUserName!=null) && (presetUserPassword!=null)) {
        
        DebugManager.logOut(DP, "Logging in with preset user info.");
        
        UIS_username=presetUserName;
        UIS_password=presetUserPassword;
        
        sendLoginAttemptMsg();
        return;
    }
      
    UIS_username=new String();
    UIS_password=new String();
    UII_username=true;
    UII_password=false;
    writeToReader(UO_USERNAME);
  }

  /** Parses user input for username and password entering.
      @param String s: the String to be parsed.
      @return boolean:
  */
  private boolean process(String s) {
    int i;
    char c;
    char ca[]=new char[1];

    // username
    if (UII_username) {
      for (i=0; i<s.length(); i++) {
        c=s.charAt(i);
        ca[0]=c;

        if ((c>32) && (c<127)) { // chars
          if (UIS_username.length()<UI_USERNAME_LENGTH) {
            UIS_username=UIS_username.concat(new String(ca));
          }
          refreshUILine(UO_USERNAME, UIS_username, true);
        }
        else if ((c==8) || (c==127)) { // DEL
          if (UIS_username.length()>0) {
            UIS_username=UIS_username.substring(0,UIS_username.length()-1);
          }
          refreshUILine(UO_USERNAME, UIS_username, true);
        }
        else if (c==10) {
          if (UII_password==false) {
            writeToReader(SSH2Constants.CRLF+UO_PASSWORD);
            UII_username=false;
            UII_password=true;
          }
        }
        else if (c==13) {
          if (UII_password==false) {
            writeToReader(SSH2Constants.CRLF+UO_PASSWORD);
            UII_username=false;
            UII_password=true;
          }
        }
        else {
          break;
        }
      }
    }

    // password
    else if (UII_password) {
      for (i=0; i<s.length(); i++) {
        c=s.charAt(i);
        ca[0]=c;

        if ((c>32) && (c<127)) { // chars
          if (UIS_password.length()<UI_PASSWORD_LENGTH) {
            UIS_password=UIS_password.concat(new String(ca));
          }
          refreshUILine(UO_PASSWORD, UIS_password, false);
        }
        else if ((c==8) || (c==127)) { // DEL
          if (UIS_password.length()>0) {
            UIS_password=UIS_password.substring(0,UIS_password.length()-1);
          }
          refreshUILine(UO_PASSWORD, UIS_password, false);
        }
        else if (c==10) {
          if (UII_password==true) {
            UII_password=false;
            writeToReader(SSH2Constants.CRLF);
            sendLoginAttemptMsg();
          }
        }
        else if (c==13) {
          if (UII_password==true) {
            UII_password=false;
            writeToReader(SSH2Constants.CRLF);
            sendLoginAttemptMsg();
          }
        }
        else {
          break;
        }
      }
    }
    return true;
  }

  /** Refreshes the current line to contain the the data received as parameters.
      @param String banner: the fixed part of the line
      @param String ui: the user input
      @param boolean type: true-normal, false-password
  */
  private void refreshUILine(String banner, String ui, boolean type) {

    int i;
    String data=new String();

    data=data.concat(""+((char)13)); // HOME
    data=data.concat(""+((char)27)+((char)91)+((char)75)); // Erase EOL
    data=data.concat(banner);
    if (type) {
      data=data.concat(ui);
    }
    else {
      for (i=0; i<ui.length(); i++) data=data.concat("*");
    }
    writeToReader(data);
  }

  /** Sends a password type login attempt to the server.
  */
  private void sendLoginAttemptMsg() {
    String userAuth=io.wByte(SSH2Constants.SSH_USERAUTH_MSG_USERAUTH_REQUEST);
    userAuth=userAuth.concat(io.wString(UIS_username));
    userAuth=userAuth.concat(io.wString("ssh-connection"));
    userAuth=userAuth.concat(io.wString("password"));
    userAuth=userAuth.concat(io.wBoolean(false));
    userAuth=userAuth.concat(io.wString(UIS_password));
    tokenizer.sendP(userAuth);
  }

  private void writeToReader(String s) {
    writeBuf=writeBuf.concat(s);
    reader.readEvent(this);
  }
  
  public boolean canContinue() {
    return (retries>0);
  }
  
  public String getInformation(int opt) {
      switch (opt) {
          case SSH2Constants.INFO_USERAUTHENTICATION_PARTNER_USERNAME :
              return username;
          case SSH2Constants.INFO_USERAUTHENTICATION_METHOD :
              return METHODNAME;
          default :
              return null;
      }
  }
  

}