package com.monica.javacom.virtualinput;

import java.awt.*;
import java.awt.event.*;
import com.monica.javacom.interfaces.*;
import com.monica.interfaces.*;

/** This class if responsible for handling keyboard events from the user. The
    catched keyboard events will be passed to the Terminal below.
*/
public class KeyHandler implements KeyListener, VirtualInput {

	private static final String NAME="KeyListener";
	private static final String VERSION="0.3";

	private char[] p;
	private char keyChar;

	private Writeable writeable;
	private Configuration configuration;

  /** Default constructor.
  */
  public KeyHandler() {
	  p=new char[2];
	}

	public String getVirtualInputName() {
	  return NAME;
	}

	public String getVirtualInputVersion() {
	  return VERSION;
	}

	public String getWriterName() {
	  return NAME;
	}

	public String getWriterVersion() {
	  return VERSION;
	}

	public void setWriteable(Writeable writeable) {
	  this.writeable=writeable;
	}

	public Writeable getWriteable() {
	  return writeable;
	}

	public void setConfiguration(Configuration configuration) {
	  this.configuration=configuration;
	}

	public Configuration getConfiguration() {
	  return configuration;
	}

  /** Handler method for the key typing event.
      @param KeyEvent e: the KeyEvent generated
  */
	public void keyTyped(KeyEvent e){
	}

  /** Handler method for the key pressing event.
      @param KeyEvent e: the KeyEvent generated
  */
	public void keyPressed(KeyEvent e){

		if ( (e.getKeyCode()!=KeyEvent.VK_CONTROL) && (e.getKeyCode()!=KeyEvent.VK_ALT) && (e.getKeyCode()!=KeyEvent.VK_SHIFT) ) {
			keyChar=e.getKeyChar();
			if( 0<keyChar && keyChar<128) {
				p[0]=0x00;
				p[1]=keyChar;
			}
			else {
				switch (e.getKeyCode()) {
					case KeyEvent.VK_F1:
						p[0]=0x04;
						p[1]=0x3B;
						break;
					case KeyEvent.VK_F2:
						p[0]=0x04;
						p[1]=0x3C;
						break;
					case KeyEvent.VK_F3:
						p[0]=0x04;
						p[1]=0x3D;
						break;
					case KeyEvent.VK_F4:
						p[0]=0x04;
						p[1]=0x3E;
						break;
					case KeyEvent.VK_F5:
						p[0]=0x04;
						p[1]=0x3F;
						break;
					case KeyEvent.VK_F6:
						p[0]=0x04;
						p[1]=0x40;
						break;
					case KeyEvent.VK_F7:
						p[0]=0x04;
						p[1]=0x41;
						break;
					case KeyEvent.VK_F8:
						p[0]=0x04;
						p[1]=0x42;
						break;
					case KeyEvent.VK_F9:
						p[0]=0x04;
						p[1]=0x43;
						break;
					case KeyEvent.VK_F10:
						p[0]=0x04;
						p[1]=0x44;
						break;
					case KeyEvent.VK_F11:
						p[0]=0x04;
						p[1]=0x85;
						break;
					case KeyEvent.VK_F12:
						p[0]=0x04;
						p[1]=0x86;
						break;
					case KeyEvent.VK_PRINTSCREEN:
						p[0]=0x04;
						p[1]=0xFB;
						break;
					case KeyEvent.VK_PAUSE:
						p[0]=0x04;
						p[1]=0xFC;
						break;
					case KeyEvent.VK_INSERT:
						p[0]=0x04;
						p[1]=0x52;
						break;
					case KeyEvent.VK_HOME:
						p[0]=0x04;
						p[1]=0x47;
						break;
					case KeyEvent.VK_PAGE_UP:
						p[0]=0x04;
						p[1]=0x49;
						break;
					case KeyEvent.VK_DELETE:
						p[0]=0x04;
						p[1]=0x53;
						break;
					case KeyEvent.VK_END:
						p[0]=0x04;
						p[1]=0x4F;
						break;
					case KeyEvent.VK_PAGE_DOWN:
						p[0]=0x04;
						p[1]=0x51;
						break;
					case KeyEvent.VK_UP:
						p[0]=0x04;
						p[1]=0x48;
						break;
					case KeyEvent.VK_LEFT:
						p[0]=0x04;
						p[1]=0x4B;
						break;
					case KeyEvent.VK_DOWN:
						p[0]=0x04;
						p[1]=0x50;
						break;
					case KeyEvent.VK_RIGHT:
						p[0]=0x04;
						p[1]=0x4D;
						break;
					case KeyEvent.VK_DIVIDE:
						p[0]=0x04;
						p[1]=0x81;
						break;
					case KeyEvent.VK_MULTIPLY:
						p[0]=0x04;
						p[1]=0x82;
						break;
					case KeyEvent.VK_SUBTRACT:
						p[0]=0x04;
						p[1]=0x83;
						break;
					case KeyEvent.VK_NUMPAD7:
						p[0]=0x04;
						p[1]=0x97;
						break;
					case KeyEvent.VK_NUMPAD8:
						p[0]=0x04;
						p[1]=0x98;
						break;
					case KeyEvent.VK_NUMPAD9:
						p[0]=0x04;
						p[1]=0x99;
						break;
					case KeyEvent.VK_ADD:
						p[0]=0x04;
						p[1]=0x80;
						break;
					case KeyEvent.VK_NUMPAD4:
						p[0]=0x04;
						p[1]=0x94;
						break;
					case KeyEvent.VK_NUMPAD5:
						p[0]=0x04;
						p[1]=0x95;
						break;
					case KeyEvent.VK_NUMPAD6:
						p[0]=0x04;
						p[1]=0x96;
						break;
					case KeyEvent.VK_NUMPAD1:
						p[0]=0x04;
						p[1]=0x91;
						break;
					case KeyEvent.VK_NUMPAD2:
						p[0]=0x04;
						p[1]=0x92;
						break;
					case KeyEvent.VK_NUMPAD3:
						p[0]=0x04;
						p[1]=0x93;
						break;
					case KeyEvent.VK_NUMPAD0:
						p[0]=0x04;
						p[1]=0x90;
						break;
					case KeyEvent.VK_DECIMAL:
						p[0]=0x04;
						p[1]=0x84;
						break;
					default:
						p[0]=0x00;
						p[1]=0x00;
						break;
				}
			}

			String s=new String(p);
			if (writeable!=null) {
			  writeable.write(s);
			  writeable.flush();
			}
	   }
	   e.consume();
	}

  /** Handler method for the key releasing event.
      @param KeyEvent e: the KeyEvent generated
  */
	public void keyReleased(KeyEvent e){
	}

}
