package com.monica.javacom.window;

import com.monica.javacom.interfaces.*;
import com.monica.interfaces.*;

import java.awt.*;
import java.awt.event.*;

/** Class for providing a framwork with AWT methods for the VirtualDisplay and
    VirtualInput implementing classes.<br><br>
    Configureable values from the <b>JavaCom.window</b> Configuration:
    <ul>
      <li><b>windowTitle</b>: the window title of the client side
    </ul>
*/
public class AWTWindow extends Frame implements com.monica.javacom.interfaces.Window {

  VirtualDisplay content;
  Configuration configuration;
  Closer closer;
  
  private static final String CONF_STREAM_NAME="JavaCom.window";

  private static final String NAME="AWTWindow";
  private static final String VERSION="0.1";

  private static final String CONF_WINDOW_TITLE="windowTitle";
  private static final String CONF_DEF_WINDOW_TITLE="MONICA JavaCom Terminal";

  /** Default constructor. The VirtualDisplay output visualization, the Configuration customization
      and a KeyListener has to be specified.
      @param VirtualDisplay content: output visualization
      @param Configuration configuration: customization
      @param KeyListener kl: the keyboard event handler
  */
  public AWTWindow(VirtualDisplay content, Configuration configuration, KeyListener kl) throws Exception {      
    super(configuration.getConfiguration(CONF_STREAM_NAME).getString(CONF_WINDOW_TITLE, CONF_DEF_WINDOW_TITLE));

    this.content=content;
    this.configuration=configuration.getConfiguration(CONF_STREAM_NAME);

    Dimension d=((Canvas)content).getSize();
    ((Canvas)content).addKeyListener(kl);
    addKeyListener(kl);
    add((Canvas)content);
    setBackground(((Canvas)content).getBackground());
    setSize(new Dimension(d.width+20, d.height+40));
  }

  /** Returns the  name of the Window.
      @return String: the  name of the Window
  */
  public String getWindowName() {
    return NAME;
  }

  /** Returns the  version of the Window.
      @return String: the  version of the Window
  */
  public String getWindowVersion() {
    return VERSION;
  }

  /** Sets the Configuration for this object.
      @param Configuration configuration: the Configuration
        object for the customization
  */
  public void setConfiguration(Configuration configuration) {
      try {
        this.configuration=configuration.getConfiguration(CONF_STREAM_NAME);
      }
      catch (Exception e) {
      }
  }

  /** Returns the Configuration with which this object is customized.
      @return Configuration: the Configuration with which this
        object is customized
  */
  public Configuration getConfiguration() {
    return configuration;
  }

  /** Sets the Closer responsible for closing of the client
      side application upon user request.
      @param Closer closer: the Closer reponsible for the
        closing of the client side application upon user request
  */
  public void setCloser(Closer closer) {
    this.closer=closer;
    addWindowListener(new AWTWindowWindowCloser(closer, this));
  }

  /** Returns the Closer responsible for the closing of the
      client side application upon user request.
      @return Closer: the Closer responsible for the closing
        of the client side application upon user request
  */
  public Closer getCloser() {
    return closer;
  }

}

/** Class for processing window close mouse events.
*/
class AWTWindowWindowCloser extends WindowAdapter {
  Closer closer;
  Frame window;

  /** Constructor.
      @param Closer closer: the application closer object
      @param Frame window: the main window to be closed
  */
  AWTWindowWindowCloser(Closer closer, Frame window) {
    this.closer=closer;
    this.window=window;
  }

  /** Method processing the window closing request.
      @param WindowEvent e: the generated WindowEvent
  */
  public void windowClosing(WindowEvent e) {
    window.setVisible(false);
    window.dispose();
    closer.closeApplication();
    System.exit(0);
  }
}

