package com.monica.streams;

import com.monica.interfaces.*;
import com.monica.util.*;
import java.io.*;

/** Class for creating a link between an InputStream and the TM
    layer hierarchy. This class acts as a Readable, this means it
    reads data from the specified InputStream and then forwards it
    to the Reader through the readEvent mechanism. Reading is
    buffered. Reading fromt the InputStream happens in a different
    thread as in which the ReadableHasAIS has been constructed.
*/    

public class ReadableHasAIS implements Readable, Runnable {

  private static final String NAME="ReadableHasAIS";
  private static final String VERSION="0.3";
  private static final int bufSize=50000;
  
  private com.monica.interfaces.Reader reader;
  private com.monica.util.StringBuffer buf;
  private InputStream is;
  private boolean connected;
  
  /** Constructor. The InputStream to read from has to be specified.
      @param InputStream is: the InputStream to read from
  */  
  public ReadableHasAIS(InputStream is) {
    this.is=is;
    buf=new com.monica.util.StringBuffer(bufSize);
    connected=false;
  }
  
  // --------------------------------------------------------------------------
  // Readable
  // --------------------------------------------------------------------------
  
  public String getReadableName() {
    return NAME;
  }
  
  public String getReadableVersion() {
    return VERSION;
  }
  
  public int getAvailable() {
    return buf.available();    
  }
  
  public String getInput() {
    String temp=buf.emptyBuffer();
    synchronized (this) {
      notify();
    }
    return temp;  
  }
  
  public boolean isReadableConnected() {
    return connected;
  }
  
  public void setReader(com.monica.interfaces.Reader reader) {
    this.reader=reader;
    connected=true;
  }
  
  public com.monica.interfaces.Reader getReader() {
    return reader;
  }        
  
  // --------------------------------------------------------------------------
  // Runnable
  // --------------------------------------------------------------------------
  
  /** Entry point for the InputStream reader thread.
  */
  public void run() {
    listenForRead();
  }
  
  /** This method reads the InputStream and sends the readEvents to
      the Reader.
  */      
  private void listenForRead() {
    int firstByte;
    byte[] tempRead;
    String tempString;
    int t;
    
    try {
      while (reader==null) {
        synchronized (this) {
	  wait();
	}
      }	  
      while (true) {
        tempRead=new byte[1];
        firstByte=is.read();

        if (firstByte==-1) {
	  connected=false;
          return;
        }

        tempRead[0]=(byte)firstByte;
	while (!buf.addToBuffer(tempRead)) {
	  reader.readEvent(this);
	  synchronized (this) {
	    wait();
	  }
	}      
	
        while (is.available()>0) {
	  while (buf.available()==0) {
	    reader.readEvent(this);
	    synchronized (this) {
	      wait();
	    }
	  }      
          if (is.available()>buf.available()) {
	    tempRead=new byte[bufSize];
	  }
	  else {
	    tempRead=new byte[is.available()];
	  }
          t=is.read(tempRead);
          buf.addToBuffer(tempRead, t);
        }

        if (buf.getBufferLength()>0) {
          reader.readEvent(this);
        }

      }
    }
    catch (Exception e) {
      connected=false;
    }
  }
}