package com.monica.streams;

import com.monica.interfaces.*;
import com.monica.util.*;
import java.io.*;

/** Class for creating a link between an OutputStream and the TM
    layer hierarchy. This class acts as a Reader, this means it 
    handles asynchronous readEvents from an Readable, and what it
    reads, it forwards to an OuputStream. The writing to the OuputStream
    is not buffered.
*/

public class ReaderHasAOS implements com.monica.interfaces.Reader {

  private static final String NAME="ReaderHasAOS";
  private static final String VERSION="0.3";
  
  private Readable readable;
  private OutputStream os;
  
  /** Constructor. The OutputStream has to be specified.
      @param OutputStream os: the OutputStream to which data has to 
        be forwarded
  */	     
  public ReaderHasAOS(OutputStream os) {
    this.os=os;
  }
  
  public String getReaderName() {
    return NAME;
  }
  
  public String getReaderVersion() {
    return VERSION;
  }
  
  public void readEvent(Readable readable) {    
    String input=readable.getInput();
    try {
      byte[] b=input.getBytes();
      os.write(b);
    }
    catch (Exception e) {
      e.printStackTrace();
    }  	            
  }
  
  public void connectionClosedEvent(Readable readable) {
    try {
      os.close();
    }
    catch (Exception e) {
    }  
  }
  
  public void errorEvent(String errorDescription, Readable readable) {
    try {
      os.close();
    }
    catch (Exception e) {
    }  
  }
  
  public void setReadable(Readable readable) {
    this.readable=readable;
  }            
  
  public Readable getReadable() {
    return readable;
  }      
}