package com.monica.streams;

import com.monica.interfaces.*;
import com.monica.util.*;
import java.io.*;

/** Class for creating a link between an OutputStream and the TM layer
    hierarchy. This class acts as a Writeable, this means an upper
    layer can write to it, and what has been written on it, will be written
    again to the OutputStream given. The writing to the OutputStream is 
    buffered.
*/

public class WriteableHasAOS implements Writeable {
  
  private static final String NAME="WriteableHasAOS";
  private static final String VERSION="0.3";
  private static final int bufSize=50000;
  
  private OutputStream os;
  private com.monica.util.StringBuffer buf;
  private String lastError;
  
  private boolean connected;
  
  /** Constructor.
      @param OutputStream os: the specified OutputStream to which the data has to
        be written
  */	  
  public WriteableHasAOS(OutputStream os) {
    this.os=os;
    buf=new com.monica.util.StringBuffer(bufSize);
    connected=true;
    lastError=new String();
  }
  
  public String getWriteableName() {
    return NAME;
  }
  
  public String getWriteableVersion() {
    return VERSION;
  } 
  
  public boolean write(char c) {
    if (connected) {
      if (!buf.addToBuffer(c)) {
        flush();
        return buf.addToBuffer(c);  
      }
      else {
        return true;	
      }	
    }
    else {
      return false;
    }    
  }
  
  /** Writes a String to the OutputStream. Writing to the
      OutputStream is buffered.
      @param String s: the String to be written to the OutputStream
      @return boolean: wether the writing has been successful
  */
  public boolean write(String s) {
    if (connected) {
      if (!buf.addToBuffer(s)) {
         flush();
         if (!buf.addToBuffer(s)) {
           writeOut(s);
	 }  
      }
      return true;    
    }
    else {
      return false;
    }    
  }
  
  /** Writes an array of chars to the OutputStream. Writing to the
      OutputStream is buffered.
      @param char[] c: the char array to be written to the OutputStream
      @return boolean: wether the writing has been successful
  */
  public boolean write(char c[]) {
    if (connected) {
      if (!buf.addToBuffer(c)) {
        flush();
        if (!buf.addToBuffer(c)) {
          writeOut(new String(c));
        }
      }	
      return true;    
    }  
    else {
      return false;
    }  
  }
  
  /** Flushes the conents of the buffer to the OutputStream.
  */
  public void flush() {
    if (connected) {
      writeOut(buf.emptyBuffer());
    }  
  }
  
  /** Flushed the contents of the buffer to the OutputStream.
      @return boolean: wether the flushing has been successful
  /*
  private boolean flushB() {
    if (connected) {
      return writeOut(buf.emptyBuffer());
    }
    else {
      return false;
    }
  }            
  
  /** Writes a String directly to the OutputStream. This method does not
      use the buffer.
      @param String s: the String to be written directly to the OutputStream
      @return boolean: wether the writing to the OutputStream has been successful
  */      
  private boolean writeOut(String s) {
    try {
      for (int i=0; i<s.length(); i++) {
        os.write((byte)s.charAt(i));
      }
      os.flush();
      return true;
    }
    catch (IOException e) {
      connected=false;
      lastError=e.getMessage();
    }    
    return false;
  }         
  
  public String getLastWriteableError() {
    String temp=lastError;
    lastError=new String();
    return temp;
  }
  
  public boolean isWriteableConnected() {
    return connected;
  }      
}