package com.monica.streams;

import com.monica.interfaces.*;
import com.monica.util.*;
import java.io.*;

/** Class for creating a link between an InputStream and the TM
    layer hierarchy. This class acts as a Writer, this means it writes 
    data read from the specified InputStream will be written to 
    the Writeable. Writing is not buffered. Reading from the 
    InputStream happens in a different thread as in which the 
    WriterHasAIS has been constructed.    
*/    

public class WriterHasAIS implements com.monica.interfaces.Writer, Runnable {

  private static final String NAME="WriterHasAIS";
  private static final String VERSION="0.3";
  private static final int maxLength=50000;
  
  private Writeable writeable;
  private InputStream is;
  private Thread myThread;
  
  /** Constructor. The InputStream to read from has to be specified. 
      @param InputStream is: the InputStream to read from
  */           
  public WriterHasAIS(InputStream is) {
    this.is=is;
    myThread=new Thread(this);
    myThread.start();
  }

  // --------------------------------------------------------------------------
  // Readable
  // --------------------------------------------------------------------------
  
  public String getWriterName() {
    return NAME;
  }
  
  public String getWriterVersion() {
    return VERSION;
  }
  
  public void setWriteable(Writeable writeable) {
    this.writeable=writeable;
    synchronized (this) {
      notify();
    }  
  }
  
  public Writeable getWriteable() {
    return writeable;
  }                  
  
  // --------------------------------------------------------------------------
  // Runnable
  // --------------------------------------------------------------------------
  
  /** The entry point for the InputStream reader thread.
  */
  public void run() {
    listenForRead();
  }
  
  /** This method reads the InputStream and writes the data read
      from it to the MCJWriteable.
  */      
  private void listenForRead() {
    int firstByte;
    byte[] tempRead;
    String tempString;
    int t;

    try {
      while (writeable==null) {
        synchronized (this) {
          wait();
        }  
      }
    
      while (true) {
        tempRead=new byte[1];
        firstByte=is.read();

        if (firstByte==-1) {
          return;
        }
	      
        if (!writeable.write((char)firstByte)) {
	  return;
	}  
	  
        while (is.available()>0) {
	  if (is.available()>maxLength) {
	    tempRead=new byte[maxLength];
	  }    
	  else {    
            tempRead=new byte[is.available()];
	  }    
          t=is.read(tempRead);
	  tempString=new String(tempRead,0,t);
	  if (!writeable.write(tempString)) {
	    return;
	  }    	  
	  writeable.flush();
         }        
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
        
}