package com.monica.util;

/** Class for implementing byte-char conversions as all the other classes assume, 
    but not all JAVA VMs do. Currently (as we have till now experienced) we have a 
    phenomenon which only occures under the Windows platform. This (for us) not wanted 
    behaviour can be tested.
*/    
public class Converter {

  private boolean isWindows;

  /** Public constructor. 
  */
  public Converter() {
    byte b[]=new byte[1];
    b[0]=-120;
    String s=new String(b);
    if (((int)s.charAt(0))==136) isWindows=false; else isWindows=true;
  }

  /** Converts a byte array to a String. 
      @param byte b[]: the byte array to be converted to String
      @return String: the String from the byte array
  */      
  public String toString(byte b[]) {
    String s;
    if (isWindows) {
      char c[]=new char[b.length];
      for (int i=0; i<b.length; i++) {
        if (((int)b[i])<0) c[i]=(char)(256+((int)b[i])); else c[i]=(char)(b[i]);
      }
      s=new String(c);
    }
    else {
      s=new String(b);
    }
    return s;
  }

  /** Converts the first l bytes of a byte array to String.
      @param byte b[]: the byte array to be converted to String
      @param int l: the first l bytes will be converted
      @return String: the resulting String
  */      
  public String toString(byte b[], int l) {
    String s;
    if (isWindows) {
      char c[]=new char[l];
      for (int i=0; i<l; i++) {
        if (((int)b[i])<0) c[i]=(char)(256+((int)b[i])); else c[i]=(char)(b[i]);
      }
      s=new String(c);
    }
    else {
      s=new String(b,0,l);
    }
    return s;
  }

  /** Converts len bytes beginning at offset off from a byte array to String.
      Indexes i, off<=i<(off+len) will be converted.
      @param byte b[] the byte array to be converted
      @param int off: the starting offset
      @param int len: how many bytes will be converted
      @return String: the resulting String
  */      
  public String toString(byte b[], int off, int len) {
    String s;
    if (isWindows) {
      char c[]=new char[len];
      for (int i=0; i<len; i++) {
        if (((int)b[i+off])<0) c[i]=(char)(256+((int)b[i+off])); else c[i]=(char)(b[i+off]);
      }
      s=new String(c);
    }
    else {
      s=new String(b,off,len);
    }
    return s;
  }

  /** Converts a String to a byte array.
      @param String s: the string to be converted to byte array
      @return byte[]: the resulting byte array
  */      
  public byte[] getBytes(String s) {
    byte b[];
    if (isWindows) {
      b=new byte[s.length()];
      for (int i=0; i<s.length(); i++) {
        if (((int)(s.charAt(i)))<128) b[i]=(byte)s.charAt(i);
        else b[i]=(byte)((int)s.charAt(i)-256);
      }
    }
    else {
      b=s.getBytes();
    }
    return b;
  }

  /** Are we running under Windows?
      @return boolean: true if Windows has been detected
  */      
  public boolean isWindows() {
    return isWindows;
  }

}