package com.monica.util;

import com.monica.interfaces.*;

/** Class for implementing basic Observer functionality. */

public class ObservedImpl {

    private java.util.Vector observers;
    
    /** Default constructor */
    public ObservedImpl() {
        observers=new java.util.Vector();
    }
    
    /** Registers an Observer to be notified.
     *  @param Observer observer: the Observer to be registered
     *  @return boolean: whether the registration was successfull
     */
    public boolean registerObserver(Observer observer) {
      if (observers.indexOf(observer)==-1) {
          observers.add(observer);
          return true;
      }
      return false;      
    }
    
    /** Unregisters an Observer from being notified.
     *  @param Observer observer: the Observer to be unregistered
     *  @return boolean: whether the unregistration was successfull
     */
    public boolean unregisterObserver(Observer observer) {
        int pos=observers.indexOf(observer);
        if (pos!=-1) {
            observers.removeElementAt(pos);
            return true;
        }
        return false;              
    }
    
    /** Notifies all registered Observers.
     *  @param Observed object: the Observed object that has changed its state 
     */
    public void observerNotify(Observed object) {
        int l=observers.size();
        for (int i=0; i<l; i++) {
            ((Observer)observers.elementAt(i)).observerNotify(object);
        }
    }
}
