package com.monica.util;

import com.monica.interfaces.*;

import com.monica.util.debug.*;

import java.awt.*;
import java.io.*;

/** Class for implementing software font rendering. This class will read the font
    configuration from an InputStream. 
*/
public class SoftFont implements Text {
    
  private static final String DP="SoftFont";  

  private static final String NAME="SoftFont";
  private static final String VERSION="0.3";

  private boolean data[];
  private int sX, sY, d_sX, d_sY;

  private String buf;

  public SoftFont() {
    buf=new String();
  }

  public String getTextName() {
    return NAME;
  }

  public String getTextVersion() {
    return VERSION;
  }

  public void init(int psX, int psY, InputStream is) {
    d_sX=psX;
    d_sY=psY;

    try {
      String format=nextLine(is);
      sX=Integer.parseInt(format.substring(7,9));
      sY=Integer.parseInt(format.substring(10,12));
      data=new boolean[256*sX*sY];

      String line, lineNo;

      for (int i=0; i<256; i++) {
        for (int j=0; j<sY; j++) {
          while (true) {
            line=nextLine(is);
            lineNo=""+i;
            if (line.substring(0,lineNo.length()).equals(lineNo)) {
              parseLine(i,j,line);
              break;
            }
          }
        }
      }

      rescale();
    }
    catch (Exception e) {
      DebugManager.logException(DP, e);
    }

  }

  public int getHeight() {
    return d_sX;
  }

  public int getWidth() {
    return d_sY;
  }

  int dc_yP, dc_pos, dc_i, dc_j, dc_charmap_line, dc_abs_pos, dc_rel_pos, dc_temp;

  public void drawChar(char c, int x, int y, Color f, Color b, Graphics g) {

    dc_pos=((int)c)*(d_sX*d_sY);
    dc_i=0;

    g.setColor(b);
    g.fillRect(x,
               y,
               d_sX,
               d_sY);

    g.setColor(f);

    for (dc_i=0; dc_i<d_sY; dc_i++) {
      dc_yP=y+dc_i;
      for (dc_j=0; dc_j<d_sX; dc_j++) {
        if (data[dc_pos]) {
          g.fillRect(x+dc_j,dc_yP,1,1);
        }
        dc_pos++;
      }
    }
  }

  private void parseLine(int i, int j, String line) throws Exception {

    String lineNo=""+i;
    char c;
    boolean v=false;
    int last_true=-1;
    int cur_pos;

    for (int k=0; k<sX; k++) {
      if (line.length()>(lineNo.length()+2+k)) {
        c=line.charAt(lineNo.length()+2+k);
        switch (c) {
          case 'x' :
            v=true;
            break;
          case ',' :
            v=false;
            break;
          case '-' :
            v=false;
            break;
          case '.' :
            v=false;
            break;
          default :
            v=false;
      //      throw new Exception("Invalid char.");
        }
      }
      else {
        v=false;
      }
      cur_pos=(i*sX*sY)+(j*sX)+k;
      data[cur_pos]=v;
    }
  }

  private String nextLine(InputStream is) throws Exception {

    int ci;
    String t;

    while (true) {
      ci=is.read();
      if (ci==-1) {
        throw new Exception("End of InputStream.");
      }

      if ((ci==13) || (ci==10)) {
        while (true) {
          ci=is.read();
          if (ci==-1) {
            return buf;
          }
          if ((ci!=10) && (ci!=13)) {
            t=buf;
            buf=new String(""+((char)ci));
            return t;
          }
        }
      }
      else {
        buf=buf.concat(new String(""+((char)ci)));
      }
    }
  }

  private void rescale() {
    boolean newdata[];
    newdata=new boolean[256*d_sX*d_sY];
    int x,y;

    for (int i=0; i<256; i++) {
      for (int j=0; j<sY; j++) {
        y=scale(j,sY,d_sY);
        for (int k=0; k<sX; k++) {
          x=scale(k,sX,d_sX);
          if (data[i*sX*sY+j*sX+k]) {
            newdata[i*d_sX*d_sY+y*d_sX+x]=true;
          }
        }
      }
    }
    data=newdata;
  }

  double fact, res;

  private int scale(int w, int s, int l) {
    fact=((double)l)/((double)s);
    res=fact*(w+1);
    int r=(int)(res-1);
    if (r>=l) r--;
    if (r<0) r++;

    return r;
  }

}