package com.monica.util;

import com.monica.interfaces.*;
import com.monica.util.*;

import java.io.*;

/** Class for providing InputStream functionality by reading from a String.
*/
public class StringInputStream extends InputStream {

  private int pos;
  private byte b[];
  private Converter converter;

  /** Default constructor.
      @param String s: the String to be read from
  */      
  public StringInputStream (String s) {
    pos=0;
    converter=new Converter();
    b=converter.getBytes(s);
  }

  public int read() throws IOException {
    if (pos==b.length) return -1;
    byte bb=b[pos];
    pos++;
    if (bb<0) return (int)(256+bb);
    return (int)bb;
  }

  public int read(byte bout[], int off, int len) throws IOException {
    int rlen=b.length-pos;
    if (rlen==0) return -1;
    if (rlen>len) rlen=len;
    for (int i=0; i<rlen; i++) {
      bout[off+i]=b[pos];
      ++pos;
    }
    return rlen;
  }

  public long skip(long n) throws IOException {
    int rlen=b.length-pos;
    if (rlen>n) rlen=(int)n;
    pos+=rlen;
    return rlen;
  }

  public int available() throws IOException {
    return (b.length-pos);
  }

  public void close() throws IOException {
  }

  public synchronized void mark(int readLimit) {
  }

  public synchronized void reset() throws IOException {
    throw new IOException("Mark/Reset not supported");
  }

  public boolean markSupported() {
    return false;
  }
  
  public synchronized void resetWithUpdate(String s) {
    pos=0;
    b=converter.getBytes(s);
  }  
}