package com.monica.util;

import com.monica.interfaces.*;

import java.util.*;

/** Class for providing Timer functionality by using Thread.sleep to 
    measure the passed time.
*/    
public class ThreadTimer implements com.monica.interfaces.Timer, Runnable {

  private static final String NAME="ThreadTimer";
  private static final String VERSION="0.3";
  
  public String getTimerName() {
    return NAME;
  }
  
  public String getTimerVersion() {
    return VERSION;
  }        

  Vector children;
  Vector threads;
  EqualityVector IDs;

  int myID;
  int myInterval;
  boolean status;
  Thread myThread;
  TimerHandler myTimerHandler;

  /** Default constructor.
      @param TimerHandler tH: the timerHandler that has to be notified at timer events
  */      
  public ThreadTimer(TimerHandler tH) {
    threads=new Vector();
    children=new Vector();
    IDs=new EqualityVector(new IntegerEquality(), true, false, Integer.MAX_VALUE);
    myTimerHandler=tH;
  }

  private ThreadTimer(int id, int interv, TimerHandler tH) {
    myID=id;
    myInterval=interv;
    status=false;
    myTimerHandler=tH;
  }

  private ThreadTimer(TimerHandler tH, int ID) {
    myTimerHandler=tH;
    myID=ID;
    status=true;
  }

  private void setThread(Thread t) {
    myThread=t;
  }

  public synchronized boolean setTimer(int timerID, int interval) {
    if (IDs.firstIndex(new Integer(timerID), true)==-1) {
      ThreadTimer tt=new ThreadTimer(timerID, interval, myTimerHandler);
      Thread t=new Thread(tt);
      tt.setThread(t);

      IDs.add(new Integer(timerID));
      threads.addElement(t);
      children.addElement(tt);

      t.start();
      return true;
    }
    return false;
  }

  public synchronized boolean killTimer(int timerID) {
    int pos=IDs.firstIndex(new Integer(timerID), true);
    if (pos!=-1) {
      ((Thread)(threads.elementAt(pos))).interrupt();
      ((ThreadTimer)(children.elementAt(pos))).status=true;
      try {
        IDs.remove(pos);
      }
      catch (Exception e) {
      }
      threads.removeElementAt(pos);
      children.removeElementAt(pos);
      return true;
    }
    return false;
  }

  public void run() {
    if (status) {
      myTimerHandler.timerEvent(myID);
    }
    else {
      try {
        ThreadTimer tt;
        Thread t;
        while (!status) {
          myThread.sleep(myInterval);
          tt=new ThreadTimer(myTimerHandler, myID);
          t=new Thread(tt);
          t.start();
        }
      }
      catch (InterruptedException ie) {
      }
    }
  }

}