package com.monica.util.ccdm;

/**
 *  Reprsenets the result of the hash operation performed by 
 *  a hash algorithm on a String.
 */
public interface CCDHash {
   
    /**
     *  Returns the type of this hash.
     *  @return String: the type of this hash
     *  @exception CCDException: signaling if there was an error
     */
    public String getType() throws CCDException;
    
    /**
     *  Returns the hash value as a String.
     *  @return String: the hash value
     *  @exception CCDException: signaling if there was an error
     */
    public String getHashAsString() throws CCDException;
    
    /**
     *  Returns the hash value as an Object.
     *  @return Object: the hash value as an Object
     *  @exception CCDExceptipon: signaling if there was an error
     */
    public Object getHashAsObject() throws CCDException;    
    
    /**
     *  Returns the core value of the hash, e.g. that value
     *  that is according to the hash's defining standard the
     *  hash result.
     *  @return String: the hash core value
     *  @exception CCDException: signaling if there was an error
     */
    public String getHashCore() throws CCDException;
    
    /**
     *  Initalises this hash object with a String type raw hash format.
     *  Generally this raw hash format is the same as the result
     *  fromt he getHashAsString() method.
     *  @param String rawData: the raw hash data
     *  @exception CCDException: if the initialization fails
     */        
    public void createFromString(String rawData) throws CCDException;
    
    /**
     *  Initalises this hash object with a String type core hash format.
     *  Generally this core hash format is the same as the result
     *  fromt he getHashCore() method.
     *  @param String rawData: the core hash data
     *  @exception CCDException: if the initialization fails
     */        
    public void createFromCore(String coreData) throws CCDException;
}
