package com.monica.util.ccdm;

/**
 *  Represents an abstract hash algorithm that can perform the
 *  hash operation on Strings.
 */
public interface CCDHashAlgorithm {
   
    /**
     *  Returns the name of the hash algorithm.
     *  @return String: the name of the hash algorithm
     *  @exception CCDException: signaling if there was an error
     */
    public int getName() throws CCDException;
    
    /**
     *  Signals the hash algorithm what hash length (hash core value, in bits) is
     *  required upon usage. If the requested hash length is not
     *  available, a CCDException will be throws. Otherwise the next
     *  hash operation will return a hash object with the requested
     *  length in bits.
     *  @param int reqLen: the requested hash length in bits
     *  @exception CCDException: signaling if there was an error
     */
    public void requireHashLength(int reqLen) throws CCDException; 
    
    /**
     *  Returns the length of the hash core value in bits for the next hash operation.
     *  @return int: the length of the hash in bits for the next hash operation
     *  @exception CCDException: signaling if there was an error
     */
    public int getHashLength() throws CCDException;

    /**
     *  Returns the hash object that matches to the data String.
     *  @param String data: the String upon what the hash operation 
     *      will be done
     *  @exception CCDException: signaling if there was an error
     */
    public CCDHash hash(String data) throws CCDException;            
}
