package com.monica.util.ccdm;

/**
 *  Class for holding data for the IV (Initialization Vector) for
 *  a cipher algorithm.
 */
public interface CCDIV {
   
    /**
     *  Returns the length of the IV core value in number of bits.
     *  @return int: the length of the IV in bits
     *  @exception CCDException: singaling if there was an error
     */
    public int getLength() throws CCDException;
    
    /**
     *  Returns the IV as a String.
     *  @return String: the IV
     *  @exception CCDException: signalin if there was an error
     */
    public String getIVAsString() throws CCDException;

    /**
     *  Returns the IV as an Object.
     *  @return Object: the IV
     *  @exception CCDException: signalin if there was an error
     */
    public Object getIVAsObject() throws CCDException;
    
    /** Returns the core value of the IV, e.g. that part
     *  that will be used by a cipher algorithm for
     *  the cryptographic operations.
     *  @return String: the IV core
     *  @exception CCDException: signaling if there was an erro
     */
    public String getIVCore() throws CCDException;

    /**
     *  Initalises this IV object with a String type raw IV format.
     *  Generally this raw IV format is the same as the result
     *  from the getIVAsString() method.
     *  @param String rawData: the raw IV data
     *  @exception CCDException: if the initialization fails
     */        
    public void createFromString() throws CCDException;
    
    /**
     *  Initalises this IV object with a String type core IV format.
     *  Generally this core IV format is the same as the result
     *  from the getIVCore() method.
     *  @param String rawData: the core IV data
     *  @exception CCDException: if the initialization fails
     */        
    public void createFromCore() throws CCDException;
}
