package com.monica.util.ccdm;

/** Represents the result of the sign operation performed by a
 *  sign algorithm on a String.
 */
public interface CCDSignature {
   
    /** Returns the type of the signature.
     *  @return String: the type of the signature
     *  @exception CCDException: signaling if there was an error
     */
    public String getSignatureType() throws CCDException;
    
    /**
     *  Returns the signer of this signature. If not known
     *  an exception will be thrown.
     *  @return CCDOwner: the signer of this signature
     *  @exception CCDException: signaling if there was an error
     */
    public CCDOwner signedBy() throws CCDException;
    
    /**
     *  Returns the signature as a String.
     *  @return String: the signature as a String
     *  @exception CCDException: signaling if there was an error
     */
    public String getSignatureAsString() throws CCDException;
    
    /**
     *  Returns the signature as an Object.
     *  @return Object: the signature as an Object
     *  @exception CCDException: signaling if there was an error
     */
    public Object getSignatureAsObject() throws CCDException;
    
    /**
     *  Returns the core value of the signature, e.g. that value that
     *  has been defined as the result of the signing operation by
     *  defining standard of the signature algorithm.
     *  @return String: the signature core
     *  @exception CCDException: signaling if there was an error
     */
    public String getSignatureCore() throws CCDException;

    /**
     *  Initalises this signature object with a String type raw signature format.
     *  Generally this raw signature format is the same as the result
     *  from the getSignatureAsString() method.
     *  @param String rawData: the raw signature data
     *  @exception CCDException: if the initialization fails
     */        
    public void createFromString(String s) throws CCDException;
    
    /**
     *  Initalises this signature object with a String type core signature format.
     *  Generally this core signature format is the same as the result
     *  from the getSignatureCore() method.
     *  @param String rawData: the core signature data
     *  @param CCDOwner issuedBy: the issuer of the signature
     *  @exception CCDException: if the initialization fails
     */        
    public void createFromCore(String s, CCDOwner issuedBy) throws CCDException;
}
