package com.monica.util.ccdm;

/**
 *  Signature Algorithm Parameter Descriptor<br><br>
 *  Represents a structure for describing the functionality of 
 *  a signature algorithm.
 */
public interface CCDSignatureParamDesc {
   
    /**
     *  Returns the name of the this signature algorithm parameter
     *  descriptor.
     *  @return String: the name of this signature algorithm parameter
     *  descriptor
     *  @exception CCDException: signaling if there was an error
     */
    public String getName() throws CCDException;
    
    /**
     *  Returns the length of the signature (of the core value) in bits
     *  if this descriptor is active in the signature algorithm.
     *  @return int: the length of the signature core value in bits
     *  @exception CCDException: signaling if there was an error
     */
    public int getLength();
    
    /**
     *  Returns whether the signature algorithm can sign if this
     *  descriptor is active.
     *  @return boolean: whether the signature algorithm can sign
     *  @exception CCDException: signaling if there was an error
     */
    public boolean canSign() throws CCDException;
    
    /**
     *  Returns whether the signature algorithm can verify a 
     *  signature's authenticity if this descriptor is active.
     *  @return boolean: whether the signature algorithm can verify a
     *  signature's authenticity
     *  @exception CCDException: signalig if there was an error
     */
    public boolean canVerify() throws CCDException;
    
}
