package com.monica.util.ccdm.dss;

import com.monica.util.ccdm.*;
import com.monica.util.ccdm.dss.*;
import com.monica.util.ccdm.impl.*;

import java.util.*;
import java.math.*;

public class CCDDSSPublicKey implements CCDPublicKey {
    
    public static final String TYPE="DSSPublicKey";    
    
    private static final String NL=""+((char)13)+((char)10);        
    private static final String ERR_NI="Key not initialized.";
    
    private DSS_public myDSS_public;
    private DSS_common myDSS_common;
    private CCDOwner myOwner;
    private String myDesignation;
    
    public CCDDSSPublicKey() {
    }
    
    public CCDDSSPublicKey(DSS_common d_c, DSS_public d_p, CCDOwner o) throws CCDException {        
        if (d_p==null) {
            throw new CCDException("The DSS_public parameter may not be null.");
        }
        if (d_c==null) {
            throw new CCDException("The DSS_common parameter may not be null.");
        }
        myDSS_common=d_c;
        myDSS_public=d_p;
        myOwner=o;        
    }
    
    // -----------------------------------------------------------------
    // CCDPublicKey
    // -----------------------------------------------------------------
    
    public void setDesignation(String s) throws CCDException {
        myDesignation=s;
    }
    
    public String getDesignation() throws CCDException {
        return myDesignation;
    }
    
    public String getKeyType() throws CCDException {
        return TYPE;
    }
            
    public int getKeyLength() throws CCDException {
        if (myDSS_public==null) {
            throw new CCDException(ERR_NI);
        }        
        return myDSS_public.y.bitLength();
    }
    
    public Object getKeyAsObject() throws CCDException {
        if (myDSS_public==null) {
            throw new CCDException(ERR_NI);
        }
        Object a[]=new Object[4];
        a[0]=myDSS_common;
        a[1]=myDSS_public;
        a[2]=myOwner;
        a[3]=myDesignation;
        return a;
    }
    
    public String getKeyCore() throws CCDException {
        return _core();
    }
    
    public String getKeyAsString() throws CCDException {
        if (myDSS_public==null) {
            throw new CCDException(ERR_NI);
        }
        String ret=new String();
        ret+=_core();
        if (myOwner!=null) {
            if (myOwner.getCN()!=null) {
                ret+="cn="+myOwner.getCN()+NL;
            }
            if (myOwner.getDN()!=null) {
                ret+="dn="+myOwner.getDN()+NL;
            }
            if (myOwner.getUID()!=null) {
                ret+="uid="+myOwner.getUID()+NL;
            }
        }        
        if (myDesignation!=null) {
            ret+="designation="+myDesignation+NL;
        }
        return ret;
    }
    
    public CCDOwner getOwner() throws CCDException {
        if (myOwner!=null) {
            return myOwner;
        }
        else {
            throw new CCDException("Owner unknown.");
        }
    }
    
    public void createFromCore(String coreData) throws CCDException {
        createFromString(coreData);
    }
    
    public void createFromString(String rawData) throws CCDException {
        int oc, cc, pc;
        oc=0;
        cc=0;
        pc=0;
        
        DSS_public t_p=new DSS_public();
        DSS_common t_c=new DSS_common();
        CCDBasicOwner t_o=new CCDBasicOwner();
        String t_d=null;
        
        StringTokenizer tok=new StringTokenizer(rawData, NL);
        String[] sa=_parse(tok);
                
        while (sa!=null) {
            if (sa[0].equals("p")) {
                t_c.p=new BigInteger(sa[1],16);
                cc++;
            }
            else if (sa[0].equals("q")) {
                t_c.q=new BigInteger(sa[1],16);
                cc++;
            }
            else if (sa[0].equals("g")) {
                t_c.g=new BigInteger(sa[1],16);
                cc++;
            }
            else if (sa[0].equals("y")) {
                t_p.y=new BigInteger(sa[1],16);
                pc++;
            }
            else if (sa[0].equals("uid")) {
                t_o.setUID(sa[1]);
                oc++;
            }
            else if (sa[0].equals("cn")) {
                t_o.setCN(sa[1]);
                oc++;
            }
            else if (sa[0].equals("dn")) {
                t_o.setDN(sa[1]);
                oc++;
            }          
            else if (sa[0].equals("designation")) {
                t_d=sa[1];
            }
            sa=_parse(tok);
        }
        if ((pc==1) && (cc==3)) {
            myDSS_common=t_c;
            myDSS_public=t_p;
            myDesignation=t_d;
            if (oc>0) {
                myOwner=t_o;
            }            
        }
        else {
            throw new CCDException("Could not create from this String.");
        }        
    }
    
    // -----------------------------------------------------------------
    // private methods
    // -----------------------------------------------------------------
        
    private String[] _parse(StringTokenizer tok) {
        if (tok.hasMoreTokens()) {
            String s=tok.nextToken();
            int pos=s.indexOf("=");
            String[] sa=new String[2];
            sa[0]=s.substring(0,pos);
            sa[1]=s.substring(pos+1);
            return sa;            
        }
        return null;
    }

    private String _core() {
        String ret=new String();
        ret+="p="+myDSS_common.p.toString(16)+NL;
        ret+="q="+myDSS_common.q.toString(16)+NL;
        ret+="g="+myDSS_common.g.toString(16)+NL;
        ret+="y="+myDSS_public.y.toString(16)+NL;
        return ret;
    }                                
                            
}
