package com.monica.util.ccdm.dss;

import com.monica.util.ccdm.*;
import com.monica.util.ccdm.impl.*;

import java.util.*;
import java.math.*;

public class CCDDSSSignature implements CCDSignature {
    
    public static final String TYPE="DSSSignature";

    private static final String NL=""+((char)13)+((char)10);
        
    protected CCDOwner myOwner;
    protected DSS_signature mySignature;
    
    public CCDDSSSignature() {
    }
    
    public CCDDSSSignature(DSS_signature signature, CCDOwner owner) {
        myOwner=owner;
        mySignature=signature;
    }
        
    // -----------------------------------------------------------------
    // CCDSignature
    // -----------------------------------------------------------------
        
    public String getSignatureType() throws CCDException {
        return TYPE;
    }
    
    public CCDOwner signedBy() throws CCDException {
        if (myOwner!=null) {
            return myOwner;
        }
        throw new CCDException("Signature issuer not known.");
    }
    
    public Object getSignatureAsObject() throws CCDException {
        Object oa[]=new Object[2];
        oa[0]=mySignature;
        oa[1]=myOwner;
        return oa;
    }
    
    public String getSignatureAsString() throws CCDException {
        String ret=new String();
        ret+="r="+mySignature.r.toString(16)+NL;
        ret+="s="+mySignature.s.toString(16)+NL;
        if (myOwner!=null) {
            if (myOwner.getCN()!=null) {
                ret+="signedBy.cn="+myOwner.getCN()+NL;
            }
            if (myOwner.getDN()!=null) {
                ret+="signedBy.dn="+myOwner.getDN()+NL;
            }
            if (myOwner.getUID()!=null) {
                ret+="signedBy.uid="+myOwner.getUID()+NL;
            }
        }
        return ret;                
    }
    
    public String getSignatureCore() throws CCDException {
        String ret=new String();
        ret+="r="+mySignature.r.toString(16)+NL;
        ret+="s="+mySignature.s.toString(16)+NL;  
        return ret;
    }
    
    public void createFromCore(String s, CCDOwner issuedBy) throws CCDException {
        createFromString(s);
        if (issuedBy!=null) {
            myOwner=issuedBy;
        }
    }
            
    public void createFromString(String s) throws CCDException {
        int oc, sc;
        oc=0;
        sc=0;
        
        DSS_signature t_s=new DSS_signature();
        CCDBasicOwner t_o=new CCDBasicOwner();
        
        StringTokenizer tok=new StringTokenizer(s, NL);
        String[] sa=_parse(tok);
                
        while (sa!=null) {
            if (sa[0].equals("r")) {
                t_s.r=new BigInteger(sa[1],16);
                sc++;
            }
            else if (sa[0].equals("s")) {
                t_s.s=new BigInteger(sa[1],16);
                sc++;
            }
            else if (sa[0].equals("signedBy.uid")) {
                t_o.setUID(sa[1]);
                oc++;
            }
            else if (sa[0].equals("signedBy.cn")) {
                t_o.setCN(sa[1]);
                oc++;
            }
            else if (sa[0].equals("signedBy.dn")) {
                t_o.setDN(sa[1]);
                oc++;
            }      
            sa=_parse(tok);
        }
        if (sc==2) {
            mySignature=t_s;
            if (oc>0) {
                myOwner=t_o;
            }            
        }
        else {
            throw new CCDException("Could not create from this String.");
        }                
    }
    
    private String[] _parse(StringTokenizer tok) {
        if (tok.hasMoreTokens()) {
            String s=tok.nextToken();
            int pos=s.indexOf("=");
            String[] sa=new String[2];
            sa[0]=s.substring(0,pos);
            sa[1]=s.substring(pos+1);
            return sa;            
        }
        return null;
    }
    
        
}
