package com.monica.util.ccdm.dss;

import com.monica.util.ccdm.impl.*;
import com.monica.util.ccdm.*;

import java.math.*;
import java.util.*;

public class CCDSSHDSSSignature implements CCDSignature {
    
    public static final String TYPE="SSHDSSSignature";
    
    private static final String NL=""+((char)13)+((char)10);
            
    private String mySignature;
    private CCDOwner myOwner;
    
    public CCDSSHDSSSignature() {
    }
    
    public CCDSSHDSSSignature(String sig, CCDOwner signer) {
        mySignature=sig;
        myOwner=signer;
    }
    
    // -----------------------------------------------------------------
    // CCDSSHDSSSignature
    // -----------------------------------------------------------------
    
    public String getSignatureType() throws CCDException {
        return TYPE;
    }    
    
    public CCDOwner signedBy() throws CCDException {
        return myOwner;
    }
    
    public Object getSignatureAsObject() throws CCDException {
        Object oa[]=new Object[2];
        oa[0]=mySignature;
        oa[1]=myOwner;
        return oa;
    }
    
    public String getSignatureAsString() throws CCDException {
        String ret=new String();
        ret="sig="+mySignature;
        if (myOwner!=null) {
            if (myOwner.getCN()!=null) {
                ret+="signedBy.cn="+myOwner.getCN()+NL;
            }
            if (myOwner.getDN()!=null) {
                ret+="signedBy.dn="+myOwner.getDN()+NL;
            }
            if (myOwner.getUID()!=null) {
                ret+="signedBy.uid="+myOwner.getUID()+NL;
            }
        }
        return ret;                        
    }
    
    public String getSignatureCore() throws CCDException {
        return mySignature;
    }
    
    public void createFromCore(String s, CCDOwner issuedBy) throws CCDException {
        if (s.length()!=55) {
            throw new CCDException("Invalid SSDDSS Signature core.");
        }
        mySignature=s;
        if (issuedBy!=null) {
            myOwner=issuedBy;
        }
    }
                
    public void createFromString(String s) throws CCDException {
                
        int oc;
        oc=0;

        String t_s=null;
        CCDBasicOwner t_o=new CCDBasicOwner();
        
        StringTokenizer tok=new StringTokenizer(s, NL);
        String[] sa=_parse(tok);
                
        while (sa!=null) {
            if (sa[0].equals("sig")) {
                t_s=sa[1];
            }
            else if (sa[0].equals("signedBy.uid")) {
                t_o.setUID(sa[1]);
                oc++;
            }
            else if (sa[0].equals("signedBy.cn")) {
                t_o.setCN(sa[1]);
                oc++;
            }
            else if (sa[0].equals("signedBy.dn")) {
                t_o.setDN(sa[1]);
                oc++;
            }      
            sa=_parse(tok);
        }
        if (t_s!=null) {
            mySignature=t_s;
            if (oc>0) {
                myOwner=t_o;
            }            
        }
        else {
            throw new CCDException("Could not create from this String.");
        }                
    }    
            
    private String[] _parse(StringTokenizer tok) {
        if (tok.hasMoreTokens()) {
            String s=tok.nextToken();
            int pos=s.indexOf("=");
            String[] sa=new String[2];
            sa[0]=s.substring(0,pos);
            sa[1]=s.substring(pos+1);
            return sa;            
        }
        return null;
    }
    
    
}
