package com.monica.util.ccdm.dss;

import com.monica.interfaces.*;
import com.monica.util.configuration.*;
import com.monica.util.hash.*;
import com.monica.util.*;
import com.monica.util.ccdm.dss.*;
import com.monica.util.debug.*;
import com.monica.util.ccdm.impl.*;
import com.monica.util.ccdm.*;

import java.io.*;
import java.util.*;
import java.math.*;

/** Application for creating a DSS public key - private key pair. */
public class CreateDSSKey {
    
  private static final String DP="CreateDSSKey";  
  
  private static final String CONF_WHOAMI="whoami.conf";
  private static final String CONF_CREATEKEY="CCDM.CreateKey";

  /** Entry point for the VM. */
  public static void main(String args[]) {
    try {
        
      if (args.length!=3) {
          System.out.println("Usage: java CreateDSSKey loaderFile privOutFile pubOutFile");
          System.exit(-1);
      }
      
      Configuration c=ConfigurationLoader.load(args[0]);  
      
      FileOutputStream fos_priv=new FileOutputStream(args[1]);
      FileOutputStream fos_pub=new FileOutputStream(args[2]);
      
      CreateDSSKey a=new CreateDSSKey(c);
      DSS dss=a.start();       
      
      Configuration conf=new StringConfiguration("");
      dss.addPublicToConfiguration(conf);
      conf.save(fos_pub);
      fos_pub.close();
      
      conf=new StringConfiguration("");
      dss.addPrivateToConfiguration(conf);
      conf.save(fos_priv);
      fos_priv.close();
    }
    catch (Exception e) {
      System.out.println("Usage: SSH2CreateDSSKey confFile privFile pubFile");
      e.printStackTrace();
    }    
  }
  
  public static CCDDSSKeyPair create(Configuration topLevel, String designation) throws CCDException {
      try {
        CreateDSSKey a=new CreateDSSKey(topLevel);      
        DSS dss=a.start();
        CCDBasicOwner bo=new CCDBasicOwner(topLevel.getConfiguration(CONF_WHOAMI));
        CCDDSSKeyPair kp=new CCDDSSKeyPair(dss.getCommon(), dss.getPrivate(), dss.getPublic(), bo);        
        kp.setDesignation(designation);
        return kp;
      }
      catch (Exception e) {
          throw new CCDException(e);
      }
  }
  
  private Configuration c;
  
  private CreateDSSKey(Configuration c) throws Exception {
    this.c=c.getConfiguration(CONF_CREATEKEY);
  }    
  
  private DSS start() {
    try {
      int L=c.getInt("DSS.L");
      int probability=c.getInt("DSS.probability");
      SHA1 sha1=new SHA1();
      int n=(L-1)/160;
      int b=(L-1)-(n*160);
      DebugManager.logOut(DP, "n="+n);
      DebugManager.logOut(DP, "b="+b);
      byte seed[]=new byte[20];
      Random rnd=new Random(); 
      BigInteger U, t1, t2, q=null, p=null;
      BigInteger g2=BigInteger.ONE.shiftLeft(160);
      BigInteger b2=BigInteger.ONE.shiftLeft(b);
      int counter;
      int offset;
      BigInteger V[]=new BigInteger[n+1];
      BigInteger X, c, t3, W;
      boolean notFound=true;
      
      DebugManager.logOut(DP, "+++ Generating p and q...");
      
      int try1=0;
      int try2=0;
      
      while (notFound) {
        while (true) {
          try1++;  
          DebugManager.logOut(DP, "STEP 1 "+try1);  
          rnd.nextBytes(seed); // STEP 1          
	  t1=sha1.hashAsBigInteger(new String(seed));
	  t2=new BigInteger(IO.print(new String(seed)),16);
	  t2.add(BigInteger.ONE);
	  t2=t2.mod(g2);
	  t2=sha1.hashAsBigInteger(fromHex(t2.toString(16)));
//          DebugManager.logOut(DP, ""+t1.bitLength());
//          DebugManager.logOut(DP, ""+t2.bitLength());
          U=t1.xor(t2); // STEP 2
//          DebugManager.logOut(DP, ""+U.bitLength());
	  q=U.setBit(159);          
	  q=q.setBit(0); // STEP 3
//          DebugManager.logOut(DP, ""+q.bitLength());
//          DebugManager.logOut(DP, "q="+q);
//          factorize(q);
	  if (q.isProbablePrime(probability)) { // STEP 4
	    break; // STEP 5
	  }
        }	        
        DebugManager.logOut(DP, "STEP 1 cycle done");
	counter=0; offset=2; // STEP 6		        
	while (true) {            
          try2++;  
          DebugManager.logOut(DP, "STEP 7 /counter="+counter+"/ "+try2);
          for (int k=0; k<=n; k++) { // STEP 7
            t1=new BigInteger(IO.print(new String(seed)),16);
	    t2=new BigInteger(""+(offset+k));
            V[k]=sha1.hashAsBigInteger(fromHex(t1.add(t2).mod(g2).toString(16)));
          }
          t3=BigInteger.ONE;
          W=BigInteger.ZERO;
	  for (int k=0; k<=n; k++) { // STEP 8
            if (k!=n) {                
	      W=W.add(V[k].multiply(t3));
            }
            else {
              W=W.add((V[k].mod(b2)).multiply(t3));
            }
	    t3=t3.shiftLeft(160);
	  }
	  X=W.add(BigInteger.ONE.shiftLeft(L-1));
	  c=X.mod(q.add(q)); 
	  p=X.subtract(c.subtract(BigInteger.ONE)); // STEP 9
          t3=BigInteger.ONE.shiftLeft(L-1);
//          DebugManager.logOut(DP, "Comparing...");
          if (p.compareTo(t3)!=-1) {
//            DebugManager.logOut(DP, "Compared...");  
//            DebugManager.logOut(DP, "p="+p);
            if (p.isProbablePrime(probability)) {  
              notFound=false;
              break;
            }
          }
          counter++;
          offset+=n+1;
          if (counter>=4096) {
            break;
          }
        }
      }
      DebugManager.logOut(DP, "+++ Finished p and q.");
      DebugManager.logOut(DP, "L="+L);
      DebugManager.logOut(DP, "p="+p);
      DebugManager.logOut(DP, "q="+q);
      t1=BigInteger.ONE.shiftLeft(L-1);
      t2=BigInteger.ONE.shiftLeft(L);
      if ((p.compareTo(t1)==1) && (p.compareTo(t2)==-1) && (p.isProbablePrime(probability))) {
        DebugManager.logOut(DP, "p ok.");
      }
      t1=BigInteger.ONE.shiftLeft(159);
      t2=BigInteger.ONE.shiftLeft(169);
      if ((q.compareTo(t1)==1) && (q.compareTo(t2)==-1) && (q.isProbablePrime(probability)) && (p.subtract(BigInteger.ONE).remainder(q).equals(BigInteger.ZERO)) ) {
        DebugManager.logOut(DP, "q ok.");
      }
      
      DebugManager.logOut(DP, "+++ Generating g and h...");
      BigInteger e,h,g;
      while (true) {
        e=p.subtract(BigInteger.ONE).divide(q);
        h=new BigInteger(L-1,0,rnd);
        g=h.modPow(e,p);
        if (!g.equals(BigInteger.ONE)) {
          break;
        }
      }
      DebugManager.logOut(DP, "Finished g and h.");
      DebugManager.logOut(DP, "g="+g);
      DebugManager.logOut(DP, "h="+h);

      DebugManager.logOut(DP, "+++ Generating x,y");
      BigInteger x=new BigInteger(159,0,rnd);
      BigInteger y=g.modPow(x,p);
      DebugManager.logOut(DP, "Finished x,y.");
      DebugManager.logOut(DP, "x="+x);
      DebugManager.logOut(DP, "y="+y);
      
      DSS dss=new DSS();
      dss.setCommon(p,q,g);
      dss.setPrivate(x);
      dss.setPublic(y);
      
      return dss;
    }
    catch (Exception e) {
        DebugManager.logException(DP, e);
        return null;
    }    
  }
  
  private void factorize(BigInteger b) {
      BigInteger a=BigInteger.ONE.shiftLeft(1);
      BigInteger d=BigInteger.ONE.add(a);
      int i=0;
      while (true) {
          if (b.remainder(d).equals(BigInteger.ZERO)) {
              DebugManager.logOut(DP, "divider="+d);
              break;
          }
          i++;
          if ((i%10000)==0) {
              DebugManager.logOut(DP, "q="+b);
              DebugManager.logOut(DP, "d="+d);
              DebugManager.logOut(DP, "...");
          }
          d=d.add(a);
      }
  }
  
  private String fromHex(String f) {
      char s[]=f.toCharArray();
      char t[]=new char[s.length/2];
      int v;
      for (int i=0; i<t.length; i++) {          
          v=_fromHex(s[i*2]);
          t[i]=(char)v;
          t[i]<<=4;
          v=_fromHex(s[i*2+1]);
          t[i]|=(char)((v)&0x0f);
      }
      return new String(t);
  }
          
  private int _fromHex(int v) {
          if (v<65) {
              v-=48;
          }
          else {
              v-=87;
          }
          return v;
  }
    
}