package com.monica.util.ccdm.dss;

import com.monica.util.ccdm.dss.*;
import com.monica.interfaces.*;
import com.monica.util.configuration.*;
import com.monica.javacom.ssh2.*;
import com.monica.util.debug.*;

import java.math.*;
import java.util.*;

/** Class for providing DSS functionaliy according to FIPS 186-2 and formatting according to SSH2 Transport Layer Protocol.
 */

public class SSHDSS {
    
  private static final String DP="SSHDSS";  

  DSS dss;
  SSH2IO io;
  
  /** Create a new instance of this class. */
  public SSHDSS() {
      io=new SSH2IO();
  }
  
  /** Sets the DSS object which will be used for the operations.
   *  @param DSS dss: the DSS object to be used
   */
  public void setDSS(DSS dss) {
    this.dss=dss;
  }    
  
  /** Returns the DSS object which is used for the operations.
   *  @return DSS: the DSS object which is used
   */
  public DSS getDSS() {
      return dss;
  }

  /** Verifies an sshdss signature on a String. Verification is done via the set DSS object. 
   *  @param String toSign: the String, whose signature should be verified
   *  @param String signature: the sshdss signature
   *  @return boolean: whether the signature is correct
   */
  public boolean ssh2_verify(String toSign, String signature) {
      
      Vector dss_blob;
      String blob=null;
                  
      if (signature.length()>40) { // standard
          dss_blob=io.parsePayload(signature,SSH2Constants.KEXDH_REPLY_DSS_BLOB_MSG_DATA_DESC);
          blob=(String)(dss_blob.elementAt(SSH2Constants.DI_KEXDH_REPLY_DSS_BLOB_DATA));
      }
      else { // fuck what the heck is that
          blob=signature;
      }
      
      DSS_signature sig=new DSS_signature();
      sig.r=io.stringAsBigInteger(blob.substring(0,20));
      sig.s=io.stringAsBigInteger(blob.substring(20));
      
      return dss.verify(toSign, sig);
  }
  
  /** Signs a String and returns an sshdss signature. Signing is done via the set DSS object.
   *  @param String toSign: the String to be signed
   *  @return String: the sshdss signature
   */
  public String ssh2_sign(String toSign) {      
      String blob;
      DSS_signature sig;
      while (true) {
        sig=dss.sign(toSign);
        blob=io.wMpintString(sig.r)+io.wMpintString(sig.s);
        if (blob.length()==40) {
            break;
        }
      }
      String returnV;      
      returnV=io.wString("ssh-dss");
      returnV+=io.wString(blob);        
      return returnV;
  }
  
  /** Returns the common and public part of the set DSS object's active key according as specified in the SSH2 Transport Layer Protocol.
   *  @return String: the common and public part of the DSS object's active key
   */
  public String ssh2_pubKey() {
      String r;
      r=io.wString("ssh-dss");
      r+=io.wMpint(dss.getCommon().p);
      r+=io.wMpint(dss.getCommon().q);
      r+=io.wMpint(dss.getCommon().g);
      r+=io.wMpint(dss.getPublic().y);
      return r;
  }
  
  /** Initialises the own DSS object to represent the common and public DSS keys as  
   *  specified in the parameter String, which should be an SSH_KEXDH_REPLY message's K_S parameter
   *  @param String s: the K_S parameter 
   */
  public void ssh2_pubKey(String s) {      
      Vector data_dss;
      BigInteger p,q,g,y;
      String blob;
      data_dss=io.parsePayload(s,SSH2Constants.KEXDH_REPLY_SSH_DSS_MSG_DATA_DESC);      
      p=(BigInteger)(data_dss.elementAt(SSH2Constants.DI_KEXDH_REPLY_SSH_DSS_P));
      q=(BigInteger)(data_dss.elementAt(SSH2Constants.DI_KEXDH_REPLY_SSH_DSS_Q));
      g=(BigInteger)(data_dss.elementAt(SSH2Constants.DI_KEXDH_REPLY_SSH_DSS_G));
      y=(BigInteger)(data_dss.elementAt(SSH2Constants.DI_KEXDH_REPLY_SSH_DSS_Y));
      dss=new DSS();
      dss.setCommon(p,q,g);
      dss.setPublic(y);            
  }
  
}