package com.monica.util.ccdm.impl;

import com.monica.interfaces.*;
import com.monica.util.ccdm.*;
import com.monica.util.*;
import com.monica.util.debug.*;

import java.util.*;
import java.sql.*;

/**
 *  ccdm.conf
 *      databaseName
 *      databaseDriver
 *      databaseUsername
 *      databasePassword
 *
 *  SQL Tables
 *      ccdm_keys
 *          id int // optional
 *          owner_dn text // QHEX
 *          owner_cn text // QHEX
 *          owner_uid text // QHEX
 *          designation text // QHEX
 *          key_type text // QHEX
 *          key_class text // QNORMAL
 *          key_blob text // QHEX
 *          
 */

public class SQLCCDManager implements CCDManager {
    
    private static final String CONF_SQL_NAME="CCDM.SQL";
    private static final String CONF_REG_NAME="CCDM.Registration";
    
    private static final String NAME="SQLCCDManager";
    private static final String VERSION="0.1";
        
    private static final String ERR_SQL_NA="SQL Framework not available.";
    
    private static final String DP="SQLCCDManager";
        
    // sql
    
    private static final String TABLE_NAME="ccdm_keys";
    
    private Configuration configuration, mySQLConfig;
    private Statement myStatement;
    private Driver myDriver;
    private Connection myConnection;
    
    // ccdm sigAlg
    
    private Vector signatureAlgorithmNames;
    
    private Hashtable sa_keyNames;
    private Hashtable signatureNames;
    
    // ccdm sig
    
    private Vector signatureClasses;
    private Hashtable signatureName_Class;
    
    // registration
    
    private Configuration myRegConfig;
        
    // -----------------------------------------------------------------
    // Constructor
    // -----------------------------------------------------------------
        
    public SQLCCDManager() {
        signatureAlgorithmNames=new Vector();
        sa_keyNames=new Hashtable();
        signatureNames=new Hashtable();
        signatureClasses=new Vector();
        signatureName_Class=new Hashtable();
    }
    
    public void finalize() {
        closeConnections();
    }
    
    // -----------------------------------------------------------------
    // CCDManager
    // -----------------------------------------------------------------
    
    public CCDKey[] getKey(CCDOwner owner) throws CCDException { 
        try {
            SQLHelper sqlSelect=SQLHelper.select(TABLE_NAME, "*", null);        
            _addSelectOwner(sqlSelect, owner);                                
            return _createKeys(sqlSelect);
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {        
            throw new CCDException(e);
        }
    }
    
    public CCDKey[] getKey(CCDOwner owner, String keyType) throws CCDException {
        try {
            SQLHelper sqlSelect=SQLHelper.select(TABLE_NAME, "*", null);        
            _addSelectOwner(sqlSelect, owner);                                
            sqlSelect.selectAnd("key_type=", keyType, SQLHelper.T_QHEX);
            return _createKeys(sqlSelect);
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {        
            throw new CCDException(e);
        }        
    }
    
    public CCDKey[] getKey(CCDOwner owner, String keyType, String designation) throws CCDException {
        try {
            SQLHelper sqlSelect=SQLHelper.select(TABLE_NAME, "*", null);        
            _addSelectOwner(sqlSelect, owner);                                
            sqlSelect.selectAnd("key_type=", keyType, SQLHelper.T_QHEX);
            sqlSelect.selectAnd("designation=", designation, SQLHelper.T_QHEX);
            return _createKeys(sqlSelect);
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {        
            throw new CCDException(e);
        }        
    }
    
    public CCDKey[] getKey(String designation) throws CCDException {
        try {
            SQLHelper sqlSelect=SQLHelper.select(TABLE_NAME, "*", null);        
            sqlSelect.selectAnd("designation=", designation, SQLHelper.T_QHEX);
            return _createKeys(sqlSelect);
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {        
            throw new CCDException(e);
        }        
    }
    
    public void putKey(CCDKey key) throws CCDException {
        if (myStatement!=null) {                        
            
            try {
                
                // DELETING OLD KEYS
                // only if fully specified
                                
                CCDOwner dO=key.getOwner();
                if ((dO!=null) && ((dO.getDN()!=null) || (dO.getCN()!=null) || (dO.getUID()!=null)) && (key.getDesignation()!=null)) {
                    
                    SQLHelper deleteCmd=SQLHelper.delete(TABLE_NAME);                
                    
                    // CN
                    if (key.getOwner().getCN()!=null) {
                        deleteCmd.deleteAnd("owner_cn=", key.getOwner().getCN(), SQLHelper.T_QHEX);
                    }
            
                    // DN
                    if (key.getOwner().getDN()!=null) {
                        deleteCmd.deleteAnd("owner_dn=", key.getOwner().getDN(), SQLHelper.T_QHEX);
                    }
            
                    // UID
                    if (key.getOwner().getUID()!=null) {
                        deleteCmd.deleteAnd("owner_uid=", key.getOwner().getUID(), SQLHelper.T_QHEX);
                    }
                    
                    deleteCmd.deleteAnd("key_type=", key.getKeyType(), SQLHelper.T_QHEX);
                    deleteCmd.deleteAnd("key_class=", key.getClass().getName(), SQLHelper.T_QNORMAL);
                    deleteCmd.deleteAnd("designation=", key.getDesignation(), SQLHelper.T_QHEX);
                    
                    String cmd=deleteCmd.deleteFlush();
                    
                    int res=myStatement.executeUpdate(cmd);
                    
                    DebugManager.logOut(DP, "Deleted "+res+" keys before insertion.");
                    
                }
                
                // INSERTING KEY
            
                SQLHelper insertCmd=SQLHelper.insert(TABLE_NAME);
            
                // --- OWNER ---
            
                // CN
                if (key.getOwner().getCN()!=null) {
                    insertCmd.insertValue("owner_cn", key.getOwner().getCN(), SQLHelper.T_QHEX);
                }
            
                // DN
                if (key.getOwner().getDN()!=null) {
                    insertCmd.insertValue("owner_dn", key.getOwner().getDN(), SQLHelper.T_QHEX);
                }
            
                // UID
                if (key.getOwner().getUID()!=null) {
                    insertCmd.insertValue("owner_uid", key.getOwner().getUID(), SQLHelper.T_QHEX);
                }
            
                // --- KEY DATA --
            
                // Key Type
                insertCmd.insertValue("key_type", key.getKeyType(), SQLHelper.T_QHEX);
            
                // Key Class
                insertCmd.insertValue("key_class", key.getClass().getName(), SQLHelper.T_QNORMAL);
            
                // Key Blob
                insertCmd.insertValue("key_blob", key.getKeyAsString(), SQLHelper.T_QHEX);
                
                // designation
                if (key.getDesignation()!=null) {
                    insertCmd.insertValue("designation", key.getDesignation(), SQLHelper.T_QHEX);
                }
                            
                String sql=insertCmd.insertFlush();
                
                int sqlRes=myStatement.executeUpdate(sql);
                
                if (sqlRes!=1) {
                    throw new CCDException("Insertion into the SQL Table failed.");
                }
            
            }
            catch (CCDException e) {
                throw e;
            }
            catch (Exception e) {
                throw new CCDException(e.toString());
            }
        }
        else {
            throw new CCDException(ERR_SQL_NA);
        }
    }
                
    public void registerSignatureAlgorithm(String signatureAlgorithmName, String[] p_keyNames, String[] p_signatureNames) throws CCDException {
        
        if (signatureAlgorithmNames.indexOf(signatureAlgorithmName)!=-1) {
            throw new CCDException("Signature algorithm already registered: "+signatureAlgorithmName);
        }
        
        Object o;
        int i;
        
        for (i=0; i<p_keyNames.length; i++) {
            o=sa_keyNames.get(p_keyNames[i]);
            if (o!=null) {
                throw new CCDException("Signature key already registered: "+p_keyNames[i]);
            }
        }

        for (i=0; i<p_signatureNames.length; i++) {
            o=signatureNames.get(p_signatureNames[i]);
            if (o!=null) {
                throw new CCDException("Signature already registered: "+p_signatureNames[i]);
            }
        }
                
        signatureAlgorithmNames.addElement(signatureAlgorithmName);
        
        for (i=0; i<p_keyNames.length; i++) {
            sa_keyNames.put(p_keyNames[i], signatureAlgorithmName);
        }

        for (i=0; i<p_signatureNames.length; i++) {
            signatureNames.put(p_signatureNames[i], signatureAlgorithmName);
        }
        
        DebugManager.logOut(DP, "Registered: "+signatureAlgorithmName);
    }
    
    public CCDSignatureAlgorithm createSignatureAlgorithm(CCDKey key) throws CCDException {
        try {
            Object o=sa_keyNames.get(key.getKeyType());
            if (o==null) {
                throw new CCDException("Key type not registered: "+key.getKeyType());
            }
            String sigAlgClass=(String)o;
            CCDSignatureAlgorithm sigAlg=(CCDSignatureAlgorithm)(Class.forName(sigAlgClass).newInstance());
            sigAlg.init(key);
            return sigAlg;
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {
            throw new CCDException(e.toString());
        }
    }
    
    public CCDSignatureAlgorithm createSignatureAlgorithm(CCDSignature signature) throws CCDException {
        try {
            Object o=signatureNames.get(signature.getSignatureType());
            if (o==null) {
                throw new CCDException("Signature type not registered: "+signature.getSignatureType());
            }
            String sigAlgClass=(String)o;
            CCDSignatureAlgorithm sigAlg=(CCDSignatureAlgorithm)(Class.forName(sigAlgClass).newInstance());
            return sigAlg;        
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {
            throw new CCDException(e.toString());
        }
    }
    
    public void registerSignature(String signatureClass) throws CCDException {
        
        if (signatureClasses.indexOf(signatureClass)!=-1) {
            throw new CCDException("Signature already registerd: "+signatureClass);
        }
        
        try {        
            CCDSignature sig=(CCDSignature)(Class.forName(signatureClass).newInstance());
            String type=sig.getSignatureType();
            signatureName_Class.put(type, signatureClass);
            
            DebugManager.logOut(DP, "Registered: "+signatureClass);
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {
            throw new CCDException(e.toString());
        }
    }
    
    public CCDSignature createSignature(String type, String core, CCDOwner issuer) throws CCDException {
        Object o=signatureName_Class.get(type);
        if (o==null) {
            throw new CCDException("Signature type not registerd: "+type);
        }
        String className=(String)o;
        try {
            CCDSignature sig=(CCDSignature)(Class.forName(className).newInstance());
            sig.createFromCore(core, issuer);
            return sig;
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {
            throw new CCDException(e.toString());
        }
    }
    

    // -----------------------------------------------------------------
    // Private functionality methods
    // -----------------------------------------------------------------
        
    private void _addSelectOwner(SQLHelper sqlSelect, CCDOwner owner) throws Exception {
        String dn, cn, uid;
        
        dn=owner.getDN();
        cn=owner.getCN();
        uid=owner.getUID();
        
        if ((dn==null) && (cn==null) && (uid==null)) {
            throw new CCDException("Both CN, CN and UID are empty. Unable to perform search in the key database.");
        }
        
        // DN
        if (owner.getDN()!=null) {
            sqlSelect.selectAnd("owner_dn=", owner.getDN(), SQLHelper.T_QHEX);
        }

        // CN
        if (owner.getCN()!=null) {
            sqlSelect.selectAnd("owner_cn=", owner.getCN(), SQLHelper.T_QHEX);
        }

        // UID
        if (owner.getUID()!=null) {
            sqlSelect.selectAnd("owner_uid=", owner.getUID(), SQLHelper.T_QHEX);
        }            
    }
    
    private CCDKey[] _createKeys(SQLHelper sqlSelect) throws CCDException {
        try {            
            ResultSet rs=myStatement.executeQuery(sqlSelect.selectFlush());
            int numLines=SQLHelper.numLines(rs);
            if (numLines==0) {
                return null;
            }
            CCDKey[] ret=new CCDKey[numLines];
            rs.first();
            String className, keyBlob;
            for (int i=0; i<numLines; i++) {
                className=sqlSelect.selectGet(rs.getString("key_class"), SQLHelper.T_QNORMAL);
                keyBlob=sqlSelect.selectGet(rs.getString("key_blob"), SQLHelper.T_QHEX);
                ret[i]=(CCDKey)(Class.forName(className).newInstance());
                ret[i].createFromString(keyBlob);
                rs.next();
            }
            return ret;
        }
        catch (CCDException e) {
            throw e;
        }
        catch (Exception e) {
            throw new CCDException(e.toString());
        }
    }
    
    // -----------------------------------------------------------------
    // Configureable
    // -----------------------------------------------------------------

    public void setConfiguration(Configuration configuration) {
        this.configuration=configuration;
        _rereadConfig();
    }
                
    public Configuration getConfiguration() {
        return configuration;
    }
    
    // -----------------------------------------------------------------
    // Private methods
    // -----------------------------------------------------------------
    
    private void _rereadConfig() {
        try {
            mySQLConfig=configuration.getConfiguration(CONF_SQL_NAME);
            
            String dbPath=mySQLConfig.getString("databasePath");
            String dbDriver=mySQLConfig.getString("databaseDriver");
            String dbUser=mySQLConfig.getString("databaseUsername");
            String dbPassword=mySQLConfig.getString("databasePassword");
            
            DebugManager.logOut(DP, "databasePath="+dbPath);
            DebugManager.logOut(DP, "databaseDriver="+dbDriver);
            DebugManager.logOut(DP, "databaseUser="+dbUser);
            DebugManager.logOut(DP, "databasePassword="+dbPassword);
            
            myDriver=(Driver)(Class.forName(dbDriver).newInstance());                        
            myConnection=DriverManager.getConnection(dbPath, dbUser, dbPassword);
            myStatement=myConnection.createStatement();            
            
            _doReg();
        }
        catch (Exception e) {
            DebugManager.logException(DP, e);
        }
    }                
    
    private void _doReg() {
        try {
            myRegConfig=configuration.getConfiguration(CONF_REG_NAME);
            
            String regList=myRegConfig.getString("reg"); // root item
            
            StringTokenizer tok=new StringTokenizer(regList, ",");
            
            String regItem;
            
            String regItem_sigAlgClass;
            String regItem_key;
            String regItem_signature;
            String regItem_signatureClass;
            
            String[] sa1, sa2;
            
            while (tok.hasMoreTokens()) {
                regItem=tok.nextToken();
                
                regItem_sigAlgClass=myRegConfig.getString(regItem+"_sigAlgClass");
                regItem_key=myRegConfig.getString(regItem+"_key");
                regItem_signature=myRegConfig.getString(regItem+"_signature");
                regItem_signatureClass=myRegConfig.getString(regItem+"_signatureClass");
                
                sa1=_toArray(regItem_key);
                sa2=_toArray(regItem_signature);
                
                registerSignatureAlgorithm(regItem_sigAlgClass, sa1, sa2);
                registerSignature(regItem_signatureClass);
            }
        }
        catch (Exception e) {
            DebugManager.logOut(DP, "Exception during registration: "+e.toString());
        }
    }
        
    private String[] _toArray(String s) {
        StringTokenizer tok=new StringTokenizer(s, ",");
        int len=tok.countTokens();
        String res[]=new String[len];
        for (int i=0; i<len; i++) {
            res[i]=tok.nextToken();
        }
        return res;
    }

    // -----------------------------------------------------------------
    // Closeable
    // -----------------------------------------------------------------
    
    public String getCloseableName() {
        return NAME;
    }
    
    public String getCloseableVersion() {
        return VERSION;
    }
    
    public synchronized void closeConnections() {
        try {
            if (myStatement!=null) {
                myStatement.close();
                DebugManager.logOut(DP, "myStatement closed.");
            }
        }
        catch (Exception e) {
            DebugManager.logException(DP, e);
        }
        try {
            if (myConnection!=null) {
                myConnection.close();
                DebugManager.logOut(DP, "myConnection closed.");
            }
        }
        catch (Exception e) {
            DebugManager.logException(DP, e);
        }                    
    }
}
