package com.monica.util.ccdm.rsa;

import com.monica.interfaces.*;

import com.monica.util.debug.*;
import com.monica.util.configuration.*;
import com.monica.util.ccdm.*;
import com.monica.util.ccdm.impl.*;

import java.math.*;
import java.util.*;
import java.io.*;

/** 
 *  Creates an RSA key pair.
 */
 
public class CreateRSAKey {
    
    private static final String DP="CreateRSAKey";
    
    private static final String CONF_WHOAMI="whoami.conf";
    private static final String CONF_CREATEKEY="CCDM.CreateKey";
    
    public static void main(String[] args) {
        
        try {
            
            if (args.length<2) {
                System.out.println("Usage: java CreateRSAKey loaderFile outputFile [keyDesignation]");
                System.exit(-1);
            }
                        
            String loaderFile=args[0];
            String outFile=args[1];
            
            String designation=null;
            
            if (args.length>2) {
                designation=args[2];
            }
            
            Configuration topLevel=ConfigurationLoader.load(loaderFile);
            DebugManager.startUp(topLevel);
            
            CreateRSAKey crk=new CreateRSAKey(topLevel);
            Object[] oa=crk.create();
            
            CCDBasicOwner bo=new CCDBasicOwner(topLevel.getConfiguration(CONF_WHOAMI));
            
            CCDRSAKeyPair kp=new CCDRSAKeyPair(
                (BigInteger)oa[0],
                (BigInteger)oa[1],
                (BigInteger)oa[2],
                bo);
            kp.setDesignation(designation);
            
            FileOutputStream fos=new FileOutputStream(outFile);
            fos.write(kp.getKeyAsString().getBytes());
            fos.flush();
            fos.close();
        }
        catch (Exception e) {
            DebugManager.logException(DP, e);
        }
    }
    
    public static CCDRSAKeyPair create(Configuration topLevel, String designation) throws CCDException {
        try {
            CreateRSAKey crk=new CreateRSAKey(topLevel);
            Object[] oa=crk.create();
            
            CCDBasicOwner bo=new CCDBasicOwner(topLevel.getConfiguration(CONF_WHOAMI));
            
            CCDRSAKeyPair kp=new CCDRSAKeyPair(
                (BigInteger)oa[0],
                (BigInteger)oa[1],
                (BigInteger)oa[2],
                bo);
            kp.setDesignation(designation);
            return kp;
        }
        catch (Exception e) {
            throw new CCDException(e);
        }
    }
    
    private int keyLength;
    private int certainty;
    private int publicKey;
    private Random random;
    
    private CreateRSAKey(Configuration topLevel) throws Exception {
        Configuration myConfig=topLevel.getConfiguration(CONF_CREATEKEY);
        
        keyLength=myConfig.getInt("RSA.keyLength");
        if ((keyLength<16) || (keyLength>65536)) {
            throw new CCDException("RSA: keyLength must be between 16 and 65536");
        }
        
        certainty=myConfig.getInt("RSA.certainty");        
        if ((certainty<1) || (certainty>65536)) {
            throw new CCDException("RSA: certainty must be between 1 and 65536");
        }
        
        publicKey=myConfig.getInt("RSA.publicKey");
        if ((publicKey<3) || (publicKey%2==0)) {
            throw new CCDException("RSA: publicKey must be odd and greater than 2");
        }
        
        random=new Random();        
    }
    
    private Object[] create() throws Exception {
        
        BigInteger p = new BigInteger(keyLength, certainty, random);
        BigInteger q = new BigInteger(keyLength, certainty, random);
        BigInteger m = p.multiply(q);

        BigInteger p1 = p.subtract(BigInteger.valueOf(1));
        BigInteger q1 = q.subtract(BigInteger.valueOf(1));
        BigInteger fiem = p1.multiply(q1);

        BigInteger e = BigInteger.valueOf(publicKey);
        while (e.gcd(fiem).intValue() > 1) e = e.add(BigInteger.valueOf(2));

        BigInteger d = solveEquvality(fiem, e);
        
        Object oa[]=new Object[3];
        oa[0]=m;
        oa[1]=e;
        oa[2]=d;
        
        return oa;
    }	

    private BigInteger solveEquvality(BigInteger a0, BigInteger a1) {
		
        int i;
        int hossz=a0.bitLength()+1;
        BigInteger a[], q[], n[], e[];
        a=new BigInteger[hossz];
        q=new BigInteger[hossz];
        e=new BigInteger[hossz];

        a[0]=a0; a[1]=a1;
        q[0]=BigInteger.valueOf(0);
        e[0]=BigInteger.valueOf(0); e[1]=BigInteger.valueOf(0);
        i=1;
        do {
            q[i]=a[i-1].divide(a[i]);
            a[i+1]=a[i-1].remainder(a[i]);
        } while (a[++i].compareTo(BigInteger.valueOf(1)) != 0);

        int lepesszam=i+1;
        e[2]=BigInteger.valueOf(-1).multiply(q[1]); e[3]=q[1].multiply(q[2]).add(BigInteger.valueOf(1));
        for (i=4; i<lepesszam; i++) {
            e[i]=e[i-2].subtract(e[i-1].multiply(q[i-1]));
        }

	if (e[lepesszam-1].compareTo(BigInteger.valueOf(0)) > 0) {
            return e[lepesszam-1];
        }
	else {
            return e[lepesszam-1].add(a0);
	}
    }

}
