package com.monica.util.compression;

import com.monica.interfaces.*;
import com.monica.util.*;
import com.monica.util.debug.*;

import java.util.zip.*;

/** Zlib RFC 1950
 */

public class ZLibCompression implements CompressionAlgorithm {
  
  // -------------------------------------------------------
  // Constants
  // -------------------------------------------------------
    
  private static final String DP="ZLibCompression";  
  
  // common  
  private static final String NAME="ZLibCompression";
  private static final String VERSION="0.3";
    
  // -------------------------------------------------------
  // Variables
  // -------------------------------------------------------
    
  // common
  private Converter converter;
  
  private Inflater inf;
  private Deflater def;
  private com.monica.util.StringBuffer strBufInf, strBufDef;
  private byte d_b[];
  private int d_read;
  private Configuration configuration;
       
  // -------------------------------------------------------
  // Constructors
  // -------------------------------------------------------
 
  /** Default constructor for the class.
  */
  
  public ZLibCompression() {
    
    // commonly used variables initalization
    converter=new Converter();
    
    // DEFLATE
    strBufInf=new com.monica.util.StringBuffer(50000);    
    strBufDef=new com.monica.util.StringBuffer(50000);    
    inf=new Inflater();    
    def=new Deflater(9,false);    
//    def.setStrategy(Deflater.DEFLATED);
  }

  public String getCompressionAlgorithmName() {
    return NAME;
  }
    
  public String getCompressionAlgorithmVersion() {
    return VERSION;
  }
    
  /** Compresses the given String.
    * @param String s: the to be compressed String
    * @return String: the compressed String
    */
  public String compress(String data) {
    try {
      return inflate(data);
    }
    catch (Exception e) {
      System.err.println("Exception in ZLIB compress: "+e.toString());
      return null;
    }  
  }
    
  /** Decompresses the given String.
    * @param String s: the to be decompressed String
    * @return String: the decompressed String
    */
  public String decompress(String data) {    
    try {
      return deflate(data);
    }  
    catch (Exception e) {
      System.err.println("Exception in ZLIB decompress: "+e.toString());
      return null;
    }  
  }
  
  private String deflate(String data) throws Exception {        
    inf.setInput(converter.getBytes(data));
    d_read=100;
    while (d_read==100) {
      d_b=new byte[100];
      d_read=inf.inflate(d_b);
      strBufInf.addToBuffer(d_b, d_read);
    } 
    return strBufInf.emptyBuffer();    
  }  
  
  private String inflate(String data) throws Exception {
    def.setInput(converter.getBytes(data));
    def.finish();    
    d_read=100;
    while (d_read==100) {
      d_b=new byte[100];
      d_read=def.deflate(d_b);      
      strBufDef.addToBuffer(d_b, d_read);
    }          
    return strBufDef.emptyBuffer();    
    
  }
  
  public void setConfiguration(Configuration conf) {
    configuration=conf;
  }
  
  public Configuration getConfiguration() {
    return configuration;
  }        
    
  
}