package com.monica.util.configuration;

import com.monica.interfaces.*;
import com.monica.util.debug.*;

import java.util.*;
import java.io.*;

/** Class for implementing the complex loading of configurations.
 *  A configuration that can be loaded, has to be conform to
 *  the following rules:
 *  <ul>
 *    <li>there has to be a "load" key which contains the 
 *      comma separated list of the names of the stream that will be
 *      loaded to this configuration
 *    <li>each stream name has to appear in the configuration as key,
 *      and its value has to point to a filename, that will be loaded as
 *      the stream associated with the stream name
 * </ul>
 */

public class ConfigurationLoader {
    
    private static final String MAIN="load";
    private static final String DP="ConfigurationLoader";
   
    /** Loads a Configuration fromt he specified filename.
     *  @param String filename: the filename of the Configuration
     *  @return Configuration: the Configuration that has been loaded
     *  @exception Exception: if an error occures, an Exception will be thrown
     */
    public static Configuration load(String fileName) throws Exception {
        FileConfiguration fC=new FileConfiguration(fileName);
//        StringConfiguration sC=new StringConfiguration("");
        String main=fC.getString(MAIN);
        StringTokenizer tok=new StringTokenizer(main,",");
        FileInputStream fis;
        String name;
        while (tok.hasMoreTokens()) {
            name=tok.nextToken();
            fileName=fC.getString(name);
            fis=new FileInputStream(fileName);            
//            sC.setStream(name, fis);
            fC.setStream(name, fis);            
	    System.out.println("Loaded: "+fileName+" as "+name);
            DebugManager.logOut(DP, "Loaded: "+fileName+" as "+name);
        }
//        return sC;
        return fC;        
    }
    
}