package com.monica.util.debug;

import com.monica.interfaces.*;

import java.io.*;
import java.util.*;

/** Logs messages into files and eventually prints them as well to System.out and System.err as well. <br><br>
 *  Configureable values from the <b>debug.conf</b> Configuration:
 *  <ul>
 *      <li><b>FileDebug.fileName.out</b>: file name, where to write logOut messages
 *      <li><b>FileDebug.fileName.out</b>: file name where to write logErr messages
 *      <li><b>FileDebug.verbose</b>: true/false, whether to use System.out and System.err as well
 *      <li><b>FileDebug.postfixHash</b>: true/false, whether to append identity hash of the current object to the file names of the out/err files
 *  </ul>
 */
public class FileDebug implements DebugOutput {
    
    private Configuration topLevel, myConfig;   
    private FileOutputStream fosOut, fosErr;
    private boolean verbose;
    private boolean postfixHash;
    private boolean useErr;
    
    private static final String CONF_DEBUG="debug.conf";
    private static final byte[] NL={0x0d, 0x0a};
    
    private static final boolean CONF_DEF_USE_ERR=true;
    
    public void setConfiguration(Configuration configuration) {
        topLevel=configuration;
        try {
            myConfig=topLevel.getConfiguration(CONF_DEBUG);
            postfixHash=myConfig.getBoolean("FileDebug.postfixHash", false);
            
            String outName=myConfig.getString("FileDebug.fileName.out");
            String errName=myConfig.getString("FileDebug.fileName.err");
            
            useErr=myConfig.getBoolean("useErr", CONF_DEF_USE_ERR);
            
            if (postfixHash) {
                Random rnd=new Random();
                int i=rnd.nextInt();
                outName+="-"+i;
                errName+="-"+i;
            }
            
            fosOut=new FileOutputStream(outName);
            if (useErr) {
                fosErr=new FileOutputStream(errName);            
            }
            verbose=myConfig.getBoolean("FileDebug.verbose", true);
            
        }
        catch (Exception e) {
            fosOut=null;
            fosErr=null;
        }
    }
        
    public Configuration getConfiguration() {
        return topLevel;
    }
    
    public void logOut(String line) {
        try {
            if (fosOut!=null) {
                if (verbose) {
                    System.out.println(line);
                }
                fosOut.write(line.getBytes());
                fosOut.write(NL);
                fosOut.flush();
            }
        }
        catch (Exception e) {
        }
    }
        
    public void logErr(String line) {
        try {
            if (useErr) {
                if (fosErr!=null) {
                    if (verbose) {
                        System.err.println(line);
                    }
                    fosErr.write(line.getBytes());
                    fosErr.write(NL);
                    fosErr.flush();
                }  
            }
            else {
                logOut(line);
            }
        }
        catch (Exception e) {
        }        
    }
} 
