package com.monica.util.hash;

import com.monica.interfaces.HashAlgorithm;
import com.monica.util.Converter;
import com.monica.util.IO;
import com.monica.interfaces.Configuration;

import com.monica.util.debug.*;

import java.math.BigInteger;
import java.io.PrintStream;

/** Class for providing MD5 hash functionality as described in RFC-1321. 
    Currently this class is in beta version and should be used with care.
*/
public class MD5 implements HashAlgorithm {
    
  private static final String DP="MD5";  

  private static final String NAME="MD5";
  private static final String VERSION="0.3";

  private int X[];
  private int T[];  
  
  private Configuration configuration;

  /** Default constructor. */
  public MD5() {
    initT();
  }

  public String getHashAlgorithmName() {
    return NAME;
  }

  public String getHashAlgorithmVersion() {
    return VERSION;
  }

  public int getHashLength() {
    return 16;
  }


  public String hash(String s) {

    int i,j,k;
    String temp;

    // Message padding ------------------------------------------------------------------------ //

    String p_msg=new String(s);

    int p_l=p_msg.length();
    p_l*=8; // bits
    p_l+=65; // "1" + 64bit length

    int p_lr=(p_l/512); // number of rounds
    if ((p_lr*512)<p_l) p_lr++;
    int p_zp=(p_lr*512)-p_l; // number of zero bits to be added
    p_l+=p_zp; // total length of the padded message

    p_lr=(p_l/512); // number of rounds

    char p_p[]=new char[p_lr*64]; // padded message

    for (i=0; i<p_msg.length(); i++) p_p[i]=p_msg.charAt(i); // copying original
    p_p[p_msg.length()]=(char)0x0080; // "1" and 7 "0"

    p_zp=(p_zp-1)/8; // number of zero bytes to be added
    for (i=0; i<p_zp; i++) p_p[p_msg.length()+1+i]=(char)0;

    temp=IO.wUint64((long)p_msg.length()*8); // original length as 64 bit
    for (i=0; i<8; i++) p_p[p_p.length-i-1]=temp.charAt(i); // the final padded data

    // Computing the message digest ----------------------------------------------------------- //

    int A, B, C, D, AA, BB, CC, DD;
    X=new int[16];

    A=0x67452301;
    B=0xefcdab89;
    C=0x98badcfe;
    D=0x10325476;

    for (i=0; i<p_lr; i++) {

      // a //
      for (j=0; j<16; j++) {
        X[j]=0;
        for (k=3; k>=0; k--) {
          X[j]<<=8;
          X[j]|=(p_p[i*64+j*4+k]&0x00FF);
        }
      }

      AA=A;
      BB=B;
      CC=C;
      DD=D;

      // Round 1

      A=R1(A,B,C,D, 0, 7, 1);
      D=R1(D,A,B,C, 1,12, 2);
      C=R1(C,D,A,B, 2,17, 3);
      B=R1(B,C,D,A, 3,22, 4);
      A=R1(A,B,C,D, 4, 7, 5);
      D=R1(D,A,B,C, 5,12, 6);
      C=R1(C,D,A,B, 6,17, 7);
      B=R1(B,C,D,A, 7,22, 8);
      A=R1(A,B,C,D, 8, 7, 9);
      D=R1(D,A,B,C, 9,12,10);
      C=R1(C,D,A,B,10,17,11);
      B=R1(B,C,D,A,11,22,12);
      A=R1(A,B,C,D,12, 7,13);
      D=R1(D,A,B,C,13,12,14);
      C=R1(C,D,A,B,14,17,15);
      B=R1(B,C,D,A,15,22,16);

      // Round 2

      A=R2(A,B,C,D, 1, 5,17);
      D=R2(D,A,B,C, 6, 9,18);
      C=R2(C,D,A,B,11,14,19);
      B=R2(B,C,D,A, 0,20,20);
      A=R2(A,B,C,D, 5, 5,21);
      D=R2(D,A,B,C,10, 9,22);
      C=R2(C,D,A,B,15,14,23);
      B=R2(B,C,D,A, 4,20,24);
      A=R2(A,B,C,D, 9, 5,25);
      D=R2(D,A,B,C,14, 9,26);
      C=R2(C,D,A,B, 3,14,27);
      B=R2(B,C,D,A, 8,20,28);
      A=R2(A,B,C,D,13, 5,29);
      D=R2(D,A,B,C, 2, 9,30);
      C=R2(C,D,A,B, 7,14,31);
      B=R2(B,C,D,A,12,20,32);

      // Round 3

      A=R3(A,B,C,D, 5, 4,33);
      D=R3(D,A,B,C, 8,11,34);
      C=R3(C,D,A,B,11,16,35);
      B=R3(B,C,D,A,14,23,36);
      A=R3(A,B,C,D, 1, 4,37);
      D=R3(D,A,B,C, 4,11,38);
      C=R3(C,D,A,B, 7,16,39);
      B=R3(B,C,D,A,10,23,40);
      A=R3(A,B,C,D,13, 4,41);
      D=R3(D,A,B,C, 0,11,42);
      C=R3(C,D,A,B, 3,16,43);
      B=R3(B,C,D,A, 6,23,44);
      A=R3(A,B,C,D, 9, 4,45);
      D=R3(D,A,B,C,12,11,46);
      C=R3(C,D,A,B,15,16,47);
      B=R3(B,C,D,A, 2,23,48);

      // Round 4

      A=R4(A,B,C,D, 0, 6,49);
      D=R4(D,A,B,C, 7,10,50);
      C=R4(C,D,A,B,14,15,51);
      B=R4(B,C,D,A, 5,21,52);
      A=R4(A,B,C,D,12, 6,53);
      D=R4(D,A,B,C, 3,10,54);
      C=R4(C,D,A,B,10,15,55);
      B=R4(B,C,D,A, 1,21,56);
      A=R4(A,B,C,D, 8, 6,57);
      D=R4(D,A,B,C,15,10,58);
      C=R4(C,D,A,B, 6,15,59);
      B=R4(B,C,D,A,13,21,60);
      A=R4(A,B,C,D, 4, 6,61);
      D=R4(D,A,B,C,11,10,62);
      C=R4(C,D,A,B, 2,15,63);
      B=R4(B,C,D,A, 9,21,64);

      A+=AA;
      B+=BB;
      C+=CC;
      D+=DD;
    }

    String result=new String();
    result=result.concat(IO.wLowOrderUint32(A));
    result=result.concat(IO.wLowOrderUint32(B));
    result=result.concat(IO.wLowOrderUint32(C));
    result=result.concat(IO.wLowOrderUint32(D));

    return result;

  }

  public BigInteger hashAsBigInteger(String data) {
    return IO.stringAsBigInteger(hash(data));
  }

  private int F(int X, int Y, int Z) {
    return ((X & Y) | ((~X) & Z));
  }

  private int G(int X, int Y, int Z) {
    return ((X & Z) | (Y & (~Z)));
  }

  private int H(int X, int Y, int Z) {
    return (X ^ Y ^ Z);
  }

  private int I(int X, int Y, int Z) {
    return (Y ^ (X | (~Z)));
  }

  private int S(int n, int X) {
    return ((X << n) | (X >>> (32-n)));
  }

  private int R1(int a, int b, int c, int d, int k, int s, int i) {
    return b+S(s,(a + F(b,c,d) + X[k] + TA(i)));
  }

  private int R2(int a, int b, int c, int d, int k, int s, int i) {
    return b+S(s,(a + G(b,c,d) + X[k] + TA(i)));
  }

  private int R3(int a, int b, int c, int d, int k, int s, int i) {
    return b+S(s,(a + H(b,c,d) + X[k] + TA(i)));
  }

  private int R4(int a, int b, int c, int d, int k, int s, int i) {
    return b+S(s,(a + I(b,c,d) + X[k] + TA(i)));
  }

  private int TA(int i) {
    return T[i-1];
  }

  private void initT() {
    T=new int[64];
    T[ 0]=0xd76aa478;
    T[ 1]=0xe8c7b756;
    T[ 2]=0x242070db;
    T[ 3]=0xc1bdceee;
    T[ 4]=0xf57c0faf;
    T[ 5]=0x4787c62a;
    T[ 6]=0xa8304613;
    T[ 7]=0xfd469501;
    T[ 8]=0x698098d8;
    T[ 9]=0x8b44f7af;
    T[10]=0xffff5bb1;
    T[11]=0x895cd7be;
    T[12]=0x6b901122;
    T[13]=0xfd987193;
    T[14]=0xa679438e;
    T[15]=0x49b40821;

    T[16]=0xf61e2562;
    T[17]=0xc040b340;
    T[18]=0x265e5a51;
    T[19]=0xe9b6c7aa;
    T[20]=0xd62f105d;
    T[21]= 0x2441453;
    T[22]=0xd8a1e681;
    T[23]=0xe7d3fbc8;
    T[24]=0x21e1cde6;
    T[25]=0xc33707d6;
    T[26]=0xf4d50d87;
    T[27]=0x455a14ed;
    T[28]=0xa9e3e905;
    T[29]=0xfcefa3f8;
    T[30]=0x676f02d9;
    T[31]=0x8d2a4c8a;

    T[32]=0xfffa3942;
    T[33]=0x8771f681;
    T[34]=0x6d9d6122;
    T[35]=0xfde5380c;
    T[36]=0xa4beea44;
    T[37]=0x4bdecfa9;
    T[38]=0xf6bb4b60;
    T[39]=0xbebfbc70;
    T[40]=0x289b7ec6;
    T[41]=0xeaa127fa;
    T[42]=0xd4ef3085;
    T[43]= 0x4881d05;
    T[44]=0xd9d4d039;
    T[45]=0xe6db99e5;
    T[46]=0x1fa27cf8;
    T[47]=0xc4ac5665;

    T[48]=0xf4292244;
    T[49]=0x432aff97;
    T[50]=0xab9423a7;
    T[51]=0xfc93a039;
    T[52]=0x655b59c3;
    T[53]=0x8f0ccc92;
    T[54]=0xffeff47d;
    T[55]=0x85845dd1;
    T[56]=0x6fa87e4f;
    T[57]=0xfe2ce6e0;
    T[58]=0xa3014314;
    T[59]=0x4e0811a1;
    T[60]=0xf7537e82;
    T[61]=0xbd3af235;
    T[62]=0x2ad7d2bb;
    T[63]=0xeb86d391;
  }
  
  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
  }
  
  public Configuration getConfiguration() {
    return configuration;
  }        
}




