package com.monica.util.hash;

import com.monica.interfaces.HashAlgorithm;
import com.monica.util.Converter;
import com.monica.util.IO;
import com.monica.interfaces.Configuration;

import com.monica.util.debug.*;

import java.math.BigInteger;
import java.io.PrintStream;

/** Class for providing SHA1 functionality as described in FIPS-180-1 */
public class SHA1 implements HashAlgorithm {
    
  private static final String DP="SHA1";  

  private static final String NAME="SHA1";
  private static final String VERSION="0.3";

  private Configuration configuration;

  /** Default constructor. */
  public SHA1() {
  }

  public String getHashAlgorithmName() {
    return NAME;
  }

  public String getHashAlgorithmVersion() {
    return VERSION;
  }

  public int getHashLength() {
    return 20;
  }


  public String hash(String s) {

    int i,j,k;
    String temp;

    // Message padding ------------------------------------------------------------------------ //

    String p_msg=new String(s);

    int p_l=p_msg.length();
    
    int p_l_or=p_msg.length();
    
    p_l*=8; // bits
    p_l+=65; // "1" + 64bit length

    int p_lr=(p_l/512); // number of rounds
    if ((p_lr*512)<p_l) p_lr++;
    int p_zp=(p_lr*512)-p_l; // number of zero bits to be added
    p_l+=p_zp; // total length of the padded message

    p_lr=(p_l/512); // number of rounds

    char p_p[]=new char[p_lr*64]; // padded message
    
    char p_msg_ca[]=p_msg.toCharArray();

    for (i=0; i<p_l_or; i++) p_p[i]=p_msg_ca[i]; // copying original
    p_p[p_l_or]=(char)0x0080; // "1" and 8 "0"

    p_zp=(p_zp-1)/8; // number of zero bytes to be added
    for (i=0; i<p_zp; i++) p_p[p_l_or+1+i]=(char)0;

    temp=IO.wUint64((long)p_l_or*8); // original length as 64 bit
    for (i=0; i<8; i++) p_p[p_p.length-i-1]=temp.charAt(7-i); // the final padded data

    // Computing the message digest ----------------------------------------------------------- //

    int A, B, C, D, E, H0, H1, H2, H3, H4, W[], TEMP;
    W=new int[80];

    H0=0x67452301;
    H1=0xEFCDAB89;
    H2=0x98BADCFE;
    H3=0x10325476;
    H4=0xC3D2E1F0;

    for (i=0; i<p_lr; i++) {

      // a //
      for (j=0; j<16; j++) {
        W[j]=0;
        for (k=0; k<4; k++) {
          W[j]<<=8;
          W[j]|=(p_p[i*64+j*4+k]&0x00FF);
        }
      }

      // b //
      for (j=16; j<80; j++) {
        W[j]=S(1,(W[j-3] ^ W[j-8] ^ W[j-14] ^ W[j-16]));
      }

      // c //
      A=H0;
      B=H1;
      C=H2;
      D=H3;
      E=H4;

      // d //
      for (j=0; j<80; j++) {
        TEMP = S(5,A) + f(j,B,C,D) + E + W[j] + K(j);
        E=D;
        D=C;
        C=S(30,B);
        B=A;
        A=TEMP;
      }

      // e //
      H0+=A;
      H1+=B;
      H2+=C;
      H3+=D;
      H4+=E;

    }

    String result=new String();
    result=result.concat(IO.wUint32(H0));
    result=result.concat(IO.wUint32(H1));
    result=result.concat(IO.wUint32(H2));
    result=result.concat(IO.wUint32(H3));
    result=result.concat(IO.wUint32(H4));

    return result;

  }

  public BigInteger hashAsBigInteger(String data) {
    return IO.stringAsBigInteger(hash(data));
  }

  private int f(int t, int B, int C, int D) {
    if ((t>=0) && (t<=19)) {
      return ((B & C) | ((~ B) & D));
    }
    else if ((t>=20) && (t<=39)) {
      return (B ^ C ^ D);
    }
    else if ((t>=40) && (t<=59)) {
      return ((B & C) | (B & D) | (C & D));
    }
    else {
      return (B ^ C ^ D);
    }
  }

  private int K(int t) {
    if ((t>=0) && (t<=19)) {
      return (0x5a827999);
    }
    else if ((t>=20) && (t<=39)) {
      return (0x6ed9eba1);
    }
    else if ((t>=40) && (t<=59)) {
      return (0x8f1bbcdc);
    }
    else {
      return (0xca62c1d6);
    }
  }

  private int S(int n, int X) {
    return ((X << n) | (X >>> (32-n)));
  }
  
  public void setConfiguration(Configuration configuration) {
    this.configuration=configuration;
  }
  
  public Configuration getConfiguration() {
    return configuration;
  }        


}
